@@ -18,5 +18,10 @@
  */
 package org.apache.pinot.broker.api;
 
+import org.apache.pinot.spi.utils.CommonConstants;
+
 public abstract class RequesterIdentity {
+  public String getClientIp() {
+    return CommonConstants.UNKNOWN;
+  }
 }
