@@ -18,14 +18,14 @@
  */
 package org.apache.pinot.client;
 
+import java.util.Arrays;
 import java.util.List;
 import java.util.Properties;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.Future;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
+import java.util.concurrent.CompletableFuture;
 import javax.annotation.Nullable;
-import org.apache.pinot.sql.parsers.CalciteSqlCompiler;
+import org.apache.pinot.common.utils.request.RequestUtils;
+import org.apache.pinot.sql.parsers.CalciteSqlParser;
+import org.apache.pinot.sql.parsers.SqlNodeAndOptions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -37,24 +37,24 @@ public class Connection {
   public static final String FAIL_ON_EXCEPTIONS = "failOnExceptions";
   private static final Logger LOGGER = LoggerFactory.getLogger(Connection.class);
 
-  private final PinotClientTransport _transport;
+  private final PinotClientTransport<?> _transport;
   private final BrokerSelector _brokerSelector;
   private final boolean _failOnExceptions;
 
-  Connection(List<String> brokerList, PinotClientTransport transport) {
+  Connection(List<String> brokerList, PinotClientTransport<?> transport) {
     this(new Properties(), new SimpleBrokerSelector(brokerList), transport);
   }
 
-  Connection(Properties properties, List<String> brokerList, PinotClientTransport transport) {
+  Connection(Properties properties, List<String> brokerList, PinotClientTransport<?> transport) {
     this(properties, new SimpleBrokerSelector(brokerList), transport);
     LOGGER.info("Created connection to broker list {}", brokerList);
   }
 
-  Connection(BrokerSelector brokerSelector, PinotClientTransport transport) {
+  Connection(BrokerSelector brokerSelector, PinotClientTransport<?> transport) {
     this(new Properties(), brokerSelector, transport);
   }
 
-  Connection(Properties properties, BrokerSelector brokerSelector, PinotClientTransport transport) {
+  Connection(Properties properties, BrokerSelector brokerSelector, PinotClientTransport<?> transport) {
     _brokerSelector = brokerSelector;
     _transport = transport;
 
@@ -116,10 +116,10 @@ public class Connection {
    */
   public ResultSetGroup execute(@Nullable String tableName, String query)
       throws PinotClientException {
-    tableName = tableName == null ? resolveTableName(query) : tableName;
-    String brokerHostPort = _brokerSelector.selectBroker(tableName);
+    String[] tableNames = (tableName == null) ? resolveTableName(query) : new String[]{tableName};
+    String brokerHostPort = _brokerSelector.selectBroker(tableNames);
     if (brokerHostPort == null) {
-      throw new PinotClientException("Could not find broker to query for table: " + tableName);
+      throw new PinotClientException("Could not find broker to query for table(s): " + Arrays.asList(tableNames));
     }
     BrokerResponse response = _transport.executeQuery(brokerHostPort, query);
     if (response.hasExceptions() && _failOnExceptions) {
@@ -148,7 +148,7 @@ public class Connection {
    * @return A future containing the result of the query
    * @throws PinotClientException If an exception occurs while processing the query
    */
-  public Future<ResultSetGroup> executeAsync(String query)
+  public CompletableFuture<ResultSetGroup> executeAsync(String query)
       throws PinotClientException {
     return executeAsync(null, query);
   }
@@ -161,7 +161,7 @@ public class Connection {
    * @throws PinotClientException If an exception occurs while processing the query
    */
   @Deprecated
-  public Future<ResultSetGroup> executeAsync(Request request)
+  public CompletableFuture<ResultSetGroup> executeAsync(Request request)
       throws PinotClientException {
     return executeAsync(null, request.getQuery());
   }
@@ -173,24 +173,31 @@ public class Connection {
    * @return A future containing the result of the query
    * @throws PinotClientException If an exception occurs while processing the query
    */
-  public Future<ResultSetGroup> executeAsync(@Nullable String tableName, String query)
+  public CompletableFuture<ResultSetGroup> executeAsync(@Nullable String tableName, String query)
       throws PinotClientException {
-    tableName = tableName == null ? resolveTableName(query) : tableName;
-    String brokerHostPort = _brokerSelector.selectBroker(tableName);
+    String[] tableNames = (tableName == null) ? resolveTableName(query) : new String[]{tableName};
+    String brokerHostPort = _brokerSelector.selectBroker(tableNames);
     if (brokerHostPort == null) {
       throw new PinotClientException("Could not find broker to query for statement: " + query);
     }
-    return new ResultSetGroupFuture(_transport.executeQueryAsync(brokerHostPort, query));
+    return _transport.executeQueryAsync(brokerHostPort, query).thenApply(ResultSetGroup::new);
   }
 
+  /**
+   * Returns the name of all the tables used in a sql query.
+   *
+   * @return name of all the tables used in a sql query.
+   */
   @Nullable
-  private static String resolveTableName(String query) {
+  private static String[] resolveTableName(String query) {
     try {
-      return CalciteSqlCompiler.compileToBrokerRequest(query).querySource.tableName;
+      SqlNodeAndOptions sqlNodeAndOptions = CalciteSqlParser.compileToSqlNodeAndOptions(query);
+      return RequestUtils.getTableNames(CalciteSqlParser.compileSqlNodeToPinotQuery(sqlNodeAndOptions.getSqlNode()))
+          .toArray(new String[0]);
     } catch (Exception e) {
-      LOGGER.error("Cannot parse table name from query: {}", query, e);
-      return null;
+      LOGGER.error("Cannot parse table name from query: {}. Fallback to broker selector default.", query, e);
     }
+    return null;
   }
 
   /**
@@ -213,43 +220,13 @@ public class Connection {
     _brokerSelector.close();
   }
 
-  private static class ResultSetGroupFuture implements Future<ResultSetGroup> {
-    private final Future<BrokerResponse> _responseFuture;
-
-    public ResultSetGroupFuture(Future<BrokerResponse> responseFuture) {
-      _responseFuture = responseFuture;
-    }
-
-    @Override
-    public boolean cancel(boolean mayInterruptIfRunning) {
-      return _responseFuture.cancel(mayInterruptIfRunning);
-    }
-
-    @Override
-    public boolean isCancelled() {
-      return _responseFuture.isCancelled();
-    }
-
-    @Override
-    public boolean isDone() {
-      return _responseFuture.isDone();
-    }
-
-    @Override
-    public ResultSetGroup get()
-        throws InterruptedException, ExecutionException {
-      try {
-        return get(60000L, TimeUnit.MILLISECONDS);
-      } catch (TimeoutException e) {
-        throw new ExecutionException(e);
-      }
-    }
-
-    @Override
-    public ResultSetGroup get(long timeout, TimeUnit unit)
-        throws InterruptedException, ExecutionException, TimeoutException {
-      BrokerResponse response = _responseFuture.get(timeout, unit);
-      return new ResultSetGroup(response);
-    }
+  /**
+   * Provides access to the underlying transport mechanism for this connection.
+   * There may be client metrics useful for monitoring and other observability goals.
+   *
+   * @return pinot client transport.
+   */
+  public PinotClientTransport<?> getTransport() {
+    return _transport;
   }
 }
