@@ -71,7 +71,7 @@ public class DriverUtils {
   public static void handleAuth(Properties info, Map<String, String> headers)
       throws SQLException {
 
-    if (info.contains(USER_PROPERTY) && !headers.containsKey(AUTH_HEADER)) {
+    if (info.containsKey(USER_PROPERTY) && !headers.containsKey(AUTH_HEADER)) {
       String username = info.getProperty(USER_PROPERTY);
       String password = info.getProperty(PASSWORD_PROPERTY, "");
       if (StringUtils.isAnyEmpty(username, password)) {
@@ -217,4 +217,38 @@ public class DriverUtils {
     Matcher matcher = pattern.matcher(query);
     return matcher.find();
   }
+
+  public static String enableQueryOptions(String sql, Map<String, Object> options) {
+    StringBuilder optionsBuilder = new StringBuilder();
+    for (Map.Entry<String, Object> optionEntry: options.entrySet()) {
+      if (!sql.contains(optionEntry.getKey())) {
+        optionsBuilder.append(DriverUtils.createSetQueryOptionString(optionEntry.getKey(), optionEntry.getValue()));
+      }
+    }
+    optionsBuilder.append(sql);
+    return optionsBuilder.toString();
+  }
+
+  public static String createSetQueryOptionString(String optionKey, Object optionValue) {
+    StringBuilder optionBuilder = new StringBuilder();
+    optionBuilder.append("SET ").append(optionKey);
+
+    if (optionValue != null) {
+      optionBuilder.append('=');
+
+      if (optionValue instanceof Boolean) {
+        optionBuilder.append(((Boolean) optionValue).booleanValue());
+      } else if (optionValue instanceof Integer || optionValue instanceof Long) {
+        optionBuilder.append(((Number) optionValue).longValue());
+      } else if (optionValue instanceof Float || optionValue instanceof Double) {
+        optionBuilder.append(((Number) optionValue).doubleValue());
+      } else {
+        throw new IllegalArgumentException(
+          "Option Type " + optionValue.getClass().getSimpleName() + " is not supported.");
+      }
+    }
+
+    optionBuilder.append(";\n");
+    return optionBuilder.toString();
+  }
 }
