@@ -22,15 +22,18 @@ import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonIgnore;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.fasterxml.jackson.annotation.JsonPropertyOrder;
-import java.io.ByteArrayInputStream;
+import it.unimi.dsi.fastutil.doubles.DoubleArrayList;
+import it.unimi.dsi.fastutil.longs.LongArrayList;
 import java.io.ByteArrayOutputStream;
-import java.io.DataInputStream;
 import java.io.DataOutputStream;
 import java.io.IOException;
 import java.io.Serializable;
+import java.math.BigDecimal;
+import java.nio.ByteBuffer;
 import java.sql.Timestamp;
 import java.util.Arrays;
 import java.util.EnumSet;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.utils.ByteArray;
 import org.apache.pinot.spi.utils.BytesUtils;
@@ -48,6 +51,12 @@ public class DataSchema {
   private final ColumnDataType[] _columnDataTypes;
   private ColumnDataType[] _storedColumnDataTypes;
 
+  /** Used by both Broker and Server to generate results for EXPLAIN PLAN queries. */
+  public static final DataSchema EXPLAIN_RESULT_SCHEMA =
+      new DataSchema(new String[]{"Operator", "Operator_Id", "Parent_Id"}, new ColumnDataType[]{
+          ColumnDataType.STRING, ColumnDataType.INT, ColumnDataType.INT
+      });
+
   @JsonCreator
   public DataSchema(@JsonProperty("columnNames") String[] columnNames,
       @JsonProperty("columnDataTypes") ColumnDataType[] columnDataTypes) {
@@ -75,72 +84,21 @@ public class DataSchema {
     return _columnDataTypes;
   }
 
+  /**
+   * Lazy compute the _storeColumnDataTypes field.
+   */
   @JsonIgnore
   public ColumnDataType[] getStoredColumnDataTypes() {
-    if (_storedColumnDataTypes == null) {
+    ColumnDataType[] storedColumnDataTypes = _storedColumnDataTypes;
+    if (storedColumnDataTypes == null) {
       int numColumns = _columnDataTypes.length;
-      _storedColumnDataTypes = new ColumnDataType[numColumns];
+      storedColumnDataTypes = new ColumnDataType[numColumns];
       for (int i = 0; i < numColumns; i++) {
-        _storedColumnDataTypes[i] = _columnDataTypes[i].getStoredType();
-      }
-    }
-    return _storedColumnDataTypes;
-  }
-
-  /**
-   * Returns whether the given data schema is type compatible with this one.
-   * <ul>
-   *   <li>All numbers are type compatible with each other</li>
-   *   <li>Numbers are not type compatible with string</li>
-   *   <li>Non-array types are not type compatible with array types</li>
-   * </ul>
-   *
-   * @param anotherDataSchema Data schema to compare with
-   * @return Whether the two data schemas are type compatible
-   */
-  public boolean isTypeCompatibleWith(DataSchema anotherDataSchema) {
-    if (!Arrays.equals(_columnNames, anotherDataSchema._columnNames)) {
-      return false;
-    }
-    int numColumns = _columnNames.length;
-    for (int i = 0; i < numColumns; i++) {
-      if (!_columnDataTypes[i].isCompatible(anotherDataSchema._columnDataTypes[i])) {
-        return false;
-      }
-    }
-    return true;
-  }
-
-  /**
-   * Upgrade the current data schema to cover the column data types in the given data schema.
-   * <p>Data type <code>LONG</code> can cover <code>INT</code> and <code>LONG</code>.
-   * <p>Data type <code>DOUBLE</code> can cover all numbers, but with potential precision loss when use it to cover
-   * <code>LONG</code>.
-   * <p>NOTE: The given data schema should be type compatible with this one.
-   *
-   * @param anotherDataSchema Data schema to cover
-   */
-  public void upgradeToCover(DataSchema anotherDataSchema) {
-    int numColumns = _columnDataTypes.length;
-    for (int i = 0; i < numColumns; i++) {
-      ColumnDataType thisColumnDataType = _columnDataTypes[i];
-      ColumnDataType thatColumnDataType = anotherDataSchema._columnDataTypes[i];
-      if (thisColumnDataType != thatColumnDataType) {
-        if (thisColumnDataType.isArray()) {
-          if (thisColumnDataType.isWholeNumberArray() && thatColumnDataType.isWholeNumberArray()) {
-            _columnDataTypes[i] = ColumnDataType.LONG_ARRAY;
-          } else {
-            _columnDataTypes[i] = ColumnDataType.DOUBLE_ARRAY;
-          }
-        } else {
-          if (thisColumnDataType.isWholeNumber() && thatColumnDataType.isWholeNumber()) {
-            _columnDataTypes[i] = ColumnDataType.LONG;
-          } else {
-            _columnDataTypes[i] = ColumnDataType.DOUBLE;
-          }
-        }
+        storedColumnDataTypes[i] = _columnDataTypes[i].getStoredType();
       }
+      _storedColumnDataTypes = storedColumnDataTypes;
     }
+    return storedColumnDataTypes;
   }
 
   public byte[] toBytes()
@@ -170,35 +128,29 @@ public class DataSchema {
     return byteArrayOutputStream.toByteArray();
   }
 
-  public static DataSchema fromBytes(byte[] buffer)
+  /**
+   * This method use relative operations on the ByteBuffer and expects the buffer's position to be set correctly.
+   */
+  public static DataSchema fromBytes(ByteBuffer buffer)
       throws IOException {
-    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(buffer);
-    DataInputStream dataInputStream = new DataInputStream(byteArrayInputStream);
-
     // Read the number of columns.
-    int numColumns = dataInputStream.readInt();
+    int numColumns = buffer.getInt();
     String[] columnNames = new String[numColumns];
     ColumnDataType[] columnDataTypes = new ColumnDataType[numColumns];
-
     // Read the column names.
-    int readLength;
     for (int i = 0; i < numColumns; i++) {
-      int length = dataInputStream.readInt();
+      int length = buffer.getInt();
       byte[] bytes = new byte[length];
-      readLength = dataInputStream.read(bytes);
-      assert readLength == length;
+      buffer.get(bytes);
       columnNames[i] = new String(bytes, UTF_8);
     }
-
     // Read the column types.
     for (int i = 0; i < numColumns; i++) {
-      int length = dataInputStream.readInt();
+      int length = buffer.getInt();
       byte[] bytes = new byte[length];
-      readLength = dataInputStream.read(bytes);
-      assert readLength == length;
+      buffer.get(bytes);
       columnDataTypes[i] = ColumnDataType.valueOf(new String(bytes, UTF_8));
     }
-
     return new DataSchema(columnNames, columnDataTypes);
   }
 
@@ -227,8 +179,8 @@ public class DataSchema {
     }
     if (anObject instanceof DataSchema) {
       DataSchema anotherDataSchema = (DataSchema) anObject;
-      return Arrays.equals(_columnNames, anotherDataSchema._columnNames) && Arrays
-          .equals(_columnDataTypes, anotherDataSchema._columnDataTypes);
+      return Arrays.equals(_columnNames, anotherDataSchema._columnNames) && Arrays.equals(_columnDataTypes,
+          anotherDataSchema._columnDataTypes);
     }
     return false;
   }
@@ -239,46 +191,61 @@ public class DataSchema {
   }
 
   public enum ColumnDataType {
-    INT,
-    LONG,
-    FLOAT,
-    DOUBLE,
-    BOOLEAN /* Stored as INT */,
-    TIMESTAMP /* Stored as LONG */,
-    STRING,
-    JSON /* Stored as STRING */,
-    BYTES,
-    OBJECT,
-    INT_ARRAY,
-    LONG_ARRAY,
-    FLOAT_ARRAY,
-    DOUBLE_ARRAY,
-    BOOLEAN_ARRAY /* Stored as INT_ARRAY */,
-    TIMESTAMP_ARRAY /* Stored as LONG_ARRAY */,
-    BYTES_ARRAY,
-    STRING_ARRAY;
+    INT(0),
+    LONG(0L),
+    FLOAT(0f),
+    DOUBLE(0d),
+    BIG_DECIMAL(BigDecimal.ZERO),
+    BOOLEAN(INT, 0),
+    TIMESTAMP(LONG, 0L),
+    STRING(""),
+    JSON(STRING, ""),
+    BYTES(new ByteArray(new byte[0])),
+    OBJECT(null),
+    INT_ARRAY(new int[0]),
+    LONG_ARRAY(new long[0]),
+    FLOAT_ARRAY(new float[0]),
+    DOUBLE_ARRAY(new double[0]),
+    BOOLEAN_ARRAY(INT_ARRAY, new int[0]),
+    TIMESTAMP_ARRAY(LONG_ARRAY, new long[0]),
+    STRING_ARRAY(new String[0]),
+    BYTES_ARRAY(new byte[0][]),
+    UNKNOWN(null);
 
-    private static final EnumSet<ColumnDataType> NUMERIC_TYPES = EnumSet.of(INT, LONG, FLOAT, DOUBLE);
+    private static final EnumSet<ColumnDataType> NUMERIC_TYPES = EnumSet.of(INT, LONG, FLOAT, DOUBLE, BIG_DECIMAL);
     private static final EnumSet<ColumnDataType> INTEGRAL_TYPES = EnumSet.of(INT, LONG);
-    private static final EnumSet<ColumnDataType> ARRAY_TYPES = EnumSet.of(INT_ARRAY, LONG_ARRAY, FLOAT_ARRAY,
-        DOUBLE_ARRAY, STRING_ARRAY, BOOLEAN_ARRAY, TIMESTAMP_ARRAY, BYTES_ARRAY);
-    private static final EnumSet<ColumnDataType> NUMERIC_ARRAY_TYPES = EnumSet.of(INT_ARRAY, LONG_ARRAY, FLOAT_ARRAY,
-        DOUBLE_ARRAY);
+    private static final EnumSet<ColumnDataType> ARRAY_TYPES =
+        EnumSet.of(INT_ARRAY, LONG_ARRAY, FLOAT_ARRAY, DOUBLE_ARRAY, STRING_ARRAY, BOOLEAN_ARRAY, TIMESTAMP_ARRAY,
+            BYTES_ARRAY);
+    private static final EnumSet<ColumnDataType> NUMERIC_ARRAY_TYPES =
+        EnumSet.of(INT_ARRAY, LONG_ARRAY, FLOAT_ARRAY, DOUBLE_ARRAY);
     private static final EnumSet<ColumnDataType> INTEGRAL_ARRAY_TYPES = EnumSet.of(INT_ARRAY, LONG_ARRAY);
+
+    // stored data type.
+    private final ColumnDataType _storedColumnDataType;
+
+    // Placeholder for null. We need a placeholder for null so that it can be serialized in the data table
+    private final Object _nullPlaceholder;
+
+    ColumnDataType(Object nullPlaceHolder) {
+      _storedColumnDataType = this;
+      _nullPlaceholder = nullPlaceHolder;
+    }
+
+    ColumnDataType(ColumnDataType storedColumnDataType, Object nullPlaceHolder) {
+      _storedColumnDataType = storedColumnDataType;
+      _nullPlaceholder = nullPlaceHolder;
+    }
+
+    public Object getNullPlaceholder() {
+      return _nullPlaceholder;
+    }
+
     /**
      * Returns the data type stored in Pinot.
      */
     public ColumnDataType getStoredType() {
-      switch (this) {
-        case BOOLEAN:
-          return INT;
-        case TIMESTAMP:
-          return LONG;
-        case JSON:
-          return STRING;
-        default:
-          return this;
-      }
+      return _storedColumnDataType;
     }
 
     public boolean isNumber() {
@@ -310,31 +277,130 @@ public class DataSchema {
     public DataType toDataType() {
       switch (this) {
         case INT:
+        case INT_ARRAY:
           return DataType.INT;
         case LONG:
+        case LONG_ARRAY:
           return DataType.LONG;
         case FLOAT:
+        case FLOAT_ARRAY:
           return DataType.FLOAT;
         case DOUBLE:
+        case DOUBLE_ARRAY:
           return DataType.DOUBLE;
+        case BIG_DECIMAL:
+          return DataType.BIG_DECIMAL;
         case BOOLEAN:
+        case BOOLEAN_ARRAY:
           return DataType.BOOLEAN;
         case TIMESTAMP:
+        case TIMESTAMP_ARRAY:
           return DataType.TIMESTAMP;
         case STRING:
+        case STRING_ARRAY:
           return DataType.STRING;
         case JSON:
           return DataType.JSON;
         case BYTES:
+        case BYTES_ARRAY:
           return DataType.BYTES;
+        case UNKNOWN:
+          return DataType.UNKNOWN;
         default:
           throw new IllegalStateException(String.format("Cannot convert ColumnDataType: %s to DataType", this));
       }
     }
 
+    /**
+     * Converts the value from external value type to the internal value type.
+     *
+     * <p>External value type is used in the following places:
+     * <ul>
+     *   <li>Data ingestion</li>
+     *   <li>Scalar function arguments (UDF)</li>
+     *   <li>Query response</li>
+     * </ul>
+     *
+     * <p>Internal value type is used within the storage and query engine, where value is always of the stored type. For
+     * BYTES type, we use a wrapper class {@link ByteArray} to make it comparable.
+     *
+     * <p>The conversion applies to the following types:
+     * <ul>
+     *   <li>BOOLEAN: boolean -> int</li>
+     *   <li>TIMESTAMP: Timestamp -> long</li>
+     *   <li>BYTES: byte[] -> ByteArray</li>
+     *   <li>BOOLEAN_ARRAY: boolean[] -> int[]</li>
+     *   <li>TIMESTAMP_ARRAY: Timestamp[] -> long[]</li>
+     * </ul>
+     */
+    public Object toInternal(Object value) {
+      switch (this) {
+        case BOOLEAN:
+          return ((boolean) value) ? 1 : 0;
+        case TIMESTAMP:
+          return ((Timestamp) value).getTime();
+        case BYTES:
+          return new ByteArray((byte[]) value);
+        case BOOLEAN_ARRAY:
+          return fromBooleanArray((boolean[]) value);
+        case TIMESTAMP_ARRAY:
+          return fromTimestampArray((Timestamp[]) value);
+        case OBJECT:
+          // For OBJECT type, we need to convert based on the actual type of the value. This can happen when the scalar
+          // function returns Object type, e.g. cast function.
+          if (value instanceof Boolean) {
+            return ((boolean) value) ? 1 : 0;
+          }
+          if (value instanceof Timestamp) {
+            return ((Timestamp) value).getTime();
+          }
+          if (value instanceof byte[]) {
+            return new ByteArray((byte[]) value);
+          }
+          if (value instanceof boolean[]) {
+            return fromBooleanArray((boolean[]) value);
+          }
+          if (value instanceof Timestamp[]) {
+            return fromTimestampArray((Timestamp[]) value);
+          }
+          return value;
+        default:
+          return value;
+      }
+    }
+
+    /**
+     * Converts the value from internal value type to the external value type.
+     *
+     * <p>The conversion applies to the following types:
+     * <ul>
+     *   <li>BOOLEAN: int -> boolean</li>
+     *   <li>TIMESTAMP: long -> Timestamp</li>
+     *   <li>BYTES: ByteArray -> byte[]</li>
+     *   <li>BOOLEAN_ARRAY: int[] -> boolean[]</li>
+     *   <li>TIMESTAMP_ARRAY: long[] -> Timestamp[]</li>
+     * </ul>
+     */
+    public Object toExternal(Object value) {
+      switch (this) {
+        case BOOLEAN:
+          return ((int) value) == 1;
+        case TIMESTAMP:
+          return new Timestamp((long) value);
+        case BYTES:
+          return ((ByteArray) value).getBytes();
+        case BOOLEAN_ARRAY:
+          return toBooleanArray((int[]) value);
+        case TIMESTAMP_ARRAY:
+          return toTimestampArray((long[]) value);
+        default:
+          return value;
+      }
+    }
+
     /**
      * Converts the given internal value to the type for external use (e.g. as UDF argument). The given value should be
-     * compatible with the type.
+     * compatible with the type. This method should be used on the reducer side of the single-stage engine.
      */
     public Serializable convert(Object value) {
       switch (this) {
@@ -346,8 +412,10 @@ public class DataSchema {
           return ((Number) value).floatValue();
         case DOUBLE:
           return ((Number) value).doubleValue();
+        case BIG_DECIMAL:
+          return (BigDecimal) value;
         case BOOLEAN:
-          return (Integer) value == 1;
+          return ((int) value) == 1;
         case TIMESTAMP:
           return new Timestamp((long) value);
         case STRING:
@@ -366,11 +434,14 @@ public class DataSchema {
         case STRING_ARRAY:
           return (String[]) value;
         case BOOLEAN_ARRAY:
-          return toBooleanArray(value);
+          return toBooleanArray((int[]) value);
         case TIMESTAMP_ARRAY:
-          return toTimestampArray(value);
+          return toTimestampArray((long[]) value);
         case BYTES_ARRAY:
           return (byte[][]) value;
+        case UNKNOWN: // fall through
+        case OBJECT:
+          return (Serializable) value;
         default:
           throw new IllegalStateException(String.format("Cannot convert: '%s' to type: %s", value, this));
       }
@@ -386,6 +457,8 @@ public class DataSchema {
           return value.toString();
         case BYTES:
           return BytesUtils.toHexString((byte[]) value);
+        case TIMESTAMP_ARRAY:
+          return formatTimestampArray((Timestamp[]) value);
         default:
           return (Serializable) value;
       }
@@ -404,8 +477,10 @@ public class DataSchema {
           return ((Number) value).floatValue();
         case DOUBLE:
           return ((Number) value).doubleValue();
+        case BIG_DECIMAL:
+          return (BigDecimal) value;
         case BOOLEAN:
-          return (Integer) value == 1;
+          return ((int) value) == 1;
         case TIMESTAMP:
           return new Timestamp((long) value).toString();
         case STRING:
@@ -424,9 +499,9 @@ public class DataSchema {
         case STRING_ARRAY:
           return (String[]) value;
         case BOOLEAN_ARRAY:
-          return toBooleanArray(value);
+          return toBooleanArray((int[]) value);
         case TIMESTAMP_ARRAY:
-          return formatTimestampArray(value);
+          return formatTimestampArray((long[]) value);
         case BYTES_ARRAY:
           return (byte[][]) value;
         default:
@@ -437,6 +512,9 @@ public class DataSchema {
     private static double[] toDoubleArray(Object value) {
       if (value instanceof double[]) {
         return (double[]) value;
+      } else if (value instanceof DoubleArrayList) {
+        // For HistogramAggregationFunction
+        return ((DoubleArrayList) value).elements();
       } else if (value instanceof int[]) {
         int[] intValues = (int[]) value;
         int length = intValues.length;
@@ -467,6 +545,9 @@ public class DataSchema {
     private static long[] toLongArray(Object value) {
       if (value instanceof long[]) {
         return (long[]) value;
+      } else if (value instanceof LongArrayList) {
+        // For FunnelCountAggregationFunction
+        return ((LongArrayList) value).elements();
       } else {
         int[] intValues = (int[]) value;
         int length = intValues.length;
@@ -478,27 +559,58 @@ public class DataSchema {
       }
     }
 
-    private static boolean[] toBooleanArray(Object value) {
-      int[] ints = (int[]) value;
-      boolean[] booleans = new boolean[ints.length];
-      for (int i = 0; i < ints.length; i++) {
-        booleans[i] = ints[i] == 1;
+    private static boolean[] toBooleanArray(int[] intArray) {
+      int length = intArray.length;
+      boolean[] booleanArray = new boolean[length];
+      for (int i = 0; i < length; i++) {
+        booleanArray[i] = intArray[i] == 1;
       }
-      return booleans;
+      return booleanArray;
     }
 
-    private static Timestamp[] toTimestampArray(Object value) {
-      long[] longs = (long[]) value;
-      Timestamp[] timestamps = new Timestamp[longs.length];
-      Arrays.setAll(timestamps, i -> new Timestamp(longs[i]));
-      return timestamps;
+    private static int[] fromBooleanArray(boolean[] booleanArray) {
+      int length = booleanArray.length;
+      int[] intArray = new int[length];
+      for (int i = 0; i < length; i++) {
+        intArray[i] = booleanArray[i] ? 1 : 0;
+      }
+      return intArray;
     }
 
-    private static String[] formatTimestampArray(Object value) {
-      long[] longs = (long[]) value;
-      String[] formatted = new String[longs.length];
-      Arrays.setAll(formatted, i -> new Timestamp(longs[i]).toString());
-      return formatted;
+    private static Timestamp[] toTimestampArray(long[] longArray) {
+      int length = longArray.length;
+      Timestamp[] timestampArray = new Timestamp[length];
+      for (int i = 0; i < length; i++) {
+        timestampArray[i] = new Timestamp(longArray[i]);
+      }
+      return timestampArray;
+    }
+
+    private static long[] fromTimestampArray(Timestamp[] timestampArray) {
+      int length = timestampArray.length;
+      long[] longArray = new long[length];
+      for (int i = 0; i < length; i++) {
+        longArray[i] = timestampArray[i].getTime();
+      }
+      return longArray;
+    }
+
+    private static String[] formatTimestampArray(long[] longArray) {
+      int length = longArray.length;
+      String[] formattedTimestampArray = new String[length];
+      for (int i = 0; i < length; i++) {
+        formattedTimestampArray[i] = new Timestamp(longArray[i]).toString();
+      }
+      return formattedTimestampArray;
+    }
+
+    private static String[] formatTimestampArray(Timestamp[] timestampArray) {
+      int length = timestampArray.length;
+      String[] formattedTimestampArray = new String[length];
+      for (int i = 0; i < length; i++) {
+        formattedTimestampArray[i] = timestampArray[i].toString();
+      }
+      return formattedTimestampArray;
     }
 
     public static ColumnDataType fromDataType(DataType dataType, boolean isSingleValue) {
@@ -515,6 +627,8 @@ public class DataSchema {
           return FLOAT;
         case DOUBLE:
           return DOUBLE;
+        case BIG_DECIMAL:
+          return BIG_DECIMAL;
         case BOOLEAN:
           return BOOLEAN;
         case TIMESTAMP:
@@ -525,6 +639,8 @@ public class DataSchema {
           return JSON;
         case BYTES:
           return BYTES;
+        case UNKNOWN:
+          return UNKNOWN;
         default:
           throw new IllegalStateException("Unsupported data type: " + dataType);
       }
@@ -540,12 +656,12 @@ public class DataSchema {
           return FLOAT_ARRAY;
         case DOUBLE:
           return DOUBLE_ARRAY;
-        case STRING:
-          return STRING_ARRAY;
         case BOOLEAN:
           return BOOLEAN_ARRAY;
         case TIMESTAMP:
           return TIMESTAMP_ARRAY;
+        case STRING:
+          return STRING_ARRAY;
         case BYTES:
           return BYTES_ARRAY;
         default:
