@@ -18,16 +18,19 @@
  */
 package org.apache.pinot.common.utils.config;
 
-import com.google.common.base.Preconditions;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
 import java.util.Map;
+import javax.annotation.Nullable;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.helix.HelixManager;
+import org.apache.pinot.common.assignment.InstancePartitions;
+import org.apache.pinot.common.assignment.InstancePartitionsUtils;
 import org.apache.pinot.common.tier.FixedTierSegmentSelector;
+import org.apache.pinot.common.tier.PinotServerTierStorage;
 import org.apache.pinot.common.tier.Tier;
 import org.apache.pinot.common.tier.TierFactory;
 import org.apache.pinot.common.tier.TierSegmentSelector;
@@ -59,10 +62,38 @@ public final class TierConfigUtils {
     return tierName == null ? "default" : tierName;
   }
 
+  /**
+   * Consider configured tiers and compute default instance partitions for the segment
+   *
+   * @return InstancePartitions if the one can be derived from the given sorted tiers, null otherwise
+   */
+  @Nullable
+  public static InstancePartitions getTieredInstancePartitionsForSegment(String tableNameWithType,
+      String segmentName, @Nullable List<Tier> sortedTiers, HelixManager helixManager) {
+    if (CollectionUtils.isEmpty(sortedTiers)) {
+      return null;
+    }
+
+    // Find first applicable tier
+    for (Tier tier : sortedTiers) {
+      if (tier.getSegmentSelector().selectSegment(tableNameWithType, segmentName)) {
+        // Compute default instance partitions
+        PinotServerTierStorage storage = (PinotServerTierStorage) tier.getStorage();
+        return InstancePartitionsUtils.computeDefaultInstancePartitionsForTag(helixManager, tableNameWithType,
+            tier.getName(), storage.getServerTag());
+      }
+    }
+
+    // Tier not found
+    return null;
+  }
+
+  @Nullable
   public static String getDataDirForTier(TableConfig tableConfig, String tierName) {
     return getDataDirForTier(tableConfig, tierName, Collections.emptyMap());
   }
 
+  @Nullable
   public static String getDataDirForTier(TableConfig tableConfig, String tierName,
       Map<String, Map<String, String>> instanceTierConfigs) {
     String tableNameWithType = tableConfig.getTableName();
@@ -78,17 +109,17 @@ public final class TierConfigUtils {
       }
       if (tierCfg != null) {
         Map<String, String> backendProps = tierCfg.getTierBackendProperties();
-        if (backendProps != null) {
-          dataDir = backendProps.get(CommonConstants.Tier.BACKEND_PROP_DATA_DIR);
-        } else {
+        if (backendProps == null) {
           LOGGER.debug("No backend props for tier: {} in TableConfig of table: {}", tierName, tableNameWithType);
-        }
-        if (StringUtils.isNotEmpty(dataDir)) {
-          LOGGER.debug("Got dataDir: {} for tier: {} in TableConfig of table: {}", dataDir, tierName,
-              tableNameWithType);
-          return dataDir;
         } else {
-          LOGGER.debug("No dataDir for tier: {} in TableConfig of table: {}", tierName, tableNameWithType);
+          dataDir = backendProps.get(CommonConstants.Tier.BACKEND_PROP_DATA_DIR);
+          if (StringUtils.isNotEmpty(dataDir)) {
+            LOGGER.debug("Got dataDir: {} for tier: {} in TableConfig of table: {}", dataDir, tierName,
+                tableNameWithType);
+            return dataDir;
+          } else {
+            LOGGER.debug("No dataDir for tier: {} in TableConfig of table: {}", tierName, tableNameWithType);
+          }
         }
       }
     }
@@ -98,8 +129,6 @@ public final class TierConfigUtils {
       // All instance config names are lower cased while being passed down here.
       dataDir = instanceCfgs.get(CommonConstants.Tier.BACKEND_PROP_DATA_DIR.toLowerCase());
     }
-    Preconditions.checkState(StringUtils.isNotEmpty(dataDir), "No dataDir for tier: %s for table: %s", tierName,
-        tableNameWithType);
     LOGGER.debug("Got dataDir: {} for tier: {} for table: {} in instance configs", dataDir, tierName,
         tableNameWithType);
     return dataDir;
