@@ -21,6 +21,7 @@ package org.apache.pinot.common.utils.fetcher;
 import java.io.File;
 import java.net.URI;
 import java.util.List;
+import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.pinot.spi.env.PinotConfiguration;
 
 
@@ -37,6 +38,12 @@ public interface SegmentFetcher {
   void fetchSegmentToLocal(URI uri, File dest)
       throws Exception;
 
+  /**
+   * Fetches a segment from URI location and untar to local in a streamed manner
+   */
+  File fetchUntarSegmentToLocalStreamed(URI uri, File dest, long rateLimit, AtomicInteger attempts)
+      throws Exception;
+
   /**
    * Fetches a segment to local from any uri in the given list.
    */
