@@ -23,6 +23,7 @@ import com.sun.net.httpserver.HttpServer;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.net.InetSocketAddress;
+import org.apache.http.HttpException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.testng.Assert;
@@ -58,7 +59,7 @@ public class MinionClientTest {
 
   @Test
   public void testTaskSchedule()
-      throws IOException {
+      throws IOException, HttpException {
     HttpServer httpServer = startServer(14202, "/tasks/schedule",
         createHandler(200, "{\"SegmentGenerationAndPushTask\":\"Task_SegmentGenerationAndPushTask_1607470525615\"}",
             0));
@@ -70,7 +71,7 @@ public class MinionClientTest {
 
   @Test
   public void testTasksStates()
-      throws IOException {
+      throws IOException, HttpException {
     HttpServer httpServer = startServer(14203, "/tasks/SegmentGenerationAndPushTask/taskstates",
         createHandler(200, "{\"Task_SegmentGenerationAndPushTask_1607470525615\":\"IN_PROGRESS\"}", 0));
     MinionClient minionClient = new MinionClient("http://localhost:14203", null);
@@ -81,7 +82,7 @@ public class MinionClientTest {
 
   @Test
   public void testTaskState()
-      throws IOException {
+      throws IOException, HttpException {
     HttpServer httpServer = startServer(14204, "/tasks/task/Task_SegmentGenerationAndPushTask_1607470525615/state",
         createHandler(200, "\"COMPLETED\"", 0));
     MinionClient minionClient = new MinionClient("http://localhost:14204", null);
