@@ -30,26 +30,33 @@ import java.net.HttpURLConnection;
 import java.net.URL;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Objects;
 import java.util.Random;
 import java.util.Set;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import javax.annotation.Nullable;
 import javax.inject.Inject;
 import javax.ws.rs.GET;
 import javax.ws.rs.POST;
 import javax.ws.rs.Path;
 import javax.ws.rs.QueryParam;
+import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.Response;
+import org.apache.calcite.jdbc.CalciteSchemaBuilder;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.helix.model.InstanceConfig;
 import org.apache.pinot.common.Utils;
 import org.apache.pinot.common.exception.QueryException;
+import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.utils.config.TagNameUtils;
 import org.apache.pinot.common.utils.request.RequestUtils;
 import org.apache.pinot.controller.ControllerConf;
@@ -59,6 +66,11 @@ import org.apache.pinot.controller.api.access.AccessType;
 import org.apache.pinot.controller.helix.core.PinotHelixResourceManager;
 import org.apache.pinot.core.auth.ManualAuthorization;
 import org.apache.pinot.core.query.executor.sql.SqlQueryExecutor;
+import org.apache.pinot.query.QueryEnvironment;
+import org.apache.pinot.query.catalog.PinotCatalog;
+import org.apache.pinot.query.type.TypeFactory;
+import org.apache.pinot.query.type.TypeSystem;
+import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Broker.Request.QueryOptionKey;
 import org.apache.pinot.spi.utils.JsonUtils;
@@ -66,6 +78,7 @@ import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.apache.pinot.sql.parsers.CalciteSqlCompiler;
 import org.apache.pinot.sql.parsers.CalciteSqlParser;
 import org.apache.pinot.sql.parsers.PinotSqlType;
+import org.apache.pinot.sql.parsers.SqlCompilationException;
 import org.apache.pinot.sql.parsers.SqlNodeAndOptions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -105,6 +118,12 @@ public class PinotQueryResource {
       }
       LOGGER.debug("Trace: {}, Running query: {}", traceEnabled, sqlQuery);
       return executeSqlQuery(httpHeaders, sqlQuery, traceEnabled, queryOptions, "/sql");
+    } catch (ProcessingException pe) {
+      LOGGER.error("Caught exception while processing post request {}", pe.getMessage());
+      return pe.getMessage();
+    } catch (WebApplicationException wae) {
+      LOGGER.error("Caught exception while processing post request", wae);
+      throw wae;
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing post request", e);
       return QueryException.getException(QueryException.INTERNAL_ERROR, e).toString();
@@ -113,11 +132,18 @@ public class PinotQueryResource {
 
   @GET
   @Path("sql")
+  @ManualAuthorization
   public String handleGetSql(@QueryParam("sql") String sqlQuery, @QueryParam("trace") String traceEnabled,
       @QueryParam("queryOptions") String queryOptions, @Context HttpHeaders httpHeaders) {
     try {
       LOGGER.debug("Trace: {}, Running query: {}", traceEnabled, sqlQuery);
       return executeSqlQuery(httpHeaders, sqlQuery, traceEnabled, queryOptions, "/sql");
+    } catch (ProcessingException pe) {
+      LOGGER.error("Caught exception while processing get request {}", pe.getMessage());
+      return pe.getMessage();
+    } catch (WebApplicationException wae) {
+      LOGGER.error("Caught exception while processing get request", wae);
+      throw wae;
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing get request", e);
       return QueryException.getException(QueryException.INTERNAL_ERROR, e).toString();
@@ -127,7 +153,12 @@ public class PinotQueryResource {
   private String executeSqlQuery(@Context HttpHeaders httpHeaders, String sqlQuery, String traceEnabled,
       @Nullable String queryOptions, String endpointUrl)
       throws Exception {
-    SqlNodeAndOptions sqlNodeAndOptions = CalciteSqlParser.compileToSqlNodeAndOptions(sqlQuery);
+    SqlNodeAndOptions sqlNodeAndOptions;
+    try {
+      sqlNodeAndOptions = CalciteSqlParser.compileToSqlNodeAndOptions(sqlQuery);
+    } catch (SqlCompilationException ex) {
+      throw QueryException.getException(QueryException.SQL_PARSING_ERROR, ex);
+    }
     Map<String, String> options = sqlNodeAndOptions.getOptions();
     if (queryOptions != null) {
       Map<String, String> optionsFromString = RequestUtils.getOptionsFromString(queryOptions);
@@ -138,7 +169,7 @@ public class PinotQueryResource {
     if (Boolean.parseBoolean(options.get(QueryOptionKey.USE_MULTISTAGE_ENGINE))) {
       if (_controllerConf.getProperty(CommonConstants.Helix.CONFIG_OF_MULTI_STAGE_ENGINE_ENABLED,
           CommonConstants.Helix.DEFAULT_MULTI_STAGE_ENGINE_ENABLED)) {
-        return getMultiStageQueryResponse(sqlQuery, queryOptions, httpHeaders, endpointUrl);
+        return getMultiStageQueryResponse(sqlQuery, queryOptions, httpHeaders, endpointUrl, traceEnabled);
       } else {
         throw new UnsupportedOperationException("V2 Multi-Stage query engine not enabled. "
             + "Please see https://docs.pinot.apache.org/ for instruction to enable V2 engine.");
@@ -161,31 +192,40 @@ public class PinotQueryResource {
   }
 
   private String getMultiStageQueryResponse(String query, String queryOptions, HttpHeaders httpHeaders,
-      String endpointUrl) {
+      String endpointUrl, String traceEnabled) {
 
     // Validate data access
     // we don't have a cross table access control rule so only ADMIN can make request to multi-stage engine.
     AccessControl accessControl = _accessControlFactory.create();
     if (!accessControl.hasAccess(null, AccessType.READ, httpHeaders, endpointUrl)) {
-      return QueryException.ACCESS_DENIED_ERROR.toString();
+      throw new WebApplicationException("Permission denied", Response.Status.FORBIDDEN);
     }
 
-    // Get brokers, only DEFAULT tenant is supported for now.
-    // TODO: implement logic that only allows executing query where all accessed tables are within the same tenant.
-    List<String> instanceIds = new ArrayList<>(_pinotHelixResourceManager.getAllInstancesForBrokerTenant(
-        TagNameUtils.DEFAULT_TENANT_NAME));
-    if (instanceIds.isEmpty()) {
-      return QueryException.BROKER_RESOURCE_MISSING_ERROR.toString();
-    }
+    QueryEnvironment queryEnvironment = new QueryEnvironment(new TypeFactory(new TypeSystem()),
+        CalciteSchemaBuilder.asRootSchema(new PinotCatalog(_pinotHelixResourceManager.getTableCache())), null, null);
+    List<String> tableNames = queryEnvironment.getTableNamesForQuery(query);
+    List<String> instanceIds;
+    if (tableNames.size() != 0) {
+      List<TableConfig> tableConfigList = getListTableConfigs(tableNames);
+      if (tableConfigList == null || tableConfigList.size() == 0) {
+        return QueryException.getException(QueryException.TABLE_DOES_NOT_EXIST_ERROR,
+            new Exception("Unable to find table in cluster, table does not exist")).toString();
+      }
 
-    instanceIds.retainAll(_pinotHelixResourceManager.getOnlineInstanceList());
-    if (instanceIds.isEmpty()) {
-      return QueryException.BROKER_INSTANCE_MISSING_ERROR.toString();
+      // find the unions of all the broker tenant tags of the queried tables.
+      Set<String> brokerTenantsUnion = getBrokerTenantsUnion(tableConfigList);
+      if (brokerTenantsUnion.isEmpty()) {
+        return QueryException.getException(QueryException.BROKER_REQUEST_SEND_ERROR, new Exception(
+            String.format("Unable to dispatch multistage query for tables: [%s]", tableNames))).toString();
+      }
+      instanceIds = findCommonBrokerInstances(brokerTenantsUnion);
+    } else {
+      // TODO fail these queries going forward. Added this logic to take care of tautologies like BETWEEN 0 and -1.
+      instanceIds = _pinotHelixResourceManager.getAllBrokerInstances();
+      LOGGER.error("Unable to find table name from SQL {} thus dispatching to random broker.", query);
     }
-
-    // Send query to a random broker.
-    String instanceId = instanceIds.get(RANDOM.nextInt(instanceIds.size()));
-    return sendRequestToBroker(query, instanceId, "false", queryOptions, httpHeaders);
+    String instanceId = selectRandomInstanceId(instanceIds);
+    return sendRequestToBroker(query, instanceId, traceEnabled, queryOptions, httpHeaders);
   }
 
   private String getQueryResponse(String query, @Nullable SqlNode sqlNode, String traceEnabled, String queryOptions,
@@ -194,11 +234,26 @@ public class PinotQueryResource {
     String tableName;
     try {
       String inputTableName =
-          sqlNode != null ? RequestUtils.getTableName(CalciteSqlParser.compileSqlNodeToPinotQuery(sqlNode))
-              : CalciteSqlCompiler.compileToBrokerRequest(query).getQuerySource().getTableName();
+          sqlNode != null ? RequestUtils.getTableNames(CalciteSqlParser.compileSqlNodeToPinotQuery(sqlNode)).iterator()
+              .next() : CalciteSqlCompiler.compileToBrokerRequest(query).getQuerySource().getTableName();
       tableName = _pinotHelixResourceManager.getActualTableName(inputTableName);
     } catch (Exception e) {
       LOGGER.error("Caught exception while compiling query: {}", query, e);
+      try {
+        // try to compile the query using multi-stage engine and suggest using it if it succeeds.
+        LOGGER.info("Trying to compile query {} using multi-stage engine", query);
+        QueryEnvironment queryEnvironment = new QueryEnvironment(new TypeFactory(new TypeSystem()),
+            CalciteSchemaBuilder.asRootSchema(new PinotCatalog(_pinotHelixResourceManager.getTableCache())), null,
+            null);
+        queryEnvironment.getTableNamesForQuery(query);
+        LOGGER.info("Successfully compiled query using multi-stage engine: {}", query);
+        return QueryException.getException(QueryException.SQL_PARSING_ERROR, new Exception(
+            "It seems that the query is only supported by the multi-stage engine, please try it by checking the "
+                + "\"Use Multi-Stage Engine\" box above")).toString();
+      } catch (Exception multipleTablesPassingException) {
+        LOGGER.error("Caught exception while compiling query using multi-stage engine: {}",
+            query, multipleTablesPassingException);
+      }
       return QueryException.getException(QueryException.SQL_PARSING_ERROR, e).toString();
     }
     String rawTableName = TableNameBuilder.extractRawTableName(tableName);
@@ -211,19 +266,59 @@ public class PinotQueryResource {
 
     // Get brokers for the resource table.
     List<String> instanceIds = _pinotHelixResourceManager.getBrokerInstancesFor(rawTableName);
+    String instanceId = selectRandomInstanceId(instanceIds);
+    return sendRequestToBroker(query, instanceId, traceEnabled, queryOptions, httpHeaders);
+  }
+
+  // given a list of tables, returns the list of tableConfigs
+  private List<TableConfig> getListTableConfigs(List<String> tableNames) {
+    List<TableConfig> allTableConfigList = new ArrayList<>();
+    for (String tableName : tableNames) {
+      List<TableConfig> tableConfigList = new ArrayList<>();
+      if (_pinotHelixResourceManager.hasRealtimeTable(tableName)) {
+        tableConfigList.add(Objects.requireNonNull(_pinotHelixResourceManager.getRealtimeTableConfig(tableName)));
+      }
+      if (_pinotHelixResourceManager.hasOfflineTable(tableName)) {
+        tableConfigList.add(Objects.requireNonNull(_pinotHelixResourceManager.getOfflineTableConfig(tableName)));
+      }
+      if (tableConfigList.size() == 0) {
+        return null;
+      }
+      allTableConfigList.addAll(tableConfigList);
+    }
+    return allTableConfigList;
+  }
+
+  private String selectRandomInstanceId(List<String> instanceIds) {
     if (instanceIds.isEmpty()) {
       return QueryException.BROKER_RESOURCE_MISSING_ERROR.toString();
     }
 
-    // Retain only online brokers.
     instanceIds.retainAll(_pinotHelixResourceManager.getOnlineInstanceList());
     if (instanceIds.isEmpty()) {
       return QueryException.BROKER_INSTANCE_MISSING_ERROR.toString();
     }
 
     // Send query to a random broker.
-    String instanceId = instanceIds.get(RANDOM.nextInt(instanceIds.size()));
-    return sendRequestToBroker(query, instanceId, traceEnabled, queryOptions, httpHeaders);
+    return instanceIds.get(RANDOM.nextInt(instanceIds.size()));
+  }
+
+  private List<String> findCommonBrokerInstances(Set<String> brokerTenants) {
+    Stream<InstanceConfig> brokerInstanceConfigs = _pinotHelixResourceManager.getAllBrokerInstanceConfigs().stream();
+    for (String brokerTenant : brokerTenants) {
+      brokerInstanceConfigs = brokerInstanceConfigs.filter(
+          instanceConfig -> instanceConfig.containsTag(TagNameUtils.getBrokerTagForTenant(brokerTenant)));
+    }
+    return brokerInstanceConfigs.map(InstanceConfig::getInstanceName).collect(Collectors.toList());
+  }
+
+  // return the union of brokerTenants from the tables list.
+  private Set<String> getBrokerTenantsUnion(List<TableConfig> tableConfigList) {
+    Set<String> tableBrokerTenants = new HashSet<>();
+    for (TableConfig tableConfig : tableConfigList) {
+      tableBrokerTenants.add(tableConfig.getTenantConfig().getBroker());
+    }
+    return tableBrokerTenants;
   }
 
   private String sendRequestToBroker(String query, String instanceId, String traceEnabled, String queryOptions,
@@ -314,9 +409,13 @@ public class PinotQueryResource {
       /*if (LOG.isInfoEnabled()){
         LOGGER.info("The http response code is " + responseCode);
       }*/
-      if (responseCode != HttpURLConnection.HTTP_OK) {
-        throw new IOException("Failed : HTTP error code : " + responseCode + ". Root Cause: "
-            + IOUtils.toString(conn.getErrorStream(), StandardCharsets.UTF_8));
+      if (responseCode == HttpURLConnection.HTTP_FORBIDDEN) {
+        throw new WebApplicationException("Permission denied", Response.Status.FORBIDDEN);
+      } else if (responseCode != HttpURLConnection.HTTP_OK) {
+        InputStream errorStream = conn.getErrorStream();
+        throw new IOException(
+            "Failed : HTTP error code : " + responseCode + ". Root Cause: " + (errorStream != null ? IOUtils.toString(
+                errorStream, StandardCharsets.UTF_8) : "Unknown"));
       }
       final byte[] bytes = drain(new BufferedInputStream(conn.getInputStream()));
 
