@@ -24,11 +24,9 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 import java.util.function.Consumer;
-import javax.annotation.Nonnull;
 import org.apache.helix.task.TaskState;
 import org.apache.pinot.common.data.Segment;
 import org.apache.pinot.controller.helix.core.minion.ClusterInfoAccessor;
-import org.apache.pinot.controller.helix.core.minion.PinotHelixTaskResourceManager;
 import org.apache.pinot.core.common.MinionConstants;
 import org.apache.pinot.core.minion.PinotTaskConfig;
 
@@ -37,8 +35,6 @@ public class TaskGeneratorUtils {
   private TaskGeneratorUtils() {
   }
 
-  private static final long ONE_DAY_IN_MILLIS = 24 * 60 * 60 * 1000L;
-
   /**
    * If task is in final state, it will not be running any more. But note that
    * STOPPED is not a final task state in helix task framework, as a stopped task
@@ -48,31 +44,17 @@ public class TaskGeneratorUtils {
       EnumSet.of(TaskState.COMPLETED, TaskState.FAILED, TaskState.ABORTED, TaskState.TIMED_OUT);
 
   /**
-   * Returns all the segments that have been scheduled in one day but not finished.
-   * <p>
-   * NOTE: we consider tasks not finished in one day as stuck and don't count the segments in them
-   *
-   * @param taskType Task type
-   * @param clusterInfoAccessor Cluster info accessor
-   * @return Set of running segments
+   * Returns all the segments that have been scheduled but not finished.
    */
-  public static Set<Segment> getRunningSegments(@Nonnull String taskType,
-      @Nonnull ClusterInfoAccessor clusterInfoAccessor) {
+  public static Set<Segment> getRunningSegments(String taskType, ClusterInfoAccessor clusterInfoAccessor) {
     Set<Segment> runningSegments = new HashSet<>();
     Map<String, TaskState> taskStates = clusterInfoAccessor.getTaskStates(taskType);
     for (Map.Entry<String, TaskState> entry : taskStates.entrySet()) {
-      // Skip COMPLETED tasks
-      if (entry.getValue() == TaskState.COMPLETED) {
+      if (TASK_FINAL_STATES.contains(entry.getValue())) {
         continue;
       }
-
-      // Skip tasks scheduled for more than one day
       String taskName = entry.getKey();
-      if (isTaskOlderThanOneDay(taskName)) {
-        continue;
-      }
-
-      for (PinotTaskConfig pinotTaskConfig : clusterInfoAccessor.getTaskConfigs(entry.getKey())) {
+      for (PinotTaskConfig pinotTaskConfig : clusterInfoAccessor.getTaskConfigs(taskName)) {
         Map<String, String> configs = pinotTaskConfig.getConfigs();
         runningSegments.add(
             new Segment(configs.get(MinionConstants.TABLE_NAME_KEY), configs.get(MinionConstants.SEGMENT_NAME_KEY)));
@@ -82,30 +64,24 @@ public class TaskGeneratorUtils {
   }
 
   /**
-   * Gets all the tasks for the provided task type and tableName, which do not have TaskState COMPLETED
-   * @return map containing task name to task state for non-completed tasks
-   *
-   * NOTE: we consider tasks not finished in one day as stuck and don't count them
+   * Gets all the tasks for the provided task type and tableName, which have not reached final task state yet.
    */
   public static Map<String, TaskState> getIncompleteTasks(String taskType, String tableNameWithType,
       ClusterInfoAccessor clusterInfoAccessor) {
-    Map<String, TaskState> nonCompletedTasks = new HashMap<>();
+    Map<String, TaskState> incompleteTasks = new HashMap<>();
     Map<String, TaskState> taskStates = clusterInfoAccessor.getTaskStates(taskType);
     for (Map.Entry<String, TaskState> entry : taskStates.entrySet()) {
-      if (entry.getValue() == TaskState.COMPLETED) {
+      if (TASK_FINAL_STATES.contains(entry.getValue())) {
         continue;
       }
       String taskName = entry.getKey();
-      if (isTaskOlderThanOneDay(taskName)) {
-        continue;
-      }
       for (PinotTaskConfig pinotTaskConfig : clusterInfoAccessor.getTaskConfigs(taskName)) {
         if (tableNameWithType.equals(pinotTaskConfig.getConfigs().get(MinionConstants.TABLE_NAME_KEY))) {
-          nonCompletedTasks.put(taskName, entry.getValue());
+          incompleteTasks.put(taskName, entry.getValue());
         }
       }
     }
-    return nonCompletedTasks;
+    return incompleteTasks;
   }
 
   /**
@@ -130,13 +106,4 @@ public class TaskGeneratorUtils {
       }
     }
   }
-
-  /**
-   * Returns true if task's schedule time is older than 1d
-   */
-  private static boolean isTaskOlderThanOneDay(String taskName) {
-    long scheduleTimeMs =
-        Long.parseLong(taskName.substring(taskName.lastIndexOf(PinotHelixTaskResourceManager.TASK_NAME_SEPARATOR) + 1));
-    return System.currentTimeMillis() - scheduleTimeMs > ONE_DAY_IN_MILLIS;
-  }
 }
