@@ -49,9 +49,8 @@ import static org.apache.pinot.controller.recommender.rules.utils.PredicateParse
 
 
 /**
- * A query parser to simulate the nESI cost of a given query(pql/sql) in run-time, recommend an optimal set of
- * dimensions
- * to apply indices on, and calculate the estimated nESI saved by applying such indices
+ * A query parser to simulate the nESI cost of a given query in run-time, recommend an optimal set of dimensions to
+ * apply indices on, and calculate the estimated nESI saved by applying such indices
  */
 public class QueryInvertedSortedIndexRecommender {
   private static final Logger LOGGER = LoggerFactory.getLogger(QueryInvertedSortedIndexRecommender.class);
@@ -65,7 +64,6 @@ public class QueryInvertedSortedIndexRecommender {
   private IndexConfig _indexOverwritten;
   private InvertedSortedIndexJointRuleParams _params;
   private int _numColumnsIndexApplicable;
-  private String _queryType;
 
   /**
    * The plan priority for AND-connected sub-predicates in {@link FilterOperatorUtils}
@@ -289,9 +287,8 @@ public class QueryInvertedSortedIndexRecommender {
       // case: OR connected top level predicates, recursively run parseTopLevel on each on its children and
       // simply return all the results. Each result will contribute to the global recommendation equally
       List<List<PredicateParseResult>> childResults = new ArrayList<>();
-      for (int i = 0; i < filterContextTopLevel.getChildren().size(); i++) {
-        List<List<PredicateParseResult>> childResult =
-            parseTopLevel(filterContextTopLevel.getChildren().get(i), queryWeight);
+      for (FilterContext child : filterContextTopLevel.getChildren()) {
+        List<List<PredicateParseResult>> childResult = parseTopLevel(child, queryWeight);
         if (childResult != null) {
           childResults.addAll(childResult);
         }
@@ -302,6 +299,9 @@ public class QueryInvertedSortedIndexRecommender {
       } else {
         return childResults;
       }
+    } else if (type == FilterContext.Type.NOT) {
+      assert filterContextTopLevel.getChildren().size() == 1;
+      return parseTopLevel(filterContextTopLevel.getChildren().get(0), queryWeight);
     } else {
       // case: Return result directly.
       PredicateParseResult predicateParseResult = parseLeafPredicate(filterContextTopLevel, NESTED_TOP_LEVEL);
@@ -328,6 +328,7 @@ public class QueryInvertedSortedIndexRecommender {
    * Recommend inverted index for:
    * Case AND: The dimension which selects the lowest percentage of rows.
    * Case OR:  All the recommended dimensions from evaluating all its child predicates.
+   * Case NOT: Same as the underlying predicate
    * Case Leaf: See {@link QueryInvertedSortedIndexRecommender#parseLeafPredicate(FilterContext, int)}
    * @param predicateList Single or nested predicates.
    * @param depth         The depth of current AST tree. >= Second level in this function. Top level is handled in
@@ -501,6 +502,9 @@ public class QueryInvertedSortedIndexRecommender {
             .setRecommendationPriorityEnum(RecommendationPriorityEnum.NESTED).setnESI(nESI)
             .setPercentSelected(percentSelected).setnESIWithIdx(nESI).build();
       }
+    } else if (type == FilterContext.Type.NOT) {
+      assert predicateList.getChildren().size() == 1;
+      return parsePredicateList(predicateList.getChildren().get(0), depth);
     } else {
       // case:Leaf predicate
       PredicateParseResult predicateParseResult = parseLeafPredicate(predicateList, depth);
@@ -734,7 +738,6 @@ public class QueryInvertedSortedIndexRecommender {
       queryInvertedSortedIndexRecommender._numColumnsIndexApplicable =
           _inputManager.getNumColumnsInvertedSortedApplicable();
       queryInvertedSortedIndexRecommender._indexOverwritten = _inputManager.getOverWrittenConfigs().getIndexConfig();
-      queryInvertedSortedIndexRecommender._queryType = _inputManager.getQueryType();
 
       return queryInvertedSortedIndexRecommender;
     }
