@@ -20,10 +20,10 @@
 
 import React, { useEffect, useState } from 'react';
 import { makeStyles } from '@material-ui/core/styles';
-import { Grid, Checkbox, Button, FormControl, Input, InputLabel } from '@material-ui/core';
+import { Grid, Checkbox, Button, FormControl, Input, InputLabel, Box, Typography } from '@material-ui/core';
 import Alert from '@material-ui/lab/Alert';
 import FileCopyIcon from '@material-ui/icons/FileCopy';
-import { TableData } from 'Models';
+import { SqlException, TableData } from 'Models';
 import { UnControlled as CodeMirror } from 'react-codemirror2';
 import 'codemirror/lib/codemirror.css';
 import 'codemirror/theme/material.css';
@@ -88,7 +88,8 @@ const useStyles = makeStyles((theme) => ({
     paddingBottom: '48px',
   },
   sqlError: {
-    whiteSpace: 'pre-wrap',
+    whiteSpace: 'pre',
+    overflow: "auto"
   },
   timeoutControl: {
     bottom: 10
@@ -183,7 +184,7 @@ const QueryPage = () => {
 
   const [outputResult, setOutputResult] = useState('');
 
-  const [resultError, setResultError] = useState('');
+  const [resultError, setResultError] = useState<SqlException[] | string>([]);
 
   const [queryStats, setQueryStats] = useState<TableData>({
     columns: [],
@@ -271,18 +272,18 @@ const QueryPage = () => {
     setQueryLoader(true);
     queryExecuted.current = true;
     let params;
-    let queryOptions = '';
+    let queryOptions = [];
     if(queryTimeout){
-      queryOptions += `timeoutMs=${queryTimeout}`;
+      queryOptions.push(`timeoutMs=${queryTimeout}`);
     }
     if(checked.useMSE){
-      queryOptions += `useMultistageEngine=true`;
+      queryOptions.push(`useMultistageEngine=true`);
     }
     const finalQuery = `${query || inputQuery.trim()}`;
     params = JSON.stringify({
       sql: `${finalQuery}`,
       trace: checked.tracing,
-      queryOptions: `${queryOptions}`,
+      queryOptions: `${queryOptions.join(";")}`,
     });
 
     if(finalQuery !== ''){
@@ -299,7 +300,7 @@ const QueryPage = () => {
     }
 
     const results = await PinotMethodUtils.getQueryResults(params);
-    setResultError(results.error || '');
+    setResultError(results.exceptions || []);
     setResultData(results.result || { columns: [], records: [] });
     setQueryStats(results.queryStats || { columns: responseStatCols, records: [] });
     setOutputResult(JSON.stringify(results.data, null, 2) || '');
@@ -316,10 +317,6 @@ const QueryPage = () => {
          + `created with an older schema. `
          + `Please reload the table in order to refresh these segments to the new schema.`);
     }
-    if (checked.useMSE) {
-      warnings.push(`Using V2 Multi-Stage Query Engine. This is an experimental feature. Please report any bugs to `
-          + `Apache Pinot Slack channel.`);
-    }
     return warnings;
   }
 
@@ -503,14 +500,14 @@ const QueryPage = () => {
                 Tracing
               </Grid>
 
-              <Grid item xs={2}>
+              <Grid item xs={3}>
                 <Checkbox
                     name="useMSE"
                     color="primary"
                     onChange={handleChange}
                     checked={checked.useMSE}
                 />
-                Use V2 Engine
+                Use Multi-Stage Engine
               </Grid>
 
               <Grid item xs={3}>
@@ -553,93 +550,108 @@ const QueryPage = () => {
                                    </Alert>
                   )
                 }
-
-                {resultError ? (
+        
+                {/* Sql result errors */}
+                {resultError && typeof resultError === "string" && (
                   <Alert severity="error" className={classes.sqlError}>
                     {resultError}
                   </Alert>
-                ) : (
-                  <>
-                    <Grid item xs style={{ backgroundColor: 'white' }}>
-                      {resultData.columns.length ? (
-                        <>
-                          <Grid container className={classes.actionBtns}>
-                            <Button
-                              variant="contained"
-                              color="primary"
-                              size="small"
-                              className={classes.btn}
-                              onClick={() => downloadData('xls')}
-                            >
-                              Excel
-                            </Button>
-                            <Button
-                              variant="contained"
-                              color="primary"
-                              size="small"
-                              className={classes.btn}
-                              onClick={() => downloadData('csv')}
-                            >
-                              CSV
-                            </Button>
-                            <Button
-                              variant="contained"
-                              color="primary"
-                              size="small"
-                              className={classes.btn}
-                              onClick={() => copyToClipboard()}
-                            >
-                              Copy
-                            </Button>
-                            {copyMsg ? (
-                              <Alert
-                                icon={<FileCopyIcon fontSize="inherit" />}
-                                severity="info"
-                              >
-                                Copied {resultData.records.length} rows to
-                                Clipboard
-                              </Alert>
-                            ) : null}
-
-                            <FormControlLabel
-                              control={
-                                <Switch
-                                  checked={checked.showResultJSON}
-                                  onChange={handleChange}
-                                  name="showResultJSON"
-                                  color="primary"
-                                />
-                              }
-                              label="Show JSON format"
-                              className={classes.runNowBtn}
-                            />
-                          </Grid>
-                          {!checked.showResultJSON ? (
-                            <CustomizedTables
-                              title="Query Result"
-                              data={resultData}
-                              isSticky={true}
-                              showSearchBox={true}
-                              inAccordionFormat={true}
-                            />
-                          ) : resultData.columns.length ? (
-                            <SimpleAccordion
-                              headerTitle="Query Result (JSON Format)"
-                              showSearchBox={false}
-                            >
-                              <CodeMirror
-                                options={jsonoptions}
-                                value={outputResult}
-                                className={classes.queryOutput}
-                                autoCursor={false}
-                              />
-                            </SimpleAccordion>
-                          ) : null}
-                        </>
-                      ) : null}
-                    </Grid>
-                  </>
                 )}
+
+                {resultError && typeof resultError === "object" && resultError.length > 0 && (
+                  <>
+                    {
+                      resultError.map((error) => (
+                        <Box style={{paddingBottom: "16px"}}>
+                          <Alert className={classes.sqlError} severity="error">
+                            <Typography variant="body2">Error Code: {error.errorCode}</Typography>
+                            {error.message}
+                          </Alert>
+                        </Box>
+                      ))
+                    }
+                  </>
+                  )
+                }
+        
+                <Grid item xs style={{ backgroundColor: 'white' }}>
+                  {resultData.columns.length ? (
+                    <>
+                      <Grid container className={classes.actionBtns}>
+                        <Button
+                          variant="contained"
+                          color="primary"
+                          size="small"
+                          className={classes.btn}
+                          onClick={() => downloadData('xls')}
+                        >
+                          Excel
+                        </Button>
+                        <Button
+                          variant="contained"
+                          color="primary"
+                          size="small"
+                          className={classes.btn}
+                          onClick={() => downloadData('csv')}
+                        >
+                          CSV
+                        </Button>
+                        <Button
+                          variant="contained"
+                          color="primary"
+                          size="small"
+                          className={classes.btn}
+                          onClick={() => copyToClipboard()}
+                        >
+                          Copy
+                        </Button>
+                        {copyMsg ? (
+                          <Alert
+                            icon={<FileCopyIcon fontSize="inherit" />}
+                            severity="info"
+                          >
+                            Copied {resultData.records.length} rows to
+                            Clipboard
+                          </Alert>
+                        ) : null}
+
+                        <FormControlLabel
+                          control={
+                            <Switch
+                              checked={checked.showResultJSON}
+                              onChange={handleChange}
+                              name="showResultJSON"
+                              color="primary"
+                            />
+                          }
+                          label="Show JSON format"
+                          className={classes.runNowBtn}
+                        />
+                      </Grid>
+                      {!checked.showResultJSON ? (
+                        <CustomizedTables
+                          title="Query Result"
+                          data={resultData}
+                          isSticky={true}
+                          showSearchBox={true}
+                          inAccordionFormat={true}
+                        />
+                      ) : resultData.columns.length ? (
+                        <SimpleAccordion
+                          headerTitle="Query Result (JSON Format)"
+                          showSearchBox={false}
+                        >
+                          <CodeMirror
+                            options={jsonoptions}
+                            value={outputResult}
+                            className={classes.queryOutput}
+                            autoCursor={false}
+                          />
+                        </SimpleAccordion>
+                      ) : null}
+                    </>
+                  ) : null}
+                </Grid>
               </>
             )}
           </Grid>
