@@ -17,8 +17,9 @@
  * under the License.
  */
 
+import jwtDecode from "jwt-decode";
 import { get, map, each, isEqual, isArray, keys, union } from 'lodash';
-import { DataTable, SQLResult } from 'Models';
+import { DataTable, SqlException, SQLResult } from 'Models';
 import moment from 'moment';
 import {
   getTenants,
@@ -38,7 +39,7 @@ import {
   resumeTasks,
   cleanupTasks,
   deleteTasks,
-  sheduleTask,
+  scheduleTask,
   executeTask,
   getJobDetail,
   getMinionMeta,
@@ -266,21 +267,19 @@ const getQueryResults = (params) => {
   return getQueryResult(params).then(({ data }) => {
     let queryResponse = getAsObject(data);
 
-    let errorStr = '';
+    let exceptions: SqlException[] | string = [];
     let dataArray = [];
     let columnList = [];
     // if sql api throws error, handle here
     if(typeof queryResponse === 'string'){
-      errorStr = queryResponse;
-    } else if (queryResponse && queryResponse.exceptions && queryResponse.exceptions.length) {
-      errorStr = JSON.stringify(queryResponse.exceptions, null, 2);
-    } else
-    {
-      if (queryResponse.resultTable?.dataSchema?.columnNames?.length)
-      {
-        columnList = queryResponse.resultTable.dataSchema.columnNames;
-        dataArray = queryResponse.resultTable.rows;
-      }
+      exceptions = queryResponse;
+    } 
+    if (queryResponse && queryResponse.exceptions && queryResponse.exceptions.length) {
+      exceptions = queryResponse.exceptions as SqlException[];
+    } 
+    if (queryResponse.resultTable?.dataSchema?.columnNames?.length) {
+      columnList = queryResponse.resultTable.dataSchema.columnNames;
+      dataArray = queryResponse.resultTable.rows;
     }
 
     const columnStats = ['timeUsedMs',
@@ -308,7 +307,7 @@ const getQueryResults = (params) => {
     ];
 
     return {
-      error: errorStr,
+      exceptions: exceptions,
       result: {
         columns: columnList,
         records: dataArray,
@@ -575,6 +574,7 @@ const getSegmentDetails = (tableName, segmentName) => {
     const segmentMetaDataJson = { ...segmentMetaData }
     delete segmentMetaDataJson.indexes
     delete segmentMetaDataJson.columns
+    const indexes = get(segmentMetaData, 'indexes', {})
 
     return {
       replicaSet: {
@@ -583,7 +583,7 @@ const getSegmentDetails = (tableName, segmentName) => {
       },
       indexes: {
         columns: ['Field Name', 'Bloom Filter', 'Dictionary', 'Forward Index', 'Sorted', 'Inverted Index', 'JSON Index', 'Null Value Vector Reader', 'Range Index'],
-        records: Object.keys(segmentMetaData.indexes).map(fieldName => [
+        records: Object.keys(indexes).map(fieldName => [
           fieldName,
           segmentMetaData.indexes[fieldName]["bloom-filter"] === "YES",
           segmentMetaData.indexes[fieldName]["dictionary"] === "YES",
@@ -778,7 +778,12 @@ const getAllTaskTypes = async () => {
 const getTaskInfo = async (taskType) => {
   const tasksRes = await getTaskTypeTasks(taskType);
   const stateRes = await getTaskTypeState(taskType);
-  const state = get(stateRes, 'data', '');
+
+  let state = get(stateRes, 'data', '');
+  // response contains error
+  if(typeof state !== "string") {
+    state = "";
+  }
   return [tasksRes?.data?.length || 0, state];
 };
 
@@ -893,8 +898,8 @@ const deleteSegmentOp = (tableName, segmentName) => {
   });
 };
 
-const fetchTableJobs = async (tableName: string) => {
-  const response = await getTableJobs(tableName);
+const fetchTableJobs = async (tableName: string, jobTypes?: string) => {
+  const response = await getTableJobs(tableName, jobTypes);
   
   return response.data;
 }
@@ -1000,7 +1005,9 @@ const verifyAuth = (authToken) => {
 
 const getAccessTokenFromHashParams = () => {
   let accessToken = '';
-  const urlSearchParams = new URLSearchParams(location.hash.substr(1));
+  const hashParam = removeAllLeadingForwardSlash(location.hash.substring(1));
+  
+  const urlSearchParams = new URLSearchParams(hashParam);
   if (urlSearchParams.has('access_token')) {
     accessToken = urlSearchParams.get('access_token') as string;
   }
@@ -1008,6 +1015,13 @@ const getAccessTokenFromHashParams = () => {
   return accessToken;
 };
 
+const removeAllLeadingForwardSlash = (string: string) => {
+  if(!string) {
+    return "";
+  }
+
+  return string.replace(new RegExp("^/+", "g"), "");
+}
 
 // validates app redirect path with known routes
 const validateRedirectPath = (path: string): boolean => {
@@ -1101,7 +1115,7 @@ const getTableData = (params)=>{
 };
 
 const scheduleTaskAction = (tableName, taskType)=>{
-  return sheduleTask(tableName, taskType).then(response=>{
+  return scheduleTask(tableName, taskType).then(response=>{
     return response.data;
   })
 };
@@ -1142,6 +1156,52 @@ const updateUser = (userObject, passwordChanged) =>{
   })
 }
 
+const getAuthUserNameFromAccessToken = (
+  accessToken: string
+): string => {
+  if (!accessToken) {
+      return "";
+  }
+
+  let decoded;
+  try {
+      decoded = jwtDecode(accessToken);
+  } catch (e) {
+      return "";
+  }
+
+  if (!decoded) {
+      return "";
+  }
+
+  const name = get(decoded, "name") || "";
+  return name;
+};
+
+const getAuthUserEmailFromAccessToken = (
+  accessToken: string
+): string => {
+  if (!accessToken) {
+      return "";
+  }
+
+  let decoded;
+  try {
+      decoded = jwtDecode(accessToken);
+  } catch (e) {
+      return "";
+  }
+
+  if (!decoded) {
+      return "";
+  }
+
+  const email =
+      get(decoded, "email") || "";
+
+  return email;
+};
+
 export default {
   getTenantsData,
   getAllInstances,
@@ -1224,5 +1284,7 @@ export default {
   getUserList,
   addUser,
   deleteUser,
-  updateUser
+  updateUser,
+  getAuthUserNameFromAccessToken,
+  getAuthUserEmailFromAccessToken
 };
