@@ -24,11 +24,14 @@ import org.apache.pinot.controller.helix.ControllerTest;
 import org.apache.pinot.spi.data.DimensionFieldSpec;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.data.Schema;
-import org.testng.Assert;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertTrue;
+import static org.testng.Assert.fail;
+
 
 public class PinotSchemaRestletResourceTest {
   private static final ControllerTest TEST_INSTANCE = ControllerTest.getInstance();
@@ -50,12 +53,13 @@ public class PinotSchemaRestletResourceTest {
         + "  \"metricFieldSpecs\" : [ {\n" + "    \"name\" : \"score\",\n" + "    \"dataType\" : \"FLOAT\"\n"
         + "  } ]}";
     try {
-      final String response = ControllerTest
-          .sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(), schemaString);
-      Assert.assertEquals(response, "{\"unrecognizedProperties\":{},\"status\":\"transcript successfully added\"}");
+      final String response =
+          ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(),
+              schemaString);
+      assertEquals(response, "{\"unrecognizedProperties\":{},\"status\":\"transcript successfully added\"}");
     } catch (IOException e) {
       // should not reach here
-      Assert.fail("Shouldn't have caught an exception: " + e.getMessage());
+      fail("Shouldn't have caught an exception: " + e.getMessage());
     }
   }
 
@@ -67,28 +71,26 @@ public class PinotSchemaRestletResourceTest {
 
     // Add the schema
     String addSchemaUrl = TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate();
-    SimpleHttpResponse resp =
-        ControllerTest.sendMultipartPostRequest(addSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
+    SimpleHttpResponse resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl, schema.toSingleLineJsonString());
+    assertEquals(resp.getStatusCode(), 200);
 
     // Add a new column
     DimensionFieldSpec newColumnFieldSpec = new DimensionFieldSpec("newColumn", DataType.STRING, true);
     schema.addField(newColumnFieldSpec);
 
     // Update the schema with addSchema api and override off
-    resp =
-        ControllerTest.sendMultipartPostRequest(addSchemaUrl + "?override=false", schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 409);
+    resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl + "?override=false", schema.toSingleLineJsonString());
+    assertEquals(resp.getStatusCode(), 409);
 
     // Update the schema with addSchema api and override on
     resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
+    assertEquals(resp.getStatusCode(), 200);
 
     // Get the schema and verify the new column exists
     String getSchemaUrl = TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaGet(schemaName);
     Schema remoteSchema = Schema.fromString(ControllerTest.sendGetRequest(getSchemaUrl));
-    Assert.assertEquals(remoteSchema, schema);
-    Assert.assertTrue(remoteSchema.hasColumn(newColumnFieldSpec.getName()));
+    assertEquals(remoteSchema, schema);
+    assertTrue(remoteSchema.hasColumn(newColumnFieldSpec.getName()));
 
     // Add another new column
     DimensionFieldSpec newColumnFieldSpec2 = new DimensionFieldSpec("newColumn2", DataType.STRING, true);
@@ -97,43 +99,59 @@ public class PinotSchemaRestletResourceTest {
     // Update the schema with updateSchema api
     String updateSchemaUrl = TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate(schemaName);
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
+    assertEquals(resp.getStatusCode(), 200);
 
     // Get the schema and verify both the new columns exist
     remoteSchema = Schema.fromString(ControllerTest.sendGetRequest(getSchemaUrl));
-    Assert.assertEquals(remoteSchema, schema);
-    Assert.assertTrue(remoteSchema.hasColumn(newColumnFieldSpec.getName()));
-    Assert.assertTrue(remoteSchema.hasColumn(newColumnFieldSpec2.getName()));
+    assertEquals(remoteSchema, schema);
+    assertTrue(remoteSchema.hasColumn(newColumnFieldSpec.getName()));
+    assertTrue(remoteSchema.hasColumn(newColumnFieldSpec2.getName()));
 
     // Change the column data type - backward-incompatible change
     newColumnFieldSpec.setDataType(DataType.INT);
 
     // Update the schema with addSchema api and override on
     resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
 
     // Update the schema with updateSchema api
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
 
     // Change the column data type from STRING to BOOLEAN
     newColumnFieldSpec.setDataType(DataType.BOOLEAN);
 
+    // Update the schema with addSchema api and override on, force on
+    resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl + "?force=true", schema.toSingleLineJsonString());
+    assertEquals(resp.getStatusCode(), 200);
+
+    // Change another column max length from default 512 to 2000
+    newColumnFieldSpec2.setMaxLength(2000);
+    // Change another column default null value from default "null" to "0"
+    newColumnFieldSpec2.setDefaultNullValue("0");
+
     // Update the schema with addSchema api and override on
     resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
+    assertEquals(resp.getStatusCode(), 200);
 
-    // Change another column data type from STRING to BOOLEAN
-    newColumnFieldSpec2.setDataType(DataType.BOOLEAN);
-
-    // Update the schema with updateSchema api
-    resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
-
-    // Get the schema and verify the data types are not changed
+    // Get the schema and verify the default null value and max length have been changed
     remoteSchema = Schema.fromString(ControllerTest.sendGetRequest(getSchemaUrl));
-    Assert.assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec.getName()).getDataType(), DataType.STRING);
-    Assert.assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec2.getName()).getDataType(), DataType.STRING);
+    assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec2.getName()).getMaxLength(), 2000);
+    assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec2.getName()).getDefaultNullValue(), "0");
+
+    // Change another column max length from 1000
+    newColumnFieldSpec2.setMaxLength(1000);
+    // Change another column default null value from default "null" to "1"
+    newColumnFieldSpec2.setDefaultNullValue("1");
+
+    // Update the schema with updateSchema api and override on
+    resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
+    assertEquals(resp.getStatusCode(), 200);
+
+    // Get the schema and verify the default null value and max length have been changed
+    remoteSchema = Schema.fromString(ControllerTest.sendGetRequest(getSchemaUrl));
+    assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec2.getName()).getMaxLength(), 1000);
+    assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec2.getName()).getDefaultNullValue(), "1");
 
     // Add a new BOOLEAN column
     DimensionFieldSpec newColumnFieldSpec3 = new DimensionFieldSpec("newColumn3", DataType.BOOLEAN, true);
@@ -141,35 +159,34 @@ public class PinotSchemaRestletResourceTest {
 
     // Update the schema with updateSchema api
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 200);
+    assertEquals(resp.getStatusCode(), 200);
 
     // Get the schema and verify the new column has BOOLEAN data type
     remoteSchema = Schema.fromString(ControllerTest.sendGetRequest(getSchemaUrl));
-    Assert.assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec3.getName()).getDataType(), DataType.BOOLEAN);
+    assertEquals(remoteSchema.getFieldSpecFor(newColumnFieldSpec3.getName()).getDataType(), DataType.BOOLEAN);
 
     // Post invalid schema string
     String invalidSchemaString = schema.toSingleLineJsonString().substring(1);
     resp = ControllerTest.sendMultipartPostRequest(addSchemaUrl, invalidSchemaString);
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, invalidSchemaString);
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
 
     // Update schema with null schema name
     schema.setSchemaName(null);
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
 
     // Update schema with non-matching schema name
     String newSchemaName = "newSchemaName";
     schema.setSchemaName(newSchemaName);
     resp = ControllerTest.sendMultipartPutRequest(updateSchemaUrl, schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 400);
+    assertEquals(resp.getStatusCode(), 400);
 
     // Update non-existing schema
     resp = ControllerTest.sendMultipartPutRequest(
-        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate(newSchemaName),
-        schema.toSingleLineJsonString());
-    Assert.assertEquals(resp.getStatusCode(), 404);
+        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate(newSchemaName), schema.toSingleLineJsonString());
+    assertEquals(resp.getStatusCode(), 404);
   }
 
   @Test
@@ -185,21 +202,19 @@ public class PinotSchemaRestletResourceTest {
             + "  \"metricFieldSpecs\" : [ {\n" + "    \"name\" : \"score\",\n" + "    \"dataType\" : \"FLOAT\"\n"
             + "  } ]}";
 
-    String response =
-        ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaValidate(),
-            schemaStringWithExtraProps);
-    Assert.assertTrue(response.contains("/illegalKey1\" : 1"));
+    String response = ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaValidate(),
+        schemaStringWithExtraProps);
+    assertTrue(response.contains("/illegalKey1\" : 1"));
 
-    response =
-        ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(),
-            schemaStringWithExtraProps);
-    Assert.assertEquals(response,
+    response = ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(),
+        schemaStringWithExtraProps);
+    assertEquals(response,
         "{\"unrecognizedProperties\":{\"/illegalKey1\":1},\"status\":\"transcript2 successfully added\"}");
 
-    response = ControllerTest.sendPutRequest(
-        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate("transcript2"),
-        schemaStringWithExtraProps);
-    Assert.assertEquals(response,
+    response =
+        ControllerTest.sendPutRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate("transcript2"),
+            schemaStringWithExtraProps);
+    assertEquals(response,
         "{\"unrecognizedProperties\":{\"/illegalKey1\":1},\"status\":\"transcript2 successfully added\"}");
   }
 
@@ -216,19 +231,19 @@ public class PinotSchemaRestletResourceTest {
             + "  \"metricFieldSpecs\" : [ {\n" + "    \"name\" : \"score\",\n" + "    \"dataType\" : \"FLOAT\"\n"
             + "  } ]}";
 
-    SimpleHttpResponse response = ControllerTest.sendMultipartPostRequest(
-        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaValidate(), schemaStringWithExtraProps);
-    Assert.assertTrue(response.getResponse().contains("/illegalKey1\" : 1"));
+    SimpleHttpResponse response =
+        ControllerTest.sendMultipartPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaValidate(),
+            schemaStringWithExtraProps);
+    assertTrue(response.getResponse().contains("/illegalKey1\" : 1"));
 
-    response = ControllerTest.sendMultipartPostRequest(
-        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(), schemaStringWithExtraProps);
-    Assert.assertEquals(response.getResponse(),
+    response = ControllerTest.sendMultipartPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaCreate(),
+        schemaStringWithExtraProps);
+    assertEquals(response.getResponse(),
         "{\"unrecognizedProperties\":{\"/illegalKey1\":1},\"status\":\"transcript2 successfully added\"}");
 
     response = ControllerTest.sendMultipartPutRequest(
-        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate("transcript2"),
-        schemaStringWithExtraProps);
-    Assert.assertEquals(response.getResponse(),
+        TEST_INSTANCE.getControllerRequestURLBuilder().forSchemaUpdate("transcript2"), schemaStringWithExtraProps);
+    assertEquals(response.getResponse(),
         "{\"unrecognizedProperties\":{\"/illegalKey1\":1},\"status\":\"transcript2 successfully added\"}");
   }
 
