@@ -39,6 +39,7 @@ import org.apache.pinot.common.lineage.SegmentLineageUtils;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.metrics.ControllerGauge;
 import org.apache.pinot.common.metrics.ControllerMetrics;
+import org.apache.pinot.common.metrics.MetricValueUtils;
 import org.apache.pinot.common.utils.LLCSegmentName;
 import org.apache.pinot.controller.ControllerConf;
 import org.apache.pinot.controller.LeadControllerManager;
@@ -142,6 +143,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -150,25 +152,22 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.REPLICATION_FROM_CONFIG), 2);
-    Assert
-        .assertEquals(_controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENT_COUNT), 3);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENT_COUNT_INCLUDING_REPLACED),
-        5);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENTS_IN_ERROR_STATE), 1);
-    Assert
-        .assertEquals(_controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.NUMBER_OF_REPLICAS),
-            2);
-    Assert
-        .assertEquals(
-            _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_OF_REPLICAS),
-            66);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.REPLICATION_FROM_CONFIG), 2);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENT_COUNT), 3);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENT_COUNT_INCLUDING_REPLACED), 5);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), 1);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.NUMBER_OF_REPLICAS), 2);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_OF_REPLICAS), 66);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
   }
 
   @Test
@@ -233,6 +232,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -241,20 +241,18 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.REPLICATION_FROM_CONFIG), 3);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
-    Assert
-        .assertEquals(_controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.NUMBER_OF_REPLICAS),
-            3);
-    Assert
-        .assertEquals(
-            _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_OF_REPLICAS),
-            100);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
-    Assert.assertEquals(_controllerMetrics
-        .getValueOfTableGauge(externalView.getId(), ControllerGauge.MISSING_CONSUMING_SEGMENT_TOTAL_COUNT), 2);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.REPLICATION_FROM_CONFIG), 3);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.NUMBER_OF_REPLICAS), 3);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_OF_REPLICAS), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.MISSING_CONSUMING_SEGMENT_TOTAL_COUNT), 2);
   }
 
   Map<String, String> getStreamConfigMap() {
@@ -334,6 +332,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -342,15 +341,14 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENTS_IN_ERROR_STATE), 1);
-    Assert
-        .assertEquals(_controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.NUMBER_OF_REPLICAS),
-            0);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 75);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.TABLE_COMPRESSED_SIZE), 1111);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), 1);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.NUMBER_OF_REPLICAS), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 75);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.TABLE_COMPRESSED_SIZE), 1111);
   }
 
   @Test
@@ -391,6 +389,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -399,10 +398,12 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.NUMBER_OF_REPLICAS), 0);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.NUMBER_OF_REPLICAS), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
   }
 
   @Test
@@ -431,6 +432,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -439,14 +441,14 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.SEGMENTS_IN_ERROR_STATE),
-        Long.MIN_VALUE);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.NUMBER_OF_REPLICAS),
-        Long.MIN_VALUE);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.PERCENT_OF_REPLICAS),
-        Long.MIN_VALUE);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), Long.MIN_VALUE);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.NUMBER_OF_REPLICAS), Long.MIN_VALUE);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.PERCENT_OF_REPLICAS), Long.MIN_VALUE);
+    Assert.assertFalse(MetricValueUtils.tableGaugeExists(_controllerMetrics, tableName,
+            ControllerGauge.TABLE_COMPRESSED_SIZE));
   }
 
   @Test
@@ -522,6 +524,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -530,19 +533,16 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
-    Assert
-        .assertEquals(_controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.NUMBER_OF_REPLICAS),
-            2);
-    Assert
-        .assertEquals(
-            _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_OF_REPLICAS),
-            100);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
-    Assert.assertEquals(
-        _controllerMetrics.getValueOfTableGauge(externalView.getId(), ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.NUMBER_OF_REPLICAS), 2);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_OF_REPLICAS), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.TABLE_COMPRESSED_SIZE), 0);
   }
 
   @Test
@@ -579,6 +579,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -587,11 +588,14 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.NUMBER_OF_REPLICAS), 1);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.PERCENT_OF_REPLICAS), 100);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.PERCENT_SEGMENTS_AVAILABLE),
-        100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.SEGMENTS_IN_ERROR_STATE), 0);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.NUMBER_OF_REPLICAS), 1);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.PERCENT_OF_REPLICAS), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
   }
 
   @Test
@@ -625,17 +629,20 @@ public class SegmentStatusCheckerTest {
       _leadControllerManager = mock(LeadControllerManager.class);
       when(_leadControllerManager.isLeaderForTable(anyString())).thenReturn(true);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
         new SegmentStatusChecker(_helixResourceManager, _leadControllerManager, _config, _controllerMetrics,
             _executorService);
     // verify state before test
-    Assert.assertEquals(_controllerMetrics.getValueOfGlobalGauge(ControllerGauge.DISABLED_TABLE_COUNT), 0);
+    Assert.assertEquals(
+        MetricValueUtils.getGlobalGaugeValue(_controllerMetrics, ControllerGauge.DISABLED_TABLE_COUNT), 0);
     // update metrics
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfGlobalGauge(ControllerGauge.DISABLED_TABLE_COUNT), 1);
+    Assert.assertEquals(
+        MetricValueUtils.getGlobalGaugeValue(_controllerMetrics, ControllerGauge.DISABLED_TABLE_COUNT), 1);
   }
 
   @Test
@@ -665,17 +672,20 @@ public class SegmentStatusCheckerTest {
       _leadControllerManager = mock(LeadControllerManager.class);
       when(_leadControllerManager.isLeaderForTable(anyString())).thenReturn(true);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
         new SegmentStatusChecker(_helixResourceManager, _leadControllerManager, _config, _controllerMetrics,
             _executorService);
     // verify state before test
-    Assert.assertEquals(_controllerMetrics.getValueOfGlobalGauge(ControllerGauge.DISABLED_TABLE_COUNT), 0);
+    Assert.assertFalse(
+        MetricValueUtils.globalGaugeExists(_controllerMetrics, ControllerGauge.DISABLED_TABLE_COUNT));
     // update metrics
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfGlobalGauge(ControllerGauge.DISABLED_TABLE_COUNT), 1);
+    Assert.assertEquals(
+        MetricValueUtils.getGlobalGaugeValue(_controllerMetrics, ControllerGauge.DISABLED_TABLE_COUNT), 1);
   }
 
   @Test
@@ -686,6 +696,69 @@ public class SegmentStatusCheckerTest {
     noSegmentsInternal(-1);
   }
 
+  @Test
+  public void lessThanOnePercentSegmentsUnavailableTest()
+          throws Exception {
+    String tableName = "myTable_OFFLINE";
+    int numSegments = 200;
+    List<String> allTableNames = new ArrayList<String>();
+    allTableNames.add(tableName);
+    TableConfig tableConfig =
+            new TableConfigBuilder(TableType.OFFLINE).setTableName(tableName).setNumReplicas(1).build();
+
+    IdealState idealState = new IdealState(tableName);
+    for (int i = 0; i < numSegments; i++) {
+      idealState.setPartitionState("myTable_" + i, "pinot1", "ONLINE");
+    }
+    idealState.setReplicas("1");
+    idealState.setRebalanceMode(IdealState.RebalanceMode.CUSTOMIZED);
+
+    ExternalView externalView = new ExternalView(tableName);
+    externalView.setState("myTable_0", "pinot1", "OFFLINE");
+    for (int i = 1; i < numSegments; i++) {
+      externalView.setState("myTable_" + i, "pinot1", "ONLINE");
+    }
+
+    {
+      _helixResourceManager = mock(PinotHelixResourceManager.class);
+      when(_helixResourceManager.getAllTables()).thenReturn(allTableNames);
+      when(_helixResourceManager.getTableConfig(tableName)).thenReturn(tableConfig);
+      when(_helixResourceManager.getTableIdealState(tableName)).thenReturn(idealState);
+      when(_helixResourceManager.getTableExternalView(tableName)).thenReturn(externalView);
+    }
+    {
+      _helixPropertyStore = mock(ZkHelixPropertyStore.class);
+      when(_helixResourceManager.getPropertyStore()).thenReturn(_helixPropertyStore);
+      SegmentLineage segmentLineage = new SegmentLineage(tableName);
+      when(_helixPropertyStore.get(eq("/SEGMENT_LINEAGE/" + tableName), any(), eq(AccessOption.PERSISTENT)))
+              .thenReturn(segmentLineage.toZNRecord());
+    }
+    {
+      _config = mock(ControllerConf.class);
+      when(_config.getStatusCheckerFrequencyInSeconds()).thenReturn(300);
+      when(_config.getStatusCheckerWaitForPushTimeInSeconds()).thenReturn(300);
+    }
+    {
+      _leadControllerManager = mock(LeadControllerManager.class);
+      when(_leadControllerManager.isLeaderForTable(anyString())).thenReturn(true);
+    }
+    {
+      _tableSizeReader = mock(TableSizeReader.class);
+      when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
+    }
+    PinotMetricUtils.cleanUp();
+    _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
+    _controllerMetrics = new ControllerMetrics(_metricsRegistry);
+    _segmentStatusChecker =
+            new SegmentStatusChecker(_helixResourceManager, _leadControllerManager, _config, _controllerMetrics,
+                    _executorService);
+    _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
+    _segmentStatusChecker.start();
+    _segmentStatusChecker.run();
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, externalView.getId(),
+            ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 99);
+  }
+
   public void noSegmentsInternal(final int nReplicas)
       throws Exception {
     final String tableName = "myTable_REALTIME";
@@ -720,6 +793,7 @@ public class SegmentStatusCheckerTest {
       _tableSizeReader = mock(TableSizeReader.class);
       when(_tableSizeReader.getTableSizeDetails(anyString(), anyInt())).thenReturn(null);
     }
+    PinotMetricUtils.cleanUp();
     _metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry();
     _controllerMetrics = new ControllerMetrics(_metricsRegistry);
     _segmentStatusChecker =
@@ -728,12 +802,14 @@ public class SegmentStatusCheckerTest {
     _segmentStatusChecker.setTableSizeReader(_tableSizeReader);
     _segmentStatusChecker.start();
     _segmentStatusChecker.run();
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.SEGMENTS_IN_ERROR_STATE),
-        Long.MIN_VALUE);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.NUMBER_OF_REPLICAS),
-        nReplicasExpectedValue);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.PERCENT_OF_REPLICAS), 100);
-    Assert.assertEquals(_controllerMetrics.getValueOfTableGauge(tableName, ControllerGauge.PERCENT_SEGMENTS_AVAILABLE),
-        100);
+
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.SEGMENTS_IN_ERROR_STATE), Long.MIN_VALUE);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.NUMBER_OF_REPLICAS), nReplicasExpectedValue);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.PERCENT_OF_REPLICAS), 100);
+    Assert.assertEquals(MetricValueUtils.getTableGaugeValue(_controllerMetrics, tableName,
+        ControllerGauge.PERCENT_SEGMENTS_AVAILABLE), 100);
   }
 }
