@@ -475,6 +475,24 @@ public class DataBlockCache {
     _dataFetcher.fetchStringValues(column, evaluator, _docIds, _length, buffer);
   }
 
+  /**
+   * Get the bytes values for a multi-valued column.
+   *
+   * @param column Column name
+   * @return Array of bytes values
+   */
+  public byte[][][] getBytesValuesForMVColumn(String column) {
+    byte[][][] bytesValues = getValues(FieldSpec.DataType.BYTES, column);
+    if (markLoaded(FieldSpec.DataType.BYTES, column)) {
+      if (bytesValues == null) {
+        bytesValues = new byte[_length][][];
+        putValues(FieldSpec.DataType.BYTES, column, bytesValues);
+      }
+      _dataFetcher.fetchBytesValues(column, _docIds, _length, bytesValues);
+    }
+    return bytesValues;
+  }
+
   /**
    * Get the number of values for a multi-valued column.
    *
