@@ -21,7 +21,8 @@ package org.apache.pinot.core.geospatial.transform.function;
 import com.google.common.base.Preconditions;
 import java.util.List;
 import java.util.Map;
-import org.apache.pinot.core.operator.blocks.ProjectionBlock;
+import org.apache.pinot.core.operator.ColumnContext;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
 import org.apache.pinot.core.operator.transform.function.BaseTransformFunction;
 import org.apache.pinot.core.operator.transform.function.LiteralTransformFunction;
@@ -29,7 +30,6 @@ import org.apache.pinot.core.operator.transform.function.TransformFunction;
 import org.apache.pinot.core.plan.DocIdSetPlanNode;
 import org.apache.pinot.segment.local.utils.GeometrySerializer;
 import org.apache.pinot.segment.local.utils.GeometryUtils;
-import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.locationtech.jts.geom.Geometry;
 import org.locationtech.jts.geom.LineString;
@@ -57,7 +57,7 @@ public class StAreaFunction extends BaseTransformFunction {
   }
 
   @Override
-  public void init(List<TransformFunction> arguments, Map<String, DataSource> dataSourceMap) {
+  public void init(List<TransformFunction> arguments, Map<String, ColumnContext> columnContextMap) {
     Preconditions
         .checkArgument(arguments.size() == 1, "Exactly 1 argument is required for transform function: %s", getName());
     TransformFunction transformFunction = arguments.get(0);
@@ -76,13 +76,13 @@ public class StAreaFunction extends BaseTransformFunction {
   }
 
   @Override
-  public double[] transformToDoubleValuesSV(ProjectionBlock projectionBlock) {
+  public double[] transformToDoubleValuesSV(ValueBlock valueBlock) {
     if (_results == null) {
       _results = new double[DocIdSetPlanNode.MAX_DOC_PER_CALL];
     }
 
-    byte[][] values = _transformFunction.transformToBytesValuesSV(projectionBlock);
-    int numDocs = projectionBlock.getNumDocs();
+    byte[][] values = _transformFunction.transformToBytesValuesSV(valueBlock);
+    int numDocs = valueBlock.getNumDocs();
     for (int i = 0; i < numDocs; i++) {
       Geometry geometry = GeometrySerializer.deserialize(values[i]);
       _results[i] = GeometryUtils.isGeography(geometry) ? calculateGeographyArea(geometry) : geometry.getArea();
