@@ -20,17 +20,17 @@ package org.apache.pinot.core.operator.filter;
 
 import java.util.Collections;
 import java.util.List;
+import org.apache.pinot.core.common.BlockDocIdSet;
+import org.apache.pinot.core.common.ExplainPlanRows;
 import org.apache.pinot.core.common.Operator;
-import org.apache.pinot.core.operator.blocks.FilterBlock;
 import org.apache.pinot.core.operator.docidsets.MatchAllDocIdSet;
 
 
 public class MatchAllFilterOperator extends BaseFilterOperator {
   public static final String EXPLAIN_NAME = "FILTER_MATCH_ENTIRE_SEGMENT";
-  private final int _numDocs;
 
   public MatchAllFilterOperator(int numDocs) {
-    _numDocs = numDocs;
+    super(numDocs, false);
   }
 
   @Override
@@ -39,8 +39,8 @@ public class MatchAllFilterOperator extends BaseFilterOperator {
   }
 
   @Override
-  protected FilterBlock getNextBlock() {
-    return new FilterBlock(new MatchAllDocIdSet(_numDocs));
+  protected BlockDocIdSet getTrues() {
+    return new MatchAllDocIdSet(_numDocs);
   }
 
 
@@ -63,4 +63,9 @@ public class MatchAllFilterOperator extends BaseFilterOperator {
   public String toExplainString() {
     return new StringBuilder(EXPLAIN_NAME).append("(docs:").append(_numDocs).append(')').toString();
   }
+
+  @Override
+  public void prepareForExplainPlan(ExplainPlanRows explainPlanRows) {
+    explainPlanRows.setHasMatchAllFilter(true);
+  }
 }
