@@ -24,10 +24,11 @@ import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import org.apache.pinot.common.function.DateTimeUtils;
 import org.apache.pinot.common.function.TimeZoneKey;
-import org.apache.pinot.core.operator.blocks.ProjectionBlock;
+import org.apache.pinot.core.operator.ColumnContext;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
-import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.joda.time.DateTimeField;
+import org.roaringbitmap.RoaringBitmap;
 
 
 /**
@@ -96,23 +97,23 @@ public class DateTruncTransformFunction extends BaseTransformFunction {
   }
 
   @Override
-  public void init(List<TransformFunction> arguments, Map<String, DataSource> dataSourceMap) {
+  public void init(List<TransformFunction> arguments, Map<String, ColumnContext> columnContextMap) {
     Preconditions.checkArgument(arguments.size() >= 2 && arguments.size() <= 5,
         "Between two to five arguments are required, example: %s", EXAMPLE_INVOCATION);
-    String unit = ((LiteralTransformFunction) arguments.get(0)).getLiteral().toLowerCase();
+    String unit = ((LiteralTransformFunction) arguments.get(0)).getStringLiteral().toLowerCase();
     TransformFunction valueArgument = arguments.get(1);
     Preconditions.checkArgument(
         !(valueArgument instanceof LiteralTransformFunction) && valueArgument.getResultMetadata().isSingleValue(),
         "The second argument of dateTrunc transform function must be a single-valued column or a transform function");
     _mainTransformFunction = valueArgument;
     String inputTimeUnitStr =
-        (arguments.size() >= 3) ? ((LiteralTransformFunction) arguments.get(2)).getLiteral().toUpperCase()
+        (arguments.size() >= 3) ? ((LiteralTransformFunction) arguments.get(2)).getStringLiteral().toUpperCase()
             : TimeUnit.MILLISECONDS.name();
     _inputTimeUnit = TimeUnit.valueOf(inputTimeUnitStr);
 
-    String timeZone = arguments.size() >= 4 ? ((LiteralTransformFunction) arguments.get(3)).getLiteral() : UTC_TZ;
+    String timeZone = arguments.size() >= 4 ? ((LiteralTransformFunction) arguments.get(3)).getStringLiteral() : UTC_TZ;
     String outputTimeUnitStr =
-        arguments.size() >= 5 ? ((LiteralTransformFunction) arguments.get(4)).getLiteral().toUpperCase()
+        arguments.size() >= 5 ? ((LiteralTransformFunction) arguments.get(4)).getStringLiteral().toUpperCase()
             : inputTimeUnitStr;
     TimeZoneKey timeZoneKey = TimeZoneKey.getTimeZoneKey(timeZone);
 
@@ -127,12 +128,10 @@ public class DateTruncTransformFunction extends BaseTransformFunction {
   }
 
   @Override
-  public long[] transformToLongValuesSV(ProjectionBlock projectionBlock) {
-    int length = projectionBlock.getNumDocs();
-    if (_longValuesSV == null) {
-      _longValuesSV = new long[length];
-    }
-    long[] input = _mainTransformFunction.transformToLongValuesSV(projectionBlock);
+  public long[] transformToLongValuesSV(ValueBlock valueBlock) {
+    int length = valueBlock.getNumDocs();
+    initLongValuesSV(length);
+    long[] input = _mainTransformFunction.transformToLongValuesSV(valueBlock);
     for (int i = 0; i < length; i++) {
       _longValuesSV[i] =
           _outputTimeUnit.convert(_field.roundFloor(TimeUnit.MILLISECONDS.convert(input[i], _inputTimeUnit)),
@@ -140,4 +139,9 @@ public class DateTruncTransformFunction extends BaseTransformFunction {
     }
     return _longValuesSV;
   }
+
+  @Override
+  public RoaringBitmap getNullBitmap(ValueBlock valueBlock) {
+    return _mainTransformFunction.getNullBitmap(valueBlock);
+  }
 }
