@@ -23,9 +23,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import org.apache.pinot.core.operator.blocks.ProjectionBlock;
+import org.apache.pinot.core.operator.ColumnContext;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
-import org.apache.pinot.segment.spi.datasource.DataSource;
 
 
 /**
@@ -57,7 +57,7 @@ public class RegexpExtractTransformFunction extends BaseTransformFunction {
   }
 
   @Override
-  public void init(List<TransformFunction> arguments, Map<String, DataSource> dataSourceMap) {
+  public void init(List<TransformFunction> arguments, Map<String, ColumnContext> columnContextMap) {
     Preconditions.checkArgument(arguments.size() >= 2 && arguments.size() <= 4,
         "REGEXP_EXTRACT takes between 2 to 4 arguments. See usage: "
             + "REGEXP_EXTRACT(`value`, `regexp`[, `group`[, `default_value`]]");
@@ -66,14 +66,14 @@ public class RegexpExtractTransformFunction extends BaseTransformFunction {
     TransformFunction regexpFunction = arguments.get(1);
     Preconditions.checkState(regexpFunction instanceof LiteralTransformFunction,
         "`regexp` must be a literal regex expression.");
-    _regexp = Pattern.compile(((LiteralTransformFunction) regexpFunction).getLiteral());
+    _regexp = Pattern.compile(((LiteralTransformFunction) regexpFunction).getStringLiteral());
 
     if (arguments.size() >= 3) {
       TransformFunction groupFunction = arguments.get(2);
       Preconditions.checkState(groupFunction instanceof LiteralTransformFunction
-              && Integer.parseInt(((LiteralTransformFunction) groupFunction).getLiteral()) >= 0,
+              && ((LiteralTransformFunction) groupFunction).getIntLiteral() >= 0,
           "`group` must be a literal, non-negative integer.");
-      _group = Integer.parseInt(((LiteralTransformFunction) groupFunction).getLiteral());
+      _group = ((LiteralTransformFunction) groupFunction).getIntLiteral();
     } else {
       _group = 0;
     }
@@ -82,7 +82,7 @@ public class RegexpExtractTransformFunction extends BaseTransformFunction {
       TransformFunction positionFunction = arguments.get(3);
       Preconditions.checkState(positionFunction instanceof LiteralTransformFunction,
           "`default_value` must be a literal expression.");
-      _defaultValue = ((LiteralTransformFunction) regexpFunction).getLiteral();
+      _defaultValue = ((LiteralTransformFunction) regexpFunction).getStringLiteral();
     } else {
       _defaultValue = "";
     }
@@ -95,12 +95,10 @@ public class RegexpExtractTransformFunction extends BaseTransformFunction {
   }
 
   @Override
-  public String[] transformToStringValuesSV(ProjectionBlock projectionBlock) {
-    int length = projectionBlock.getNumDocs();
-    if (_stringValuesSV == null) {
-      _stringValuesSV = new String[length];
-    }
-    String[] valuesSV = _valueFunction.transformToStringValuesSV(projectionBlock);
+  public String[] transformToStringValuesSV(ValueBlock valueBlock) {
+    int length = valueBlock.getNumDocs();
+    initStringValuesSV(length);
+    String[] valuesSV = _valueFunction.transformToStringValuesSV(valueBlock);
     for (int i = 0; i < length; i++) {
       Matcher matcher = _regexp.matcher(valuesSV[i]);
       if (matcher.find() && matcher.groupCount() >= _group) {
