@@ -18,19 +18,20 @@
  */
 package org.apache.pinot.core.operator.transform.function;
 
+import com.google.common.annotations.VisibleForTesting;
 import java.util.ArrayList;
 import java.util.EnumMap;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import javax.annotation.Nullable;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.common.function.FunctionInfo;
 import org.apache.pinot.common.function.FunctionRegistry;
 import org.apache.pinot.common.function.TransformFunctionType;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FunctionContext;
+import org.apache.pinot.common.utils.HashUtil;
 import org.apache.pinot.core.geospatial.transform.function.GeoToH3Function;
 import org.apache.pinot.core.geospatial.transform.function.StAreaFunction;
 import org.apache.pinot.core.geospatial.transform.function.StAsBinaryFunction;
@@ -46,6 +47,7 @@ import org.apache.pinot.core.geospatial.transform.function.StGeometryTypeFunctio
 import org.apache.pinot.core.geospatial.transform.function.StPointFunction;
 import org.apache.pinot.core.geospatial.transform.function.StPolygonFunction;
 import org.apache.pinot.core.geospatial.transform.function.StWithinFunction;
+import org.apache.pinot.core.operator.ColumnContext;
 import org.apache.pinot.core.operator.transform.function.SingleParamMathTransformFunction.AbsTransformFunction;
 import org.apache.pinot.core.operator.transform.function.SingleParamMathTransformFunction.CeilTransformFunction;
 import org.apache.pinot.core.operator.transform.function.SingleParamMathTransformFunction.ExpTransformFunction;
@@ -68,9 +70,20 @@ import org.apache.pinot.core.operator.transform.function.TrigonometricTransformF
 import org.apache.pinot.core.operator.transform.function.TrigonometricTransformFunctions.SinhTransformFunction;
 import org.apache.pinot.core.operator.transform.function.TrigonometricTransformFunctions.TanTransformFunction;
 import org.apache.pinot.core.operator.transform.function.TrigonometricTransformFunctions.TanhTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.CosineDistanceTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.InnerProductTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.L1DistanceTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.L2DistanceTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.VectorDimsTransformFunction;
+import org.apache.pinot.core.operator.transform.function.VectorTransformFunctions.VectorNormTransformFunction;
 import org.apache.pinot.core.query.request.context.QueryContext;
+import org.apache.pinot.core.query.request.context.utils.QueryContextConverterUtils;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.spi.exception.BadQueryRequestException;
+import org.apache.pinot.sql.parsers.CalciteSqlParser;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 
 /**
  * Factory class for transformation functions.
@@ -79,6 +92,7 @@ public class TransformFunctionFactory {
   private TransformFunctionFactory() {
   }
 
+  private static final Logger LOGGER = LoggerFactory.getLogger(TransformFunctionFactory.class);
   private static final Map<String, Class<? extends TransformFunction>> TRANSFORM_FUNCTION_MAP = createRegistry();
 
   private static Map<String, Class<? extends TransformFunction>> createRegistry() {
@@ -105,14 +119,10 @@ public class TransformFunctionFactory {
     typeToImplementation.put(TransformFunctionType.TRUNCATE, TruncateDecimalTransformFunction.class);
 
     typeToImplementation.put(TransformFunctionType.CAST, CastTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.JSONEXTRACTSCALAR,
-        JsonExtractScalarTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.JSONEXTRACTKEY,
-        JsonExtractKeyTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.TIMECONVERT,
-        TimeConversionTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.DATETIMECONVERT,
-        DateTimeConversionTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.JSONEXTRACTSCALAR, JsonExtractScalarTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.JSONEXTRACTKEY, JsonExtractKeyTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.TIMECONVERT, TimeConversionTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.DATETIMECONVERT, DateTimeConversionTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.DATETRUNC, DateTruncTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.YEAR, DateTimeTransformFunction.Year.class);
     typeToImplementation.put(TransformFunctionType.YEAR_OF_WEEK, DateTimeTransformFunction.YearOfWeek.class);
@@ -131,32 +141,29 @@ public class TransformFunctionFactory {
     typeToImplementation.put(TransformFunctionType.MAPVALUE, MapValueTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.INIDSET, InIdSetTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.LOOKUP, LookupTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.CLPDECODE, CLPDecodeTransformFunction.class);
 
     typeToImplementation.put(TransformFunctionType.EXTRACT, ExtractTransformFunction.class);
 
     // Regexp functions
-    typeToImplementation.put(TransformFunctionType.REGEXP_EXTRACT,
-        RegexpExtractTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.REGEXP_EXTRACT, RegexpExtractTransformFunction.class);
 
     // Array functions
-    typeToImplementation.put(TransformFunctionType.ARRAYAVERAGE,
-        ArrayAverageTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.ARRAYAVERAGE, ArrayAverageTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.ARRAYMAX, ArrayMaxTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.ARRAYMIN, ArrayMinTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.ARRAYSUM, ArraySumTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.ARRAY_VALUE_CONSTRUCTOR, ArrayLiteralTransformFunction.class);
 
     typeToImplementation.put(TransformFunctionType.GROOVY, GroovyTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.CASE, CaseTransformFunction.class);
 
     typeToImplementation.put(TransformFunctionType.EQUALS, EqualsTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.NOT_EQUALS, NotEqualsTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.GREATER_THAN,
-        GreaterThanTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.GREATER_THAN_OR_EQUAL,
-        GreaterThanOrEqualTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.GREATER_THAN, GreaterThanTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.GREATER_THAN_OR_EQUAL, GreaterThanOrEqualTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.LESS_THAN, LessThanTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.LESS_THAN_OR_EQUAL,
-        LessThanOrEqualTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.LESS_THAN_OR_EQUAL, LessThanOrEqualTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.IN, InTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.NOT_IN, NotInTransformFunction.class);
 
@@ -167,22 +174,17 @@ public class TransformFunctionFactory {
 
     // geo functions
     // geo constructors
-    typeToImplementation.put(TransformFunctionType.ST_GEOG_FROM_TEXT,
-        StGeogFromTextFunction.class);
-    typeToImplementation.put(TransformFunctionType.ST_GEOG_FROM_WKB,
-        StGeogFromWKBFunction.class);
-    typeToImplementation.put(TransformFunctionType.ST_GEOM_FROM_TEXT,
-        StGeomFromTextFunction.class);
-    typeToImplementation.put(TransformFunctionType.ST_GEOM_FROM_WKB,
-        StGeomFromWKBFunction.class);
+    typeToImplementation.put(TransformFunctionType.ST_GEOG_FROM_TEXT, StGeogFromTextFunction.class);
+    typeToImplementation.put(TransformFunctionType.ST_GEOG_FROM_WKB, StGeogFromWKBFunction.class);
+    typeToImplementation.put(TransformFunctionType.ST_GEOM_FROM_TEXT, StGeomFromTextFunction.class);
+    typeToImplementation.put(TransformFunctionType.ST_GEOM_FROM_WKB, StGeomFromWKBFunction.class);
     typeToImplementation.put(TransformFunctionType.ST_POINT, StPointFunction.class);
     typeToImplementation.put(TransformFunctionType.ST_POLYGON, StPolygonFunction.class);
 
     // geo measurements
     typeToImplementation.put(TransformFunctionType.ST_AREA, StAreaFunction.class);
     typeToImplementation.put(TransformFunctionType.ST_DISTANCE, StDistanceFunction.class);
-    typeToImplementation.put(TransformFunctionType.ST_GEOMETRY_TYPE,
-        StGeometryTypeFunction.class);
+    typeToImplementation.put(TransformFunctionType.ST_GEOMETRY_TYPE, StGeometryTypeFunction.class);
 
     // geo outputs
     typeToImplementation.put(TransformFunctionType.ST_AS_BINARY, StAsBinaryFunction.class);
@@ -201,9 +203,12 @@ public class TransformFunctionFactory {
     typeToImplementation.put(TransformFunctionType.GREATEST, GreatestTransformFunction.class);
 
     // null handling
+    typeToImplementation.put(TransformFunctionType.IS_TRUE, IsTrueTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.IS_NOT_TRUE, IsNotTrueTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.IS_FALSE, IsFalseTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.IS_NOT_FALSE, IsNotFalseTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.IS_NULL, IsNullTransformFunction.class);
-    typeToImplementation.put(TransformFunctionType.IS_NOT_NULL,
-        IsNotNullTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.IS_NOT_NULL, IsNotNullTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.COALESCE, CoalesceTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.IS_DISTINCT_FROM, IsDistinctFromTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.IS_NOT_DISTINCT_FROM, IsNotDistinctFromTransformFunction.class);
@@ -223,9 +228,17 @@ public class TransformFunctionFactory {
     typeToImplementation.put(TransformFunctionType.DEGREES, DegreesTransformFunction.class);
     typeToImplementation.put(TransformFunctionType.RADIANS, RadiansTransformFunction.class);
 
+    // Vector functions
+    typeToImplementation.put(TransformFunctionType.COSINE_DISTANCE, CosineDistanceTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.INNER_PRODUCT, InnerProductTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.L1_DISTANCE, L1DistanceTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.L2_DISTANCE, L2DistanceTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.VECTOR_DIMS, VectorDimsTransformFunction.class);
+    typeToImplementation.put(TransformFunctionType.VECTOR_NORM, VectorNormTransformFunction.class);
+
     Map<String, Class<? extends TransformFunction>> registry = new HashMap<>(typeToImplementation.size());
     for (Map.Entry<TransformFunctionType, Class<? extends TransformFunction>> entry : typeToImplementation.entrySet()) {
-      for (String alias : entry.getKey().getAliases()) {
+      for (String alias : entry.getKey().getAlternativeNames()) {
         registry.put(canonicalize(alias), entry.getValue());
       }
     }
@@ -234,7 +247,7 @@ public class TransformFunctionFactory {
 
   /**
    * Initializes the factory with a set of transform function classes.
-   * <p>Should be called only once before calling {@link #get(ExpressionContext, Map)}.
+   * <p>Should be called only once before using the factory.
    *
    * @param transformFunctionClasses Set of transform function classes
    */
@@ -242,41 +255,30 @@ public class TransformFunctionFactory {
     for (Class<TransformFunction> transformFunctionClass : transformFunctionClasses) {
       TransformFunction transformFunction;
       try {
-        transformFunction = transformFunctionClass.newInstance();
-      } catch (InstantiationException | IllegalAccessException e) {
+        transformFunction = transformFunctionClass.getDeclaredConstructor().newInstance();
+      } catch (Exception e) {
         throw new RuntimeException(
-            "Caught exception while instantiating transform function from class: " + transformFunctionClass.toString(),
-            e);
+            "Caught exception while instantiating transform function from class: " + transformFunctionClass, e);
       }
       String transformFunctionName = canonicalize(transformFunction.getName());
-      if (TRANSFORM_FUNCTION_MAP.containsKey(transformFunctionName)) {
-        throw new IllegalArgumentException("Transform function: " + transformFunctionName + " already exists");
+      if (TRANSFORM_FUNCTION_MAP.put(transformFunctionName, transformFunctionClass) == null) {
+        LOGGER.info("Registering function: {} with class: {}", transformFunctionName, transformFunctionClass);
+      } else {
+        LOGGER.info("Replacing function: {} with class: {}", transformFunctionName, transformFunctionClass);
       }
-      TRANSFORM_FUNCTION_MAP.put(transformFunctionName, transformFunctionClass);
     }
   }
 
   /**
    * Returns an instance of transform function for the given expression.
    *
-   * @param expression Transform expression
-   * @param dataSourceMap Map from column name to column data source
+   * @param expression       Transform expression
+   * @param columnContextMap Map from column name to context
+   * @param queryContext     Query context
    * @return Transform function
    */
-  public static TransformFunction get(ExpressionContext expression, Map<String, DataSource> dataSourceMap) {
-    return get(null, expression, dataSourceMap);
-  }
-
-  /**
-   * Returns an instance of transform function for the given expression.
-   *
-   * @param queryContext the query context if available
-   * @param expression Transform expression
-   * @param dataSourceMap Map from column name to column data source
-   * @return Transform function
-   */
-  public static TransformFunction get(@Nullable QueryContext queryContext, ExpressionContext expression,
-      Map<String, DataSource> dataSourceMap) {
+  public static TransformFunction get(ExpressionContext expression, Map<String, ColumnContext> columnContextMap,
+      QueryContext queryContext) {
     switch (expression.getType()) {
       case FUNCTION:
         FunctionContext function = expression.getFunction();
@@ -284,12 +286,19 @@ public class TransformFunctionFactory {
         List<ExpressionContext> arguments = function.getArguments();
         int numArguments = arguments.size();
 
+        // Check if the function is ArrayLiteraltransform function
+        if (functionName.equalsIgnoreCase(ArrayLiteralTransformFunction.FUNCTION_NAME)) {
+          return queryContext.getOrComputeSharedValue(ArrayLiteralTransformFunction.class,
+              expression.getFunction().getArguments(),
+              ArrayLiteralTransformFunction::new);
+        }
+
         TransformFunction transformFunction;
         Class<? extends TransformFunction> transformFunctionClass = TRANSFORM_FUNCTION_MAP.get(functionName);
         if (transformFunctionClass != null) {
           // Transform function
           try {
-            transformFunction = transformFunctionClass.newInstance();
+            transformFunction = transformFunctionClass.getDeclaredConstructor().newInstance();
           } catch (Exception e) {
             throw new RuntimeException("Caught exception while constructing transform function: " + functionName, e);
           }
@@ -309,10 +318,10 @@ public class TransformFunctionFactory {
 
         List<TransformFunction> transformFunctionArguments = new ArrayList<>(numArguments);
         for (ExpressionContext argument : arguments) {
-          transformFunctionArguments.add(TransformFunctionFactory.get(queryContext, argument, dataSourceMap));
+          transformFunctionArguments.add(TransformFunctionFactory.get(argument, columnContextMap, queryContext));
         }
         try {
-          transformFunction.init(transformFunctionArguments, dataSourceMap);
+          transformFunction.init(transformFunctionArguments, columnContextMap, queryContext.isNullHandlingEnabled());
         } catch (Exception e) {
           throw new BadQueryRequestException("Caught exception while initializing transform function: " + functionName,
               e);
@@ -320,16 +329,34 @@ public class TransformFunctionFactory {
         return transformFunction;
       case IDENTIFIER:
         String columnName = expression.getIdentifier();
-        return new IdentifierTransformFunction(columnName, dataSourceMap.get(columnName));
+        return new IdentifierTransformFunction(columnName, columnContextMap.get(columnName));
       case LITERAL:
-        return queryContext == null ? new LiteralTransformFunction(expression.getLiteral())
-            : queryContext.getOrComputeSharedValue(LiteralTransformFunction.class, expression.getLiteral(),
-                LiteralTransformFunction::new);
+        return queryContext.getOrComputeSharedValue(LiteralTransformFunction.class, expression.getLiteral(),
+            LiteralTransformFunction::new);
       default:
         throw new IllegalStateException();
     }
   }
 
+  @VisibleForTesting
+  public static TransformFunction get(ExpressionContext expression, Map<String, DataSource> dataSourceMap) {
+    Map<String, ColumnContext> columnContextMap = new HashMap<>(HashUtil.getHashMapCapacity(dataSourceMap.size()));
+    dataSourceMap.forEach((k, v) -> columnContextMap.put(k, ColumnContext.fromDataSource(v)));
+    QueryContext dummy = QueryContextConverterUtils.getQueryContext(
+        CalciteSqlParser.compileToPinotQuery("SELECT * from testTable;"));
+    return get(expression, columnContextMap, dummy);
+  }
+
+  @VisibleForTesting
+  public static TransformFunction getNullHandlingEnabled(ExpressionContext expression,
+      Map<String, DataSource> dataSourceMap) {
+    Map<String, ColumnContext> columnContextMap = new HashMap<>(HashUtil.getHashMapCapacity(dataSourceMap.size()));
+    dataSourceMap.forEach((k, v) -> columnContextMap.put(k, ColumnContext.fromDataSource(v)));
+    QueryContext dummy = QueryContextConverterUtils.getQueryContext(
+        CalciteSqlParser.compileToPinotQuery("SET enableNullHandling = true; SELECT * from testTable;"));
+    return get(expression, columnContextMap, dummy);
+  }
+
   /**
    * Converts the transform function name into its canonical form
    *
