@@ -38,11 +38,6 @@ public abstract class BaseSingleInputAggregationFunction<I, F extends Comparable
     _expression = expression;
   }
 
-  @Override
-  public String getColumnName() {
-    return getType().getName() + "_" + _expression;
-  }
-
   @Override
   public String getResultColumnName() {
     return getType().getName().toLowerCase() + "(" + _expression + ")";
