@@ -45,11 +45,6 @@ public class CountMVAggregationFunction extends CountAggregationFunction {
     return AggregationFunctionType.COUNTMV;
   }
 
-  @Override
-  public String getColumnName() {
-    return AggregationFunctionType.COUNTMV.getName() + "_" + _expression;
-  }
-
   @Override
   public String getResultColumnName() {
     return AggregationFunctionType.COUNTMV.getName().toLowerCase() + "(" + _expression + ")";
