@@ -25,7 +25,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import org.apache.commons.lang3.StringUtils;
-import org.apache.datasketches.Util;
 import org.apache.datasketches.memory.Memory;
 import org.apache.datasketches.theta.AnotB;
 import org.apache.datasketches.theta.Intersection;
@@ -34,6 +33,7 @@ import org.apache.datasketches.theta.Sketch;
 import org.apache.datasketches.theta.Union;
 import org.apache.datasketches.theta.UpdateSketch;
 import org.apache.datasketches.theta.UpdateSketchBuilder;
+import org.apache.datasketches.thetacommon.ThetaUtil;
 import org.apache.pinot.common.request.Expression;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FilterContext;
@@ -101,7 +101,7 @@ public class DistinctCountThetaSketchAggregationFunction
       ExpressionContext paramsExpression = arguments.get(1);
       Preconditions.checkArgument(paramsExpression.getType() == ExpressionContext.Type.LITERAL,
           "Second argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal (parameters)");
-      Parameters parameters = new Parameters(paramsExpression.getLiteralString());
+      Parameters parameters = new Parameters(paramsExpression.getLiteral().getStringValue());
       int nominalEntries = parameters.getNominalEntries();
       _updateSketchBuilder.setNominalEntries(nominalEntries);
       _setOperationBuilder.setNominalEntries(nominalEntries);
@@ -130,8 +130,8 @@ public class DistinctCountThetaSketchAggregationFunction
         Preconditions.checkArgument(filterExpression.getType() == ExpressionContext.Type.LITERAL,
             "Third to second last argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal "
                 + "(filter expression)");
-        FilterContext filter =
-            RequestContextUtils.getFilter(CalciteSqlParser.compileToExpression(filterExpression.getLiteralString()));
+        FilterContext filter = RequestContextUtils.getFilter(
+            CalciteSqlParser.compileToExpression(filterExpression.getLiteral().getStringValue()));
         // NOTE: Collect expressions before constructing the FilterInfo so that expressionIndexMap always include the
         //       expressions in the filter.
         collectExpressions(filter, _inputExpressions, expressionIndexMap);
@@ -143,7 +143,7 @@ public class DistinctCountThetaSketchAggregationFunction
       Preconditions.checkArgument(postAggregationExpression.getType() == ExpressionContext.Type.LITERAL,
           "Last argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal (post-aggregation "
               + "expression)");
-      Expression expr = CalciteSqlParser.compileToExpression(postAggregationExpression.getLiteralString());
+      Expression expr = CalciteSqlParser.compileToExpression(postAggregationExpression.getLiteral().getStringValue());
       _postAggregationExpression = RequestContextUtils.getExpression(expr);
 
 
@@ -406,7 +406,7 @@ public class DistinctCountThetaSketchAggregationFunction
       if (_includeDefaultSketch) {
         Union defaultUnion = unions.get(0);
         for (Sketch sketch : sketches) {
-          defaultUnion.update(sketch);
+          defaultUnion.union(sketch);
         }
       }
       for (int i = 0; i < numFilters; i++) {
@@ -414,7 +414,7 @@ public class DistinctCountThetaSketchAggregationFunction
         Union union = unions.get(i + 1);
         for (int j = 0; j < length; j++) {
           if (filterEvaluator.evaluate(singleValues, valueTypes, valueArrays, j)) {
-            union.update(sketches[j]);
+            union.union(sketches[j]);
           }
         }
       }
@@ -634,11 +634,11 @@ public class DistinctCountThetaSketchAggregationFunction
         List<Union> unions = getUnions(groupByResultHolder, groupKeyArray[i]);
         Sketch sketch = sketches[i];
         if (_includeDefaultSketch) {
-          unions.get(0).update(sketch);
+          unions.get(0).union(sketch);
         }
         for (int j = 0; j < numFilters; j++) {
           if (_filterEvaluators.get(j).evaluate(singleValues, valueTypes, valueArrays, i)) {
-            unions.get(j + 1).update(sketch);
+            unions.get(j + 1).union(sketch);
           }
         }
       }
@@ -907,7 +907,7 @@ public class DistinctCountThetaSketchAggregationFunction
       if (_includeDefaultSketch) {
         for (int i = 0; i < length; i++) {
           for (int groupKey : groupKeysArray[i]) {
-            getUnions(groupByResultHolder, groupKey).get(0).update(sketches[i]);
+            getUnions(groupByResultHolder, groupKey).get(0).union(sketches[i]);
           }
         }
       }
@@ -916,7 +916,7 @@ public class DistinctCountThetaSketchAggregationFunction
         for (int j = 0; j < length; j++) {
           if (filterEvaluator.evaluate(singleValues, valueTypes, valueArrays, j)) {
             for (int groupKey : groupKeysArray[i]) {
-              getUnions(groupByResultHolder, groupKey).get(i + 1).update(sketches[i]);
+              getUnions(groupByResultHolder, groupKey).get(i + 1).union(sketches[i]);
             }
           }
         }
@@ -969,8 +969,8 @@ public class DistinctCountThetaSketchAggregationFunction
         continue;
       }
       Union union = _setOperationBuilder.buildUnion();
-      union.update(sketch1);
-      union.update(sketch2);
+      union.union(sketch1);
+      union.union(sketch2);
       // NOTE: Compact the sketch in unsorted, on-heap fashion for performance concern.
       //       See https://datasketches.apache.org/docs/Theta/ThetaSize.html for more details.
       mergedSketches.add(union.getResult(false, null));
@@ -1278,20 +1278,20 @@ public class DistinctCountThetaSketchAggregationFunction
       case SET_UNION:
         Union union = _setOperationBuilder.buildUnion();
         for (ExpressionContext argument : arguments) {
-          union.update(evaluatePostAggregationExpression(argument, sketches));
+          union.union(evaluatePostAggregationExpression(argument, sketches));
         }
         return union.getResult(false, null);
       case SET_INTERSECT:
         Intersection intersection = _setOperationBuilder.buildIntersection();
         for (ExpressionContext argument : arguments) {
-          intersection.update(evaluatePostAggregationExpression(argument, sketches));
+          intersection.intersect(evaluatePostAggregationExpression(argument, sketches));
         }
         return intersection.getResult(false, null);
       case SET_DIFF:
         AnotB diff = _setOperationBuilder.buildANotB();
-        diff.update(evaluatePostAggregationExpression(arguments.get(0), sketches),
-            evaluatePostAggregationExpression(arguments.get(1), sketches));
-        return diff.getResult(false, null);
+        diff.setA(evaluatePostAggregationExpression(arguments.get(0), sketches));
+        diff.notB(evaluatePostAggregationExpression(arguments.get(1), sketches));
+        return diff.getResult(false, null, false);
       default:
         throw new IllegalStateException();
     }
@@ -1305,7 +1305,7 @@ public class DistinctCountThetaSketchAggregationFunction
     private static final char PARAMETER_KEY_VALUE_SEPARATOR = '=';
     private static final String NOMINAL_ENTRIES_KEY = "nominalEntries";
 
-    private int _nominalEntries = Util.DEFAULT_NOMINAL_ENTRIES;
+    private int _nominalEntries = ThetaUtil.DEFAULT_NOMINAL_ENTRIES;
 
     Parameters(String parametersString) {
       StringUtils.deleteWhitespace(parametersString);
