@@ -24,7 +24,7 @@ import java.util.List;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.OrderByExpressionContext;
 import org.apache.pinot.core.common.BlockValSet;
-import org.apache.pinot.core.operator.blocks.TransformBlock;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.core.query.distinct.DistinctExecutorUtils;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
@@ -67,13 +67,13 @@ public class DictionaryBasedMultiColumnDistinctOrderByExecutor extends BaseDicti
   }
 
   @Override
-  public boolean process(TransformBlock transformBlock) {
-    int numDocs = transformBlock.getNumDocs();
+  public boolean process(ValueBlock valueBlock) {
+    int numDocs = valueBlock.getNumDocs();
     int numExpressions = _expressions.size();
     if (!_hasMVExpression) {
       int[][] dictIdsArray = new int[numDocs][numExpressions];
       for (int i = 0; i < numExpressions; i++) {
-        BlockValSet blockValueSet = transformBlock.getBlockValueSet(_expressions.get(i));
+        BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expressions.get(i));
         int[] dictIdsForExpression = blockValueSet.getDictionaryIdsSV();
         for (int j = 0; j < numDocs; j++) {
           dictIdsArray[j][i] = dictIdsForExpression[j];
@@ -86,7 +86,7 @@ public class DictionaryBasedMultiColumnDistinctOrderByExecutor extends BaseDicti
       int[][] svDictIds = new int[numExpressions][];
       int[][][] mvDictIds = new int[numExpressions][][];
       for (int i = 0; i < numExpressions; i++) {
-        BlockValSet blockValueSet = transformBlock.getBlockValueSet(_expressions.get(i));
+        BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expressions.get(i));
         if (blockValueSet.isSingleValue()) {
           svDictIds[i] = blockValueSet.getDictionaryIdsSV();
         } else {
