@@ -26,10 +26,13 @@ import java.util.List;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
+import org.apache.pinot.core.common.BlockValSet;
 import org.apache.pinot.core.data.table.Record;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.core.query.distinct.DistinctTable;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
+import org.roaringbitmap.RoaringBitmap;
 
 
 /**
@@ -59,7 +62,7 @@ abstract class BaseRawIntSingleColumnDistinctExecutor implements DistinctExecuto
   public DistinctTable getResult() {
     DataSchema dataSchema = new DataSchema(new String[]{_expression.toString()},
         new ColumnDataType[]{ColumnDataType.fromDataTypeSV(_dataType)});
-    List<Record> records = new ArrayList<>(_valueSet.size());
+    List<Record> records = new ArrayList<>(_valueSet.size() + (_hasNull ? 1 : 0));
     IntIterator valueIterator = _valueSet.iterator();
     while (valueIterator.hasNext()) {
       records.add(new Record(new Object[]{valueIterator.nextInt()}));
@@ -67,7 +70,44 @@ abstract class BaseRawIntSingleColumnDistinctExecutor implements DistinctExecuto
     if (_hasNull) {
       records.add(new Record(new Object[]{null}));
     }
-    assert records.size() <= _limit;
+    assert records.size() - (_hasNull ? 1 : 0) <= _limit;
     return new DistinctTable(dataSchema, records, _nullHandlingEnabled);
   }
+
+  @Override
+  public boolean process(ValueBlock valueBlock) {
+    BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expression);
+    int numDocs = valueBlock.getNumDocs();
+    if (blockValueSet.isSingleValue()) {
+      int[] values = blockValueSet.getIntValuesSV();
+      if (_nullHandlingEnabled) {
+        RoaringBitmap nullBitmap = blockValueSet.getNullBitmap();
+        for (int i = 0; i < numDocs; i++) {
+          if (nullBitmap != null && nullBitmap.contains(i)) {
+            _hasNull = true;
+          } else if (add(values[i])) {
+            return true;
+          }
+        }
+      } else {
+        for (int i = 0; i < numDocs; i++) {
+          if (add(values[i])) {
+            return true;
+          }
+        }
+      }
+    } else {
+      int[][] values = blockValueSet.getIntValuesMV();
+      for (int i = 0; i < numDocs; i++) {
+        for (int value : values[i]) {
+          if (add(value)) {
+            return true;
+          }
+        }
+      }
+    }
+    return false;
+  }
+
+  protected abstract boolean add(int val);
 }
