@@ -118,6 +118,9 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
 
   @Override
   public void submit(ServerRequest request, StreamObserver<ServerResponse> responseObserver) {
+    _serverMetrics.addMeteredGlobalValue(ServerMeter.GRPC_QUERIES, 1);
+    _serverMetrics.addMeteredGlobalValue(ServerMeter.GRPC_BYTES_RECEIVED, request.getSerializedSize());
+
     // Deserialize the request
     ServerQueryRequest queryRequest;
     try {
@@ -146,7 +149,8 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
     // Process the query
     InstanceResponseBlock instanceResponse;
     try {
-      instanceResponse = _queryExecutor.execute(queryRequest, _executorService, responseObserver);
+      instanceResponse =
+          _queryExecutor.execute(queryRequest, _executorService, new GrpcResultsBlockStreamer(responseObserver));
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing request {}: {} from broker: {}", queryRequest.getRequestId(),
           queryRequest.getQueryContext(), queryRequest.getBrokerId(), e);
@@ -168,6 +172,7 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
       return;
     }
     responseObserver.onNext(serverResponse);
+    _serverMetrics.addMeteredGlobalValue(ServerMeter.GRPC_BYTES_SENT, serverResponse.getSerializedSize());
     responseObserver.onCompleted();
   }
 }
