@@ -191,7 +191,6 @@ public class BaseTableDataManagerTest {
       fail();
     } catch (Exception e) {
       // As expected, segment reloading fails due to missing the local segment dir.
-      assertTrue(e.getMessage().contains("does not exist or is not a directory"));
     }
   }
 
@@ -312,7 +311,6 @@ public class BaseTableDataManagerTest {
       fail();
     } catch (Exception e) {
       // As expected, segment reloading fails due to missing the local segment dir.
-      assertTrue(e.getMessage().contains("does not exist or is not a directory"));
     }
 
     tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, true);
@@ -721,7 +719,7 @@ public class BaseTableDataManagerTest {
 
     OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
     tableDataManager.init(config, "dummyInstance", mock(ZkHelixPropertyStore.class),
-        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), mock(HelixManager.class), null,
+        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), mock(HelixManager.class), null, null,
         new TableDataManagerParams(0, false, -1));
     tableDataManager.start();
     return tableDataManager;
@@ -730,7 +728,7 @@ public class BaseTableDataManagerTest {
   private static BaseTableDataManager createTableManager(TableDataManagerConfig config, HelixManager helixManager) {
     OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
     tableDataManager.init(config, "dummyInstance", mock(ZkHelixPropertyStore.class),
-        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), helixManager, null,
+        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), helixManager, null, null,
         new TableDataManagerParams(0, false, -1));
     tableDataManager.start();
     return tableDataManager;
@@ -739,7 +737,7 @@ public class BaseTableDataManagerTest {
   private static OfflineTableDataManager createSpyOfflineTableManager(TableDataManagerConfig tableDataManagerConfig) {
     OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
     tableDataManager.init(tableDataManagerConfig, "dummyInstance", mock(ZkHelixPropertyStore.class),
-        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), mock(HelixManager.class), null,
+        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), mock(HelixManager.class), null, null,
         new TableDataManagerParams(0, false, -1));
     tableDataManager.start();
     return Mockito.spy(tableDataManager);
