@@ -18,7 +18,10 @@
  */
 package org.apache.pinot.core.operator.filter;
 
+import com.google.common.collect.ImmutableList;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import org.apache.pinot.core.common.BlockDocIdIterator;
 import org.apache.pinot.segment.spi.Constants;
@@ -31,13 +34,14 @@ public class AndFilterOperatorTest {
 
   @Test
   public void testIntersectionForTwoLists() {
+    int numDocs = 40;
     int[] docIds1 = new int[]{2, 3, 10, 15, 16, 28};
     int[] docIds2 = new int[]{3, 6, 8, 20, 28};
 
     List<BaseFilterOperator> operators = new ArrayList<>();
-    operators.add(new TestFilterOperator(docIds1));
-    operators.add(new TestFilterOperator(docIds2));
-    AndFilterOperator andOperator = new AndFilterOperator(operators);
+    operators.add(new TestFilterOperator(docIds1, numDocs));
+    operators.add(new TestFilterOperator(docIds2, numDocs));
+    AndFilterOperator andOperator = new AndFilterOperator(operators, null, numDocs, false);
 
     BlockDocIdIterator iterator = andOperator.nextBlock().getBlockDocIdSet().iterator();
     Assert.assertEquals(iterator.next(), 3);
@@ -47,15 +51,16 @@ public class AndFilterOperatorTest {
 
   @Test
   public void testIntersectionForThreeLists() {
+    int numDocs = 40;
     int[] docIds1 = new int[]{2, 3, 6, 10, 15, 16, 28};
     int[] docIds2 = new int[]{3, 6, 8, 20, 28};
     int[] docIds3 = new int[]{1, 2, 3, 6, 30};
 
     List<BaseFilterOperator> operators = new ArrayList<>();
-    operators.add(new TestFilterOperator(docIds1));
-    operators.add(new TestFilterOperator(docIds2));
-    operators.add(new TestFilterOperator(docIds3));
-    AndFilterOperator andOperator = new AndFilterOperator(operators);
+    operators.add(new TestFilterOperator(docIds1, numDocs));
+    operators.add(new TestFilterOperator(docIds2, numDocs));
+    operators.add(new TestFilterOperator(docIds3, numDocs));
+    AndFilterOperator andOperator = new AndFilterOperator(operators, null, numDocs, false);
 
     BlockDocIdIterator iterator = andOperator.nextBlock().getBlockDocIdSet().iterator();
     Assert.assertEquals(iterator.next(), 3);
@@ -65,19 +70,20 @@ public class AndFilterOperatorTest {
 
   @Test
   public void testComplex() {
+    int numDocs = 40;
     int[] docIds1 = new int[]{2, 3, 6, 10, 15, 16, 28};
     int[] docIds2 = new int[]{3, 6, 8, 20, 28};
     int[] docIds3 = new int[]{1, 2, 3, 6, 30};
 
     List<BaseFilterOperator> childOperators = new ArrayList<>();
-    childOperators.add(new TestFilterOperator(docIds1));
-    childOperators.add(new TestFilterOperator(docIds2));
-    AndFilterOperator childAndOperator = new AndFilterOperator(childOperators);
+    childOperators.add(new TestFilterOperator(docIds1, numDocs));
+    childOperators.add(new TestFilterOperator(docIds2, numDocs));
+    AndFilterOperator childAndOperator = new AndFilterOperator(childOperators, null, numDocs, false);
 
     List<BaseFilterOperator> operators = new ArrayList<>();
     operators.add(childAndOperator);
-    operators.add(new TestFilterOperator(docIds3));
-    AndFilterOperator andOperator = new AndFilterOperator(operators);
+    operators.add(new TestFilterOperator(docIds3, numDocs));
+    AndFilterOperator andOperator = new AndFilterOperator(operators, null, numDocs, false);
 
     BlockDocIdIterator iterator = andOperator.nextBlock().getBlockDocIdSet().iterator();
     Assert.assertEquals(iterator.next(), 3);
@@ -112,8 +118,8 @@ public class AndFilterOperatorTest {
               numDocs));
     }
 
-    AndFilterOperator andFilterOperator1 = new AndFilterOperator(childOperators1);
-    AndFilterOperator andFilterOperator2 = new AndFilterOperator(childOperators2);
+    AndFilterOperator andFilterOperator1 = new AndFilterOperator(childOperators1, null, numDocs, false);
+    AndFilterOperator andFilterOperator2 = new AndFilterOperator(childOperators2, null, numDocs, false);
     BlockDocIdIterator iterator1 = andFilterOperator1.getNextBlock().getBlockDocIdSet().iterator();
     BlockDocIdIterator iterator2 = andFilterOperator2.getNextBlock().getBlockDocIdSet().iterator();
     Assert.assertEquals(iterator1.next(), 0);
@@ -133,19 +139,20 @@ public class AndFilterOperatorTest {
 
   @Test
   public void testComplexWithOr() {
+    int numDocs = 40;
     int[] docIds1 = new int[]{2, 3, 6, 10, 15, 16, 28};
     int[] docIds2 = new int[]{3, 6, 8, 20, 28};
     int[] docIds3 = new int[]{1, 2, 3, 6, 30};
 
     List<BaseFilterOperator> childOperators = new ArrayList<>();
-    childOperators.add(new TestFilterOperator(docIds3));
-    childOperators.add(new TestFilterOperator(docIds2));
-    OrFilterOperator childOrOperator = new OrFilterOperator(childOperators, 40);
+    childOperators.add(new TestFilterOperator(docIds3, numDocs));
+    childOperators.add(new TestFilterOperator(docIds2, numDocs));
+    OrFilterOperator childOrOperator = new OrFilterOperator(childOperators, null, numDocs, false);
 
     List<BaseFilterOperator> operators = new ArrayList<>();
     operators.add(childOrOperator);
-    operators.add(new TestFilterOperator(docIds1));
-    AndFilterOperator andOperator = new AndFilterOperator(operators);
+    operators.add(new TestFilterOperator(docIds1, numDocs));
+    AndFilterOperator andOperator = new AndFilterOperator(operators, null, numDocs, false);
 
     BlockDocIdIterator iterator = andOperator.nextBlock().getBlockDocIdSet().iterator();
     Assert.assertEquals(iterator.next(), 2);
@@ -154,4 +161,35 @@ public class AndFilterOperatorTest {
     Assert.assertEquals(iterator.next(), 28);
     Assert.assertEquals(iterator.next(), Constants.EOF);
   }
+
+  @Test
+  public void testAndWithNull() {
+    int numDocs = 10;
+    int[] docIds1 = new int[]{1, 2, 3};
+    int[] docIds2 = new int[]{0, 1, 2};
+    int[] nullDocIds1 = new int[]{4, 5, 6};
+    int[] nullDocIds2 = new int[]{3, 4, 5, 6, 7};
+
+    AndFilterOperator andFilterOperator = new AndFilterOperator(
+        Arrays.asList(new TestFilterOperator(docIds1, nullDocIds1, numDocs),
+            new TestFilterOperator(docIds2, nullDocIds2, numDocs)), null, numDocs, true);
+
+    Assert.assertEquals(TestUtils.getDocIds(andFilterOperator.getTrues()), ImmutableList.of(1, 2));
+    Assert.assertEquals(TestUtils.getDocIds(andFilterOperator.getFalses()), ImmutableList.of(0, 7, 8, 9));
+  }
+
+  @Test
+  public void testAndWithNullOneFilterIsEmpty() {
+    int numDocs = 10;
+    int[] docIds1 = new int[]{1, 2, 3};
+    int[] nullDocIds1 = new int[]{4, 5, 6};
+
+    AndFilterOperator andFilterOperator = new AndFilterOperator(
+        Arrays.asList(new TestFilterOperator(docIds1, nullDocIds1, numDocs), EmptyFilterOperator.getInstance()), null,
+        numDocs, true);
+
+    Assert.assertEquals(TestUtils.getDocIds(andFilterOperator.getTrues()), Collections.emptyList());
+    Assert.assertEquals(TestUtils.getDocIds(andFilterOperator.getFalses()),
+        ImmutableList.of(0, 1, 2, 3, 4, 5, 6, 7, 8, 9));
+  }
 }
