@@ -24,6 +24,7 @@ import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.RequestContextUtils;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.exception.BadQueryRequestException;
+import org.roaringbitmap.RoaringBitmap;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
@@ -35,37 +36,36 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @DataProvider
   public static Object[][] rejectedParameters() {
-    return new Object[][] {
-        {"()"},
-        {String.format("(%s)", INT_MV_COLUMN)},
-        {String.format("(%s)", LONG_MV_COLUMN)},
-        {String.format("(%s)", FLOAT_MV_COLUMN)},
-        {String.format("(%s)", DOUBLE_MV_COLUMN)},
-        {String.format("(%s)", STRING_MV_COLUMN)},
-        {String.format("(%s, %s)", INT_MV_COLUMN, INT_SV_COLUMN)},
-        {String.format("(%s, %s)", STRING_SV_COLUMN, INT_SV_COLUMN)},
-        {String.format("(%s, %s)", STRING_SV_COLUMN, LONG_SV_COLUMN)},
-        {String.format("(%s, %s)", STRING_SV_COLUMN, FLOAT_SV_COLUMN)},
-        {String.format("(%s, %s)", STRING_SV_COLUMN, DOUBLE_SV_COLUMN)},
-        {String.format("(%s, %s)", STRING_SV_COLUMN, TIMESTAMP_COLUMN)},
-        {String.format("(%s, %s)", INT_SV_COLUMN, TIMESTAMP_COLUMN)},
-        {String.format("(%s, %s)", FLOAT_SV_COLUMN, TIMESTAMP_COLUMN)},
-        {String.format("(%s, %s)", DOUBLE_SV_COLUMN, TIMESTAMP_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, INT_SV_COLUMN)},
-        {String.format("(%s, %s)", INT_SV_COLUMN, INT_MV_COLUMN)},
-        {String.format("(%s, %s)", INT_SV_COLUMN, STRING_SV_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, INT_SV_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, FLOAT_SV_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, DOUBLE_SV_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, STRING_SV_COLUMN)},
-        {String.format("(%s, %s)", TIMESTAMP_COLUMN, TIME_COLUMN)}
+    return new Object[][]{
+        {"()"}, {String.format("(%s)", INT_MV_COLUMN)}, {String.format("(%s)", LONG_MV_COLUMN)}, {
+        String.format("(%s)", FLOAT_MV_COLUMN)
+    }, {String.format("(%s)", DOUBLE_MV_COLUMN)}, {String.format("(%s)", STRING_MV_COLUMN)}, {
+        String.format("(%s, %s)", INT_MV_COLUMN, INT_SV_COLUMN)
+    }, {String.format("(%s, %s)", STRING_SV_COLUMN, INT_SV_COLUMN)}, {
+        String.format("(%s, %s)", STRING_SV_COLUMN, LONG_SV_COLUMN)
+    }, {String.format("(%s, %s)", STRING_SV_COLUMN, FLOAT_SV_COLUMN)}, {
+        String.format("(%s, %s)", STRING_SV_COLUMN, DOUBLE_SV_COLUMN)
+    }, {String.format("(%s, %s)", STRING_SV_COLUMN, TIMESTAMP_COLUMN)}, {
+        String.format("(%s, %s)", INT_SV_COLUMN, TIMESTAMP_COLUMN)
+    }, {String.format("(%s, %s)", FLOAT_SV_COLUMN, TIMESTAMP_COLUMN)}, {
+        String.format("(%s, %s)", DOUBLE_SV_COLUMN, TIMESTAMP_COLUMN)
+    }, {String.format("(%s, %s)", TIMESTAMP_COLUMN, INT_SV_COLUMN)}, {
+        String.format("(%s, %s)", INT_SV_COLUMN, INT_MV_COLUMN)
+    }, {String.format("(%s, %s)", INT_SV_COLUMN, STRING_SV_COLUMN)}, {
+        String.format("(%s, %s)", TIMESTAMP_COLUMN, INT_SV_COLUMN)
+    }, {String.format("(%s, %s)", TIMESTAMP_COLUMN, FLOAT_SV_COLUMN)}, {
+        String.format("(%s, %s)", TIMESTAMP_COLUMN, DOUBLE_SV_COLUMN)
+    }, {String.format("(%s, %s)", TIMESTAMP_COLUMN, STRING_SV_COLUMN)}, {
+        String.format("(%s, %s)", TIMESTAMP_COLUMN, TIME_COLUMN)
+    }
     };
   }
 
   @Test
   public void testLeastTransformFunctionInt() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, %d, cast(%s as INT))", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
+    // -1 will be passed in as a long.
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, %d, cast(%s as INT))", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.INT);
     int[] intValues = transformFunction.transformToIntValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -88,8 +88,7 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionUnaryInt() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s)", INT_SV_COLUMN));
+    TransformFunction transformFunction = testLeastPreconditions(String.format("least(%s)", INT_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.INT);
     int[] intValues = transformFunction.transformToIntValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -99,8 +98,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionIntLong() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, LONG_SV_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, LONG_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.LONG);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -110,8 +109,7 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionUnaryLong() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s)", LONG_SV_COLUMN));
+    TransformFunction transformFunction = testLeastPreconditions(String.format("least(%s)", LONG_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.LONG);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -121,8 +119,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionIntFloat() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -132,8 +130,7 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionUnaryFloat() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s)", FLOAT_SV_COLUMN));
+    TransformFunction transformFunction = testLeastPreconditions(String.format("least(%s)", FLOAT_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.FLOAT);
     float[] floatValues = transformFunction.transformToFloatValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -143,8 +140,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionIntDouble() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, DOUBLE_SV_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, %d, %s)", INT_SV_COLUMN, -1, DOUBLE_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -154,8 +151,7 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionUnaryDouble() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s)", DOUBLE_SV_COLUMN));
+    TransformFunction transformFunction = testLeastPreconditions(String.format("least(%s)", DOUBLE_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -177,8 +173,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionNumericTypes2() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, %s, %s)", INT_SV_COLUMN, FLOAT_SV_COLUMN, LONG_SV_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, %s, %s)", INT_SV_COLUMN, FLOAT_SV_COLUMN, LONG_SV_COLUMN));
     // Note: In the current code, least return type is DOUBLE if there is a float input, but output values are float!
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     float[] floatValues = transformFunction.transformToFloatValuesSV(_projectionBlock);
@@ -189,8 +185,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionTime() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, cast(%s AS TIMESTAMP))", TIMESTAMP_COLUMN, LONG_SV_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, cast(%s AS TIMESTAMP))", TIMESTAMP_COLUMN, LONG_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.TIMESTAMP);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -210,8 +206,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testLeastTransformFunctionTimestampWithLegacyTimeColumn() {
-    TransformFunction transformFunction = testLeastPreconditions(
-        String.format("least(%s, cast(%s as TIMESTAMP))", TIMESTAMP_COLUMN, TIME_COLUMN));
+    TransformFunction transformFunction =
+        testLeastPreconditions(String.format("least(%s, cast(%s as TIMESTAMP))", TIMESTAMP_COLUMN, TIME_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.TIMESTAMP);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -219,6 +215,60 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
     }
   }
 
+  @Test
+  public void testLeastTransformFunctionNullLiteral() {
+    TransformFunction transformFunction = testLeastPreconditionsNullHandlingEnabled(
+        String.format("least(%s, null, %s)", INT_SV_COLUMN, DOUBLE_SV_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
+    RoaringBitmap nullBitmap = new RoaringBitmap();
+    nullBitmap.add(0, (long) NUM_ROWS);
+
+    testNullBitmap(transformFunction, nullBitmap);
+  }
+
+  @Test
+  public void testLeastTransformFunctionNullColumn() {
+    TransformFunction transformFunction = testLeastPreconditionsNullHandlingEnabled(
+        String.format("least(%s, %s)", INT_SV_NULL_COLUMN, DOUBLE_SV_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
+    double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
+    RoaringBitmap nullBitmap = transformFunction.getNullBitmap(_projectionBlock);
+    for (int i = 0; i < NUM_ROWS; i++) {
+      if (isNullRow(i)) {
+        assertTrue(nullBitmap.contains(i));
+      } else {
+        assertEquals(doubleValues[i], Math.min(_intSVValues[i], _doubleSVValues[i]));
+      }
+    }
+  }
+
+  @Test
+  public void testLeastTransformFunctionAllNulls() {
+    TransformFunction transformFunction =
+        testLeastPreconditionsNullHandlingEnabled(String.format("least(null, null, null)"));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.UNKNOWN);
+    RoaringBitmap expectedNull = new RoaringBitmap();
+    expectedNull.add(0L, NUM_ROWS);
+    assertEquals(transformFunction.getNullBitmap(_projectionBlock), expectedNull);
+  }
+
+  @Test
+  public void testLeastTransformFunctionPartialAllNulls() {
+    TransformFunction transformFunction = testLeastPreconditionsNullHandlingEnabled(
+        String.format("least(%s, %s, %s)", INT_SV_NULL_COLUMN, INT_SV_NULL_COLUMN, INT_SV_NULL_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.INT);
+   int[] intValues = transformFunction.transformToIntValuesSV(_projectionBlock);
+    RoaringBitmap expectedNull = new RoaringBitmap();
+    for (int i = 0; i < NUM_ROWS; i++) {
+      if (isNullRow(i)) {
+        expectedNull.add(i);
+      } else {
+        assertEquals(intValues[i], _intSVValues[i]);
+      }
+    }
+    testNullBitmap(transformFunction, expectedNull);
+  }
+
   @Test(dataProvider = "rejectedParameters", expectedExceptions = BadQueryRequestException.class)
   public void testRejectLeast(String params) {
     testGreatestPreconditions("least" + params);
@@ -247,8 +297,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionIntLong() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, LONG_SV_COLUMN));
+    TransformFunction transformFunction =
+        testGreatestPreconditions(String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, LONG_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.LONG);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -268,8 +318,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionIntFloat() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
+    TransformFunction transformFunction =
+        testGreatestPreconditions(String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, FLOAT_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -299,8 +349,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionIntDouble() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, DOUBLE_SV_COLUMN));
+    TransformFunction transformFunction =
+        testGreatestPreconditions(String.format("greatest(%s, %d, %s)", INT_SV_COLUMN, -1, DOUBLE_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -308,6 +358,61 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
     }
   }
 
+  @Test
+  public void testGreatestTransformFunctionNullLiteral() {
+    TransformFunction transformFunction = testGreatestPreconditionsNullHandlingEnabled(
+        String.format("greatest(%s, null, %s)", INT_SV_COLUMN, DOUBLE_SV_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
+    RoaringBitmap nullBitmap = new RoaringBitmap();
+    nullBitmap.add(0, (long) NUM_ROWS);
+
+    testNullBitmap(transformFunction, nullBitmap);
+  }
+
+  @Test
+  public void testGreatestTransformFunctionNullColumn() {
+    TransformFunction transformFunction = testGreatestPreconditionsNullHandlingEnabled(
+        String.format("greatest(%s, %s)", INT_SV_NULL_COLUMN, DOUBLE_SV_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
+    double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
+    RoaringBitmap nullBitmap = transformFunction.getNullBitmap(_projectionBlock);
+    for (int i = 0; i < NUM_ROWS; i++) {
+      if (isNullRow(i)) {
+        assertTrue(nullBitmap.contains(i));
+      } else {
+        assertEquals(doubleValues[i], Math.max(_intSVValues[i], _doubleSVValues[i]));
+      }
+    }
+  }
+
+  @Test
+  public void testGreatestTransformFunctionAllNulls() {
+    TransformFunction transformFunction =
+        testGreatestPreconditionsNullHandlingEnabled(String.format("greatest(null, null, null)"));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.UNKNOWN);
+    double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
+    RoaringBitmap expectedNull = new RoaringBitmap();
+    expectedNull.add(0L, NUM_ROWS);
+    testNullBitmap(transformFunction, expectedNull);
+  }
+
+  @Test
+  public void testGreatestTransformFunctionPartialAllNulls() {
+    TransformFunction transformFunction = testGreatestPreconditionsNullHandlingEnabled(
+        String.format("greatest(%s, %s, %s)", INT_SV_NULL_COLUMN, INT_SV_NULL_COLUMN, INT_SV_NULL_COLUMN));
+    assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.INT);
+    int[] intValues = transformFunction.transformToIntValuesSV(_projectionBlock);
+    RoaringBitmap expectedNull = new RoaringBitmap();
+    for (int i = 0; i < NUM_ROWS; i++) {
+      if (isNullRow(i)) {
+        expectedNull.add(i);
+      } else {
+        assertEquals(intValues[i], _intSVValues[i]);
+      }
+    }
+    testNullBitmap(transformFunction, expectedNull);
+  }
+
   @Test
   public void testGreatestTransformFunctionString() {
     TransformFunction transformFunction = testGreatestPreconditions(
@@ -323,8 +428,7 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionUnaryDouble() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s)", DOUBLE_SV_COLUMN));
+    TransformFunction transformFunction = testGreatestPreconditions(String.format("greatest(%s)", DOUBLE_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.DOUBLE);
     double[] doubleValues = transformFunction.transformToDoubleValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -334,8 +438,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionUnaryBigDecimal() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s)", BIG_DECIMAL_SV_COLUMN));
+    TransformFunction transformFunction =
+        testGreatestPreconditions(String.format("greatest(%s)", BIG_DECIMAL_SV_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.BIG_DECIMAL);
     BigDecimal[] bigDecimalValues = transformFunction.transformToBigDecimalValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -368,8 +472,8 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
 
   @Test
   public void testGreatestTransformFunctionTimestampWithLegacyTimeColumn() {
-    TransformFunction transformFunction = testGreatestPreconditions(
-        String.format("greatest(%s, cast(%s as TIMESTAMP))", TIMESTAMP_COLUMN, TIME_COLUMN));
+    TransformFunction transformFunction =
+        testGreatestPreconditions(String.format("greatest(%s, cast(%s as TIMESTAMP))", TIMESTAMP_COLUMN, TIME_COLUMN));
     assertEquals(transformFunction.getResultMetadata().getDataType(), FieldSpec.DataType.TIMESTAMP);
     long[] longValues = transformFunction.transformToLongValuesSV(_projectionBlock);
     for (int i = 0; i < NUM_ROWS; i++) {
@@ -397,4 +501,20 @@ public class TupleSelectionTransformFunctionsTest extends BaseTransformFunctionT
     assertEquals(transformFunction.getName(), TransformFunctionType.GREATEST.getName());
     return transformFunction;
   }
+
+  private TransformFunction testLeastPreconditionsNullHandlingEnabled(String expressionStr) {
+    ExpressionContext expression = RequestContextUtils.getExpression(expressionStr);
+    TransformFunction transformFunction = TransformFunctionFactory.getNullHandlingEnabled(expression, _dataSourceMap);
+    assertTrue(transformFunction instanceof LeastTransformFunction);
+    assertEquals(transformFunction.getName(), TransformFunctionType.LEAST.getName());
+    return transformFunction;
+  }
+
+  private TransformFunction testGreatestPreconditionsNullHandlingEnabled(String expressionStr) {
+    ExpressionContext expression = RequestContextUtils.getExpression(expressionStr);
+    TransformFunction transformFunction = TransformFunctionFactory.getNullHandlingEnabled(expression, _dataSourceMap);
+    assertTrue(transformFunction instanceof GreatestTransformFunction);
+    assertEquals(transformFunction.getName(), TransformFunctionType.GREATEST.getName());
+    return transformFunction;
+  }
 }
