@@ -18,30 +18,21 @@
  */
 package org.apache.pinot.core.query.pruner;
 
-import com.google.common.base.Charsets;
 import com.google.common.collect.ImmutableSet;
-import com.google.common.hash.BloomFilter;
-import com.google.common.hash.Funnels;
-import java.io.File;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.nio.file.Files;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.query.request.context.utils.QueryContextConverterUtils;
-import org.apache.pinot.segment.local.segment.index.readers.bloom.OnHeapGuavaBloomFilterReader;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.SegmentMetadata;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.segment.spi.datasource.DataSourceMetadata;
-import org.apache.pinot.segment.spi.index.reader.BloomFilterReader;
-import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
 import org.apache.pinot.segment.spi.partition.PartitionFunctionFactory;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.env.PinotConfiguration;
+import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
 import static org.mockito.Mockito.mock;
@@ -53,14 +44,17 @@ import static org.testng.Assert.assertTrue;
 public class ColumnValueSegmentPrunerTest {
   private static final ColumnValueSegmentPruner PRUNER = new ColumnValueSegmentPruner();
 
-  @Test
-  public void testMinMaxValuePruning() {
+  @BeforeClass
+  public void setUp() {
     Map<String, Object> properties = new HashMap<>();
-    //override default value
-    properties.put(PRUNER.IN_PREDICATE_THRESHOLD, 5);
+    // override default value
+    properties.put(ColumnValueSegmentPruner.IN_PREDICATE_THRESHOLD, 5);
     PinotConfiguration configuration = new PinotConfiguration(properties);
     PRUNER.init(configuration);
+  }
 
+  @Test
+  public void testMinMaxValuePruning() {
     IndexSegment indexSegment = mockIndexSegment();
 
     DataSource dataSource = mock(DataSource.class);
@@ -147,66 +141,42 @@ public class ColumnValueSegmentPrunerTest {
   }
 
   @Test
-  public void testBloomFilterPredicatePruning()
-      throws IOException {
-    Map<String, Object> properties = new HashMap<>();
-    // override default value
-    properties.put(ColumnValueSegmentPruner.IN_PREDICATE_THRESHOLD, 5);
-    PinotConfiguration configuration = new PinotConfiguration(properties);
-    PRUNER.init(configuration);
+  public void testIsApplicableTo() {
+    // EQ, RANGE and IN (with small number of values) are applicable for min/max/partitionId based pruning.
+    QueryContext queryContext =
+        QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable WHERE column = 1");
+    assertTrue(PRUNER.isApplicableTo(queryContext));
+    queryContext = QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable WHERE column IN (1, 2)");
+    assertTrue(PRUNER.isApplicableTo(queryContext));
+    queryContext =
+        QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable WHERE column BETWEEN 1 AND 2");
+    assertTrue(PRUNER.isApplicableTo(queryContext));
 
-    IndexSegment indexSegment = mockIndexSegment();
-    DataSource dataSource = mock(DataSource.class);
-    when(indexSegment.getDataSource("column")).thenReturn(dataSource);
-    // Add support for bloom filter
-    DataSourceMetadata dataSourceMetadata = mock(DataSourceMetadata.class);
-    BloomFilterReader bloomFilterReader = new BloomFilterReaderBuilder()
-        .put("1.0")
-        .put("2.0")
-        .put("3.0")
-        .put("5.0")
-        .put("7.0")
-        .put("21.0")
-        .build();
+    // NOT is not applicable
+    queryContext = QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable WHERE NOT column = 1");
+    assertFalse(PRUNER.isApplicableTo(queryContext));
+    // Too many values for IN clause
+    queryContext = QueryContextConverterUtils.getQueryContext(
+        "SELECT COUNT(*) FROM testTable WHERE column IN (1, 2, 3, 4, 5, 6, 7)");
+    assertFalse(PRUNER.isApplicableTo(queryContext));
+    // Other predicate types are not applicable
+    queryContext = QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable WHERE column LIKE 5");
+    assertFalse(PRUNER.isApplicableTo(queryContext));
 
-    when(dataSourceMetadata.getDataType()).thenReturn(DataType.DOUBLE);
-    when(dataSource.getDataSourceMetadata()).thenReturn(dataSourceMetadata);
-    when(dataSource.getBloomFilter()).thenReturn(bloomFilterReader);
-    when(dataSourceMetadata.getMinValue()).thenReturn(5.0);
-    when(dataSourceMetadata.getMaxValue()).thenReturn(10.0);
+    // AND with one applicable child filter is applicable
+    queryContext = QueryContextConverterUtils.getQueryContext(
+        "SELECT COUNT(*) FROM testTable WHERE column NOT IN (1, 2) AND column = 3");
+    assertTrue(PRUNER.isApplicableTo(queryContext));
 
-    // all out the bloom filter and out of range
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (0.0)"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 0.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (0.0, 3.0, 4.0)"));
+    // OR with one child filter that's not applicable is not applicable
+    queryContext = QueryContextConverterUtils.getQueryContext(
+        "SELECT COUNT(*) FROM testTable WHERE column = 3 OR column NOT IN (1, 2)");
+    assertFalse(PRUNER.isApplicableTo(queryContext));
 
-    // some in the bloom filter but all out of range
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 1.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (1.0)"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 21.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (21.0)"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (21.0, 30.0)"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 21.0 AND column = 30.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 21.0 OR column = 30.0"));
-
-    // all out the bloom filter but some in range
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 6.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (6.0)"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 0.0 OR column = 6.0"));
-
-    // all in the bloom filter and in range
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 5.0"));
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (5.0)"));
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 5.0 OR column = 7.0"));
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (5.0, 7.0)"));
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 5.0 AND column = 7.0"));
-
-    // some in the bloom filter and in range
-    assertFalse(
-        runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (0.0, 5.0)"));
-    assertFalse(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 5.0 OR column = 0.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column = 5.0 AND column = 0.0"));
-    assertTrue(runPruner(indexSegment, "SELECT COUNT(*) FROM testTable WHERE column IN (8.0, 10.0)"));
+    // Nested with AND/OR
+    queryContext = QueryContextConverterUtils.getQueryContext(
+        "SELECT COUNT(*) FROM testTable WHERE column = 3 OR (column NOT IN (1, 2) AND column BETWEEN 4 AND 5)");
+    assertTrue(PRUNER.isApplicableTo(queryContext));
   }
 
   private IndexSegment mockIndexSegment() {
@@ -222,25 +192,4 @@ public class ColumnValueSegmentPrunerTest {
     QueryContext queryContext = QueryContextConverterUtils.getQueryContext(query);
     return PRUNER.prune(Arrays.asList(indexSegment), queryContext).isEmpty();
   }
-
-  private static class BloomFilterReaderBuilder {
-    private BloomFilter<String> _bloomfilter = BloomFilter.create(Funnels.stringFunnel(Charsets.UTF_8), 100, 0.01);
-    public BloomFilterReaderBuilder put(String value) {
-      _bloomfilter.put(value);
-      return this;
-    }
-
-    public BloomFilterReader build() throws IOException {
-      File file = Files.createTempFile("test", ".bloom").toFile();
-      try (FileOutputStream fos = new FileOutputStream(file)) {
-        _bloomfilter.writeTo(fos);
-        try (PinotDataBuffer pinotDataBuffer = PinotDataBuffer.loadBigEndianFile(file)) {
-          // on heap filter should never use the buffer, so we can close it and delete the file
-          return new OnHeapGuavaBloomFilterReader(pinotDataBuffer);
-        }
-      } finally {
-        file.delete();
-      }
-    }
-  }
 }
