@@ -22,7 +22,9 @@ import com.google.common.collect.Lists;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import org.apache.pinot.segment.local.utils.SchemaUtils;
 import org.apache.pinot.spi.config.table.TableConfig;
@@ -136,13 +138,17 @@ public class SchemaUtilsTest {
     schema = new Schema.SchemaBuilder().setSchemaName(TABLE_NAME)
         .addDateTime(TIME_COLUMN, DataType.LONG, "1:MILLISECONDS:EPOCH", "1:HOURS").build();
     tableConfig =
-        new TableConfigBuilder(TableType.REALTIME).setTableName(TABLE_NAME).setTimeColumnName(TIME_COLUMN).build();
+        new TableConfigBuilder(TableType.REALTIME).setTableName(TABLE_NAME)
+            .setStreamConfigs(getStreamConfigs())
+            .setTimeColumnName(TIME_COLUMN).build();
     SchemaUtils.validate(schema, Lists.newArrayList(tableConfig));
 
     // schema doesn't have destination columns from transformConfigs
     schema = new Schema.SchemaBuilder().setSchemaName(TABLE_NAME)
         .addDateTime(TIME_COLUMN, DataType.LONG, "1:MILLISECONDS:EPOCH", "1:HOURS").build();
-    tableConfig = new TableConfigBuilder(TableType.REALTIME).setTableName(TABLE_NAME).setTimeColumnName(TIME_COLUMN)
+    tableConfig = new TableConfigBuilder(TableType.REALTIME).setTableName(TABLE_NAME)
+        .setStreamConfigs(getStreamConfigs())
+        .setTimeColumnName(TIME_COLUMN)
         .setIngestionConfig(ingestionConfig).build();
     try {
       SchemaUtils.validate(schema, Lists.newArrayList(tableConfig));
@@ -157,6 +163,16 @@ public class SchemaUtilsTest {
     SchemaUtils.validate(schema, Lists.newArrayList(tableConfig));
   }
 
+  private Map<String, String> getStreamConfigs() {
+    Map<String, String> streamConfigs = new HashMap<>();
+    streamConfigs.put("streamType", "kafka");
+    streamConfigs.put("stream.kafka.consumer.type", "lowlevel");
+    streamConfigs.put("stream.kafka.topic.name", "test");
+    streamConfigs.put("stream.kafka.decoder.class.name",
+        "org.apache.pinot.plugin.stream.kafka.KafkaJSONMessageDecoder");
+    return streamConfigs;
+  }
+
   /**
    * TODO: transform functions have moved to tableConfig#ingestionConfig. However, these tests remain to test
    * backward compatibility/
@@ -248,6 +264,19 @@ public class SchemaUtilsTest {
     checkValidationFails(pinotSchema);
   }
 
+  @Test
+  public void testValidateCaseInsensitive() {
+    Schema pinotSchema;
+    pinotSchema =
+      new Schema.SchemaBuilder().addTime(new TimeGranularitySpec(DataType.LONG, TimeUnit.MILLISECONDS, "incoming"),
+          new TimeGranularitySpec(DataType.INT, TimeUnit.DAYS, "outgoing"))
+        .addSingleValueDimension("dim1", DataType.INT)
+        .addSingleValueDimension("Dim1", DataType.INT)
+        .build();
+
+    checkValidationFails(pinotSchema, true);
+  }
+
   @Test
   public void testValidatePrimaryKeyColumns() {
     Schema pinotSchema;
@@ -416,12 +445,16 @@ public class SchemaUtilsTest {
     checkValidationFails(pinotSchema);
   }
 
-  private void checkValidationFails(Schema pinotSchema) {
+  private void checkValidationFails(Schema pinotSchema, boolean isIgnoreCase) {
     try {
-      SchemaUtils.validate(pinotSchema);
+      SchemaUtils.validate(pinotSchema, isIgnoreCase);
       Assert.fail("Schema validation should have failed.");
     } catch (IllegalArgumentException | IllegalStateException e) {
       // expected
     }
   }
+
+  private void checkValidationFails(Schema pinotSchema) {
+    checkValidationFails(pinotSchema, false);
+  }
 }
