@@ -57,8 +57,10 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
   private static final String TABLE_NAME = "MyTable";
   private static final String SEGMENT_NAME_LUCENE = "testSegmentLucene";
   private static final String SEGMENT_NAME_NATIVE = "testSegmentNative";
-  private static final String DOMAIN_NAMES_COL_LUCENE = "DOMAIN_NAMES_LUCENE";
-  private static final String DOMAIN_NAMES_COL_NATIVE = "DOMAIN_NAMES_NATIVE";
+  private static final String QUOTES_COL_LUCENE = "QUOTES_LUCENE";
+  private static final String QUOTES_COL_NATIVE = "QUOTES_NATIVE";
+  private static final String QUOTES_COL_LUCENE_MV = "QUOTES_LUCENE_MV";
+  private static final String QUOTES_COL_NATIVE_MV = "QUOTES_NATIVE_MV";
   private static final Integer NUM_ROWS = 1024;
 
   private IndexSegment _indexSegment;
@@ -107,8 +109,8 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
     FileUtils.deleteQuietly(INDEX_DIR);
   }
 
-  private List<String> getDomainNames() {
-    return Arrays.asList("Prince Andrew kept looking with an amused smile from Pierre",
+  private String[] getTextData() {
+    return new String[]{"Prince Andrew kept looking with an amused smile from Pierre",
         "vicomte and from the vicomte to their hostess. In the first moment of",
         "Pierre’s outburst Anna Pávlovna, despite her social experience, was",
         "horror-struck. But when she saw that Pierre’s sacrilegious words",
@@ -116,17 +118,34 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
         "impossible to stop him, she rallied her forces and joined the vicomte in", "a vigorous attack on the orator",
         "horror-struck. But when she", "she rallied her forces and joined", "outburst Anna Pávlovna",
         "she rallied her forces and", "despite her social experience", "had not exasperated the vicomte",
-        " despite her social experience", "impossible to stop him", "despite her social experience");
+        " despite her social experience", "impossible to stop him", "despite her social experience"};
+  }
+
+  private String[][] getMVTextData() {
+    return new String[][]{
+        {"Prince Andrew kept", "looking with an"}, {"amused smile", "from Pierre"}, {"vicomte and from the"}, {
+          "vicomte to", "their hostess."}, {"In the first moment of"}, {"Pierre’s outburst Anna Pávlovna,"}, {
+          "despite her", "social", "experience, was"}, {"horror-struck.", "But when she"}, {"saw that Pierre’s"}, {
+          "sacrilegious words"}, {"had not exasperated the vicomte, and had convinced herself that it was"}, {
+          "impossible to stop him,", "she rallied her"}, {"forces and joined the vicomte in", "a vigorous attack on "
+        + "the orator"}, {"horror-struck. But when she", "she rallied her forces and joined", "outburst Anna "
+        + "Pávlovna"}, {"she rallied her forces and", "despite her social experience", "had not exasperated the "
+        + "vicomte"}, {"despite her social experience", "impossible to stop him", "despite her social experience"}
+    };
   }
 
   private List<GenericRow> createTestData(int numRows) {
     List<GenericRow> rows = new ArrayList<>();
-    List<String> domainNames = getDomainNames();
+    String[] textData = getTextData();
+    String[][] mvTextData = getMVTextData();
     for (int i = 0; i < numRows; i++) {
-      String domain = domainNames.get(i % domainNames.size());
+      String doc = textData[i % textData.length];
+      String[] mvDoc = mvTextData[i % mvTextData.length];
       GenericRow row = new GenericRow();
-      row.putField(DOMAIN_NAMES_COL_LUCENE, domain);
-      row.putField(DOMAIN_NAMES_COL_NATIVE, domain);
+      row.putValue(QUOTES_COL_LUCENE, doc);
+      row.putValue(QUOTES_COL_NATIVE, doc);
+      row.putValue(QUOTES_COL_LUCENE_MV, mvDoc);
+      row.putValue(QUOTES_COL_NATIVE_MV, mvDoc);
       rows.add(row);
     }
 
@@ -139,13 +158,18 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
     List<FieldConfig> fieldConfigs = new ArrayList<>();
 
     fieldConfigs.add(
-        new FieldConfig(DOMAIN_NAMES_COL_LUCENE, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
+        new FieldConfig(QUOTES_COL_LUCENE, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
+            null));
+    fieldConfigs.add(
+        new FieldConfig(QUOTES_COL_LUCENE_MV, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
             null));
 
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME)
-        .setInvertedIndexColumns(Arrays.asList(DOMAIN_NAMES_COL_LUCENE)).setFieldConfigList(fieldConfigs).build();
+        .setInvertedIndexColumns(Arrays.asList(QUOTES_COL_LUCENE, QUOTES_COL_LUCENE_MV))
+        .setFieldConfigList(fieldConfigs).build();
     Schema schema = new Schema.SchemaBuilder().setSchemaName(TABLE_NAME)
-        .addSingleValueDimension(DOMAIN_NAMES_COL_LUCENE, FieldSpec.DataType.STRING).build();
+        .addSingleValueDimension(QUOTES_COL_LUCENE, FieldSpec.DataType.STRING)
+        .addMultiValueDimension(QUOTES_COL_LUCENE_MV, FieldSpec.DataType.STRING).build();
     SegmentGeneratorConfig config = new SegmentGeneratorConfig(tableConfig, schema);
     config.setOutDir(INDEX_DIR.getPath());
     config.setTableName(TABLE_NAME);
@@ -168,13 +192,18 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
     propertiesMap.put(FieldConfig.TEXT_FST_TYPE, FieldConfig.TEXT_NATIVE_FST_LITERAL);
 
     fieldConfigs.add(
-        new FieldConfig(DOMAIN_NAMES_COL_NATIVE, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
+        new FieldConfig(QUOTES_COL_NATIVE, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
+            propertiesMap));
+    fieldConfigs.add(
+        new FieldConfig(QUOTES_COL_NATIVE_MV, FieldConfig.EncodingType.DICTIONARY, FieldConfig.IndexType.TEXT, null,
             propertiesMap));
 
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME)
-        .setInvertedIndexColumns(Arrays.asList(DOMAIN_NAMES_COL_NATIVE)).setFieldConfigList(fieldConfigs).build();
+        .setInvertedIndexColumns(Arrays.asList(QUOTES_COL_NATIVE, QUOTES_COL_NATIVE_MV))
+        .setFieldConfigList(fieldConfigs).build();
     Schema schema = new Schema.SchemaBuilder().setSchemaName(TABLE_NAME)
-        .addSingleValueDimension(DOMAIN_NAMES_COL_NATIVE, FieldSpec.DataType.STRING).build();
+        .addSingleValueDimension(QUOTES_COL_NATIVE, FieldSpec.DataType.STRING)
+        .addMultiValueDimension(QUOTES_COL_NATIVE_MV, FieldSpec.DataType.STRING).build();
     SegmentGeneratorConfig config = new SegmentGeneratorConfig(tableConfig, schema);
     config.setOutDir(INDEX_DIR.getPath());
     config.setTableName(TABLE_NAME);
@@ -192,10 +221,12 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
       throws Exception {
     IndexLoadingConfig indexLoadingConfig = new IndexLoadingConfig();
     Set<String> textIndexCols = new HashSet<>();
-    textIndexCols.add(DOMAIN_NAMES_COL_LUCENE);
+    textIndexCols.add(QUOTES_COL_LUCENE);
+    textIndexCols.add(QUOTES_COL_LUCENE_MV);
     indexLoadingConfig.setTextIndexColumns(textIndexCols);
     Set<String> invertedIndexCols = new HashSet<>();
-    invertedIndexCols.add(DOMAIN_NAMES_COL_LUCENE);
+    invertedIndexCols.add(QUOTES_COL_LUCENE);
+    invertedIndexCols.add(QUOTES_COL_LUCENE_MV);
     indexLoadingConfig.setInvertedIndexColumns(invertedIndexCols);
     return ImmutableSegmentLoader.load(new File(INDEX_DIR, SEGMENT_NAME_LUCENE), indexLoadingConfig);
   }
@@ -209,13 +240,16 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
 
     Map<String, Map<String, String>> columnPropertiesParentMap = new HashMap<>();
     Set<String> textIndexCols = new HashSet<>();
-    textIndexCols.add(DOMAIN_NAMES_COL_NATIVE);
+    textIndexCols.add(QUOTES_COL_NATIVE);
+    textIndexCols.add(QUOTES_COL_NATIVE_MV);
     indexLoadingConfig.setTextIndexColumns(textIndexCols);
     indexLoadingConfig.setFSTIndexType(fstType);
     Set<String> invertedIndexCols = new HashSet<>();
-    invertedIndexCols.add(DOMAIN_NAMES_COL_NATIVE);
+    invertedIndexCols.add(QUOTES_COL_NATIVE);
+    invertedIndexCols.add(QUOTES_COL_NATIVE_MV);
     indexLoadingConfig.setInvertedIndexColumns(invertedIndexCols);
-    columnPropertiesParentMap.put(DOMAIN_NAMES_COL_NATIVE, propertiesMap);
+    columnPropertiesParentMap.put(QUOTES_COL_NATIVE, propertiesMap);
+    columnPropertiesParentMap.put(QUOTES_COL_NATIVE_MV, propertiesMap);
     indexLoadingConfig.setColumnProperties(columnPropertiesParentMap);
     return ImmutableSegmentLoader.load(new File(INDEX_DIR, SEGMENT_NAME_NATIVE), indexLoadingConfig);
   }
@@ -247,29 +281,51 @@ public class NativeAndLuceneComparisonTest extends BaseQueriesTest {
       }
     }
   }
-
   @Test
   public void testQueries() {
-    String nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'vico.*') LIMIT 50000";
-    String luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(DOMAIN_NAMES_LUCENE, 'vico*') LIMIT 50000";
+
+    String nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE, 'vico.*') LIMIT 50000";
+    String luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE, 'vico*') LIMIT 50000";
     testSelectionResults(nativeQuery, luceneQuery);
 
-    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'convi.*ced') LIMIT 50000";
-    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(DOMAIN_NAMES_LUCENE, 'convi*ced') LIMIT 50000";
+    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE, 'convi.*ced') LIMIT 50000";
+    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE, 'convi*ced') LIMIT 50000";
     testSelectionResults(nativeQuery, luceneQuery);
 
-    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'vicomte') AND "
-        + "TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'hos.*') LIMIT 50000";
-    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(DOMAIN_NAMES_LUCENE, 'vicomte AND hos*') LIMIT 50000";
+    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE, 'vicomte') AND "
+        + "TEXT_CONTAINS(QUOTES_NATIVE, 'hos.*') LIMIT 50000";
+    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE, 'vicomte AND hos*') LIMIT 50000";
     testSelectionResults(nativeQuery, luceneQuery);
 
-    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'sac.*') OR "
-        + "TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'herself') LIMIT 50000";
-    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(DOMAIN_NAMES_LUCENE, 'sac* OR herself') LIMIT 50000";
+    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE, 'sac.*') OR "
+        + "TEXT_CONTAINS(QUOTES_NATIVE, 'herself') LIMIT 50000";
+    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE, 'sac* OR herself') LIMIT 50000";
     testSelectionResults(nativeQuery, luceneQuery);
 
-    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(DOMAIN_NAMES_NATIVE, 'vicomte') LIMIT 50000";
-    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(DOMAIN_NAMES_LUCENE, 'vicomte') LIMIT 50000";
+    nativeQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE, 'vicomte') LIMIT 50000";
+    luceneQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE, 'vicomte') LIMIT 50000";
     testSelectionResults(nativeQuery, luceneQuery);
+
+    String nativeMVQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE_MV, 'vico.*') LIMIT 50000";
+    String luceneMVQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE_MV, 'vico*') LIMIT 50000";
+    testSelectionResults(nativeMVQuery, luceneMVQuery);
+
+    nativeMVQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE_MV, 'convi.*ced') LIMIT 50000";
+    luceneMVQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE_MV, 'convi*ced') LIMIT 50000";
+    testSelectionResults(nativeMVQuery, luceneMVQuery);
+
+    nativeMVQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE_MV, 'vicomte') AND "
+        + "TEXT_CONTAINS(QUOTES_NATIVE_MV, 'hos.*') LIMIT 50000";
+    luceneMVQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE_MV, 'vicomte AND hos*') LIMIT 50000";
+    testSelectionResults(nativeMVQuery, luceneMVQuery);
+
+    nativeMVQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE_MV, 'sac.*') OR "
+        + "TEXT_CONTAINS(QUOTES_NATIVE_MV, 'herself') LIMIT 50000";
+    luceneMVQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE_MV, 'sac* OR herself') LIMIT 50000";
+    testSelectionResults(nativeMVQuery, luceneMVQuery);
+
+    nativeMVQuery = "SELECT * FROM MyTable WHERE TEXT_CONTAINS(QUOTES_NATIVE_MV, 'vicomte') LIMIT 50000";
+    luceneMVQuery = "SELECT * FROM MyTable WHERE TEXT_MATCH(QUOTES_LUCENE_MV, 'vicomte') LIMIT 50000";
+    testSelectionResults(nativeMVQuery, luceneMVQuery);
   }
 }
