@@ -21,7 +21,7 @@ package org.apache.pinot.queries;
 import com.tdunning.math.stats.TDigest;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.Arrays;
 import java.util.List;
 import org.apache.pinot.core.query.aggregation.function.PercentileTDigestAggregationFunction;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentIndexCreationDriverImpl;
@@ -42,7 +42,10 @@ import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
  * Tests for PERCENTILE_TDIGEST and PERCENTILE_TDIGEST_MV aggregation functions.
  *
  * <ul>
- *   <li>Generates a segment with a double multi-valued column, a TDigest column and a group-by column</li>
+ *   <li>
+ *     Generates a segment with a double multi-valued column, a TDigest column, a custom compression TDigest column
+ *     and a group-by column
+ *   </li>
  *   <li>Runs aggregation and group-by queries on the generated segment</li>
  *   <li>
  *     Compares the results for PERCENTILE_TDIGEST_MV on double multi-valued column, PERCENTILE_TDIGEST on TDigest
@@ -63,10 +66,12 @@ public class PercentileTDigestMVQueriesTest extends PercentileTDigestQueriesTest
       int numMultiValues = RANDOM.nextInt(MAX_NUM_MULTI_VALUES) + 1;
       Double[] values = new Double[numMultiValues];
       TDigest tDigest = TDigest.createMergingDigest(PercentileTDigestAggregationFunction.DEFAULT_TDIGEST_COMPRESSION);
+      TDigest tDigestCustom = TDigest.createMergingDigest(CUSTOM_COMPRESSION);
       for (int j = 0; j < numMultiValues; j++) {
         double value = RANDOM.nextDouble() * VALUE_RANGE;
         values[j] = value;
         tDigest.add(value);
+        tDigestCustom.add(value);
       }
       row.putValue(DOUBLE_COLUMN, values);
 
@@ -74,6 +79,10 @@ public class PercentileTDigestMVQueriesTest extends PercentileTDigestQueriesTest
       tDigest.asBytes(byteBuffer);
       row.putValue(TDIGEST_COLUMN, byteBuffer.array());
 
+      ByteBuffer byteBufferCustom = ByteBuffer.allocate(tDigestCustom.byteSize());
+      tDigestCustom.asBytes(byteBufferCustom);
+      row.putValue(TDIGEST_CUSTOM_COMPRESSION_COLUMN, byteBufferCustom.array());
+
       String group = GROUPS[RANDOM.nextInt(GROUPS.length)];
       row.putValue(GROUP_BY_COLUMN, group);
 
@@ -83,6 +92,7 @@ public class PercentileTDigestMVQueriesTest extends PercentileTDigestQueriesTest
     Schema schema = new Schema();
     schema.addField(new DimensionFieldSpec(DOUBLE_COLUMN, FieldSpec.DataType.DOUBLE, false));
     schema.addField(new MetricFieldSpec(TDIGEST_COLUMN, FieldSpec.DataType.BYTES));
+    schema.addField(new MetricFieldSpec(TDIGEST_CUSTOM_COMPRESSION_COLUMN, FieldSpec.DataType.BYTES));
     schema.addField(new DimensionFieldSpec(GROUP_BY_COLUMN, FieldSpec.DataType.STRING, true));
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
 
@@ -90,7 +100,7 @@ public class PercentileTDigestMVQueriesTest extends PercentileTDigestQueriesTest
     config.setOutDir(INDEX_DIR.getPath());
     config.setTableName(TABLE_NAME);
     config.setSegmentName(SEGMENT_NAME);
-    config.setRawIndexCreationColumns(Collections.singletonList(TDIGEST_COLUMN));
+    config.setRawIndexCreationColumns(Arrays.asList(TDIGEST_COLUMN, TDIGEST_CUSTOM_COMPRESSION_COLUMN));
 
     SegmentIndexCreationDriverImpl driver = new SegmentIndexCreationDriverImpl();
     try (RecordReader recordReader = new GenericRowRecordReader(rows)) {
@@ -102,8 +112,9 @@ public class PercentileTDigestMVQueriesTest extends PercentileTDigestQueriesTest
   @Override
   protected String getAggregationQuery(int percentile) {
     return String.format("SELECT PERCENTILE%1$dMV(%2$s), PERCENTILETDIGEST%1$dMV(%2$s), PERCENTILETDIGEST%1$d(%3$s), "
-            + "PERCENTILEMV(%2$s, %1$d), PERCENTILETDIGESTMV(%2$s, %1$d), PERCENTILETDIGEST(%3$s, %1$d) FROM %4$s",
-        percentile, DOUBLE_COLUMN, TDIGEST_COLUMN, TABLE_NAME);
+            + "PERCENTILEMV(%2$s, %1$d), PERCENTILETDIGESTMV(%2$s, %1$d), PERCENTILETDIGEST(%3$s, %1$d), "
+            + "PERCENTILETDIGESTMV(%2$s, %1$d, %6$d), PERCENTILETDIGEST(%5$s, %1$d, %6$d) FROM %4$s",
+        percentile, DOUBLE_COLUMN, TDIGEST_COLUMN, TABLE_NAME, TDIGEST_CUSTOM_COMPRESSION_COLUMN, CUSTOM_COMPRESSION);
   }
 
   @Override
