@@ -31,6 +31,8 @@ import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Broker.FailureDetector;
 import org.apache.pinot.spi.utils.CommonConstants.Helix.StateModel.BrokerResourceStateModel;
 import org.apache.pinot.util.TestUtils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
@@ -43,6 +45,7 @@ import static org.testng.Assert.fail;
  * Integration test that extends OfflineClusterIntegrationTest but start multiple brokers and servers.
  */
 public class MultiNodesOfflineClusterIntegrationTest extends OfflineClusterIntegrationTest {
+  private static final Logger LOGGER = LoggerFactory.getLogger(MultiNodesOfflineClusterIntegrationTest.class);
   private static final int NUM_BROKERS = 2;
   private static final int NUM_SERVERS = 3;
 
@@ -136,30 +139,34 @@ public class MultiNodesOfflineClusterIntegrationTest extends OfflineClusterInteg
     testCountStarQuery(3, false);
     assertEquals(getCurrentCountStarResult(), expectedCountStarResult);
 
+    LOGGER.warn("Shutting down server " + _serverStarters.get(NUM_SERVERS - 1).getInstanceId());
     // Take a server and shut down its query server to mimic a hard failure
     BaseServerStarter serverStarter = _serverStarters.get(NUM_SERVERS - 1);
-    serverStarter.getServerInstance().shutDown();
+    try {
+      serverStarter.getServerInstance().shutDown();
 
-    // First query should hit all servers and get connection refused exception
-    testCountStarQuery(NUM_SERVERS, true);
+      // First query should hit all servers and get connection refused or reset exception
+      // TODO: This is a flaky test. There is a race condition between shutDown and the query being executed.
+      testCountStarQuery(NUM_SERVERS, true);
 
-    // Second query should not hit the failed server, and should return the correct result
-    testCountStarQuery(NUM_SERVERS - 1, false);
-
-    // Restart the failed server, and it should be included in the routing again
-    serverStarter.stop();
-    serverStarter = startOneServer(NUM_SERVERS - 1);
-    _serverStarters.set(NUM_SERVERS - 1, serverStarter);
-    TestUtils.waitForCondition(aVoid -> {
-      try {
-        JsonNode queryResult = postQuery("SELECT COUNT(*) FROM mytable");
-        // Result should always be correct
-        assertEquals(queryResult.get("resultTable").get("rows").get(0).get(0).longValue(), getCountStarResult());
-        return queryResult.get("numServersQueried").intValue() == NUM_SERVERS;
-      } catch (Exception e) {
-        throw new RuntimeException(e);
-      }
-    }, 10_000L, "Failed to include the restarted server into the routing");
+      // Second query should not hit the failed server, and should return the correct result
+      testCountStarQuery(NUM_SERVERS - 1, false);
+    } finally {
+      // Restart the failed server, and it should be included in the routing again
+      serverStarter.stop();
+      serverStarter = startOneServer(NUM_SERVERS - 1);
+      _serverStarters.set(NUM_SERVERS - 1, serverStarter);
+      TestUtils.waitForCondition((aVoid) -> {
+        try {
+          JsonNode queryResult = postQuery("SELECT COUNT(*) FROM mytable");
+          // Result should always be correct
+          assertEquals(queryResult.get("resultTable").get("rows").get(0).get(0).longValue(), getCountStarResult());
+          return queryResult.get("numServersQueried").intValue() == NUM_SERVERS;
+        } catch (Exception e) {
+          throw new RuntimeException(e);
+        }
+      }, 10_000L, "Failed to include the restarted server into the routing. Other tests may be affected");
+    }
   }
 
   private void testCountStarQuery(int expectedNumServersQueried, boolean exceptionExpected)
@@ -171,8 +178,12 @@ public class MultiNodesOfflineClusterIntegrationTest extends OfflineClusterInteg
       JsonNode exceptions = queryResult.get("exceptions");
       assertEquals(exceptions.size(), 2);
       JsonNode firstException = exceptions.get(0);
+      // NOTE:
+      // Only verify the error code but not the exception message because there can be different messages:
+      // - Connection refused
+      // - Connection reset
+      // - Channel is inactive
       assertEquals(firstException.get("errorCode").intValue(), QueryException.BROKER_REQUEST_SEND_ERROR_CODE);
-      assertTrue(firstException.get("message").textValue().contains("Connection refused"));
       JsonNode secondException = exceptions.get(1);
       assertEquals(secondException.get("errorCode").intValue(), QueryException.SERVER_NOT_RESPONDING_ERROR_CODE);
     } else {
@@ -183,21 +194,19 @@ public class MultiNodesOfflineClusterIntegrationTest extends OfflineClusterInteg
 
   // Disabled because with multiple replicas, there is no guarantee that all replicas are reloaded
   @Test(enabled = false)
-  @Override
-  public void testStarTreeTriggering() {
+  public void testStarTreeTriggering(boolean useMultiStageQueryEngine) {
     // Ignored
   }
 
   // Disabled because with multiple replicas, there is no guarantee that all replicas are reloaded
   @Test(enabled = false)
   @Override
-  public void testDefaultColumns() {
+  public void testDefaultColumns(boolean useMultiStageQueryEngineg) {
     // Ignored
   }
 
   // Disabled because with multiple replicas, there is no guarantee that all replicas are reloaded
   @Test(enabled = false)
-  @Override
   public void testBloomFilterTriggering() {
     // Ignored
   }
@@ -205,7 +214,8 @@ public class MultiNodesOfflineClusterIntegrationTest extends OfflineClusterInteg
   // Disabled because with multiple replicas, there is no guarantee that all replicas are reloaded
   @Test(enabled = false)
   @Override
-  public void testRangeIndexTriggering() {
+  public void testRangeIndexTriggering(boolean useMultiStageQueryEngine)
+      throws Exception {
     // Ignored
   }
 
