@@ -34,7 +34,7 @@ import org.apache.pinot.common.auth.AuthProviderUtils;
 import org.apache.pinot.common.metrics.MinionGauge;
 import org.apache.pinot.common.metrics.MinionMeter;
 import org.apache.pinot.common.metrics.MinionMetrics;
-import org.apache.pinot.common.metrics.MinionQueryPhase;
+import org.apache.pinot.common.metrics.MinionTimer;
 import org.apache.pinot.core.common.MinionConstants;
 import org.apache.pinot.core.minion.PinotTaskConfig;
 import org.apache.pinot.minion.MinionContext;
@@ -58,6 +58,18 @@ import org.slf4j.MDC;
  */
 public class TaskFactoryRegistry {
   private static final Logger LOGGER = LoggerFactory.getLogger(TaskFactoryRegistry.class);
+  // we use 1000 as the limit for the following reasons:
+  // 1. Task results reported (inlcudeing info messages) here will be saved into znode
+  //    (<pinot cluster name>/PROPERTYSTORE/TaskRebalancer/<Helix job name>), which has a default limit of 1M.
+  //    Since tasks belonging to the same helix job will be saved in the same znode, info messages cannot be too long,
+  //    Otherwise, the znode may be too large, causing communication issue between zookeeper clients and zookeeper,
+  //    which results in instance crash and znode not updatable.
+  // 2. Info messages stored in znode are fetched by minion task APIs (/tasks/{taskType}/debug, /tasks/{taskType}/debug,
+  //    /tasks/task/{taskName}/debug) for debugging purpose, so the info must contain enough info.
+  // 1000 is a reasonable choice, because with this length, the info message will contain relatively rich info for
+  // debugging. At the same time, with znode compressed, the helix job znode can hold ~1000 helix tasks (1M/1000=1000),
+  // which is enough in most use cases.
+  private static final int MAX_TASK_RESULT_INFO_LEN = 1000;
 
   private final Map<String, TaskFactory> _taskFactoryRegistry = new HashMap<>();
 
@@ -85,59 +97,99 @@ public class TaskFactoryRegistry {
               long jobInQueueTimeMs = jobContext.getStartTime();
               long jobDequeueTimeMs = System.currentTimeMillis();
               _minionMetrics
-                  .addPhaseTiming(taskType, MinionQueryPhase.TASK_QUEUEING, jobDequeueTimeMs - jobInQueueTimeMs,
+                  .addTimedValue(taskType, MinionTimer.TASK_QUEUEING, jobDequeueTimeMs - jobInQueueTimeMs,
                       TimeUnit.MILLISECONDS);
+              String tableName = null;
               try {
                 // Set taskId in MDC so that one may config logger to route task logs to separate file.
                 MDC.put("taskId", _taskConfig.getId());
+                PinotTaskConfig pinotTaskConfig = PinotTaskConfig.fromHelixTaskConfig(_taskConfig);
+                tableName = pinotTaskConfig.getTableName();
                 _minionMetrics.addValueToGlobalGauge(MinionGauge.NUMBER_OF_TASKS, 1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS, 1L);
+                if (tableName != null) {
+                  _minionMetrics
+                      .addTimedTableValue(tableName, taskType, MinionTimer.TASK_QUEUEING,
+                          jobDequeueTimeMs - jobInQueueTimeMs, TimeUnit.MILLISECONDS);
+                  _minionMetrics.addValueToTableGauge(tableName, MinionGauge.NUMBER_OF_TASKS, 1L);
+                  _minionMetrics.addMeteredTableValue(tableName, taskType, MinionMeter.NUMBER_TASKS, 1L);
+                }
                 MinionEventObservers.getInstance().addMinionEventObserver(_taskConfig.getId(), _eventObserver);
-                return runInternal();
+                return runInternal(pinotTaskConfig);
               } finally {
                 MinionEventObservers.getInstance().removeMinionEventObserver(_taskConfig.getId());
                 _minionMetrics.addValueToGlobalGauge(MinionGauge.NUMBER_OF_TASKS, -1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS, -1L);
                 long executionTimeMs = System.currentTimeMillis() - jobDequeueTimeMs;
                 _minionMetrics
-                    .addPhaseTiming(taskType, MinionQueryPhase.TASK_EXECUTION, executionTimeMs, TimeUnit.MILLISECONDS);
+                    .addTimedValue(taskType, MinionTimer.TASK_EXECUTION, executionTimeMs, TimeUnit.MILLISECONDS);
+                if (tableName != null) {
+                  _minionMetrics.addValueToTableGauge(tableName, MinionGauge.NUMBER_OF_TASKS, -1L);
+                  _minionMetrics.addMeteredTableValue(tableName, taskType, MinionMeter.NUMBER_TASKS, -1L);
+                  _minionMetrics
+                      .addTimedTableValue(tableName, taskType, MinionTimer.TASK_EXECUTION,
+                          executionTimeMs, TimeUnit.MILLISECONDS);
+                }
                 LOGGER.info("Task: {} completed in: {}ms", _taskConfig.getId(), executionTimeMs);
                 // Clear taskId from MDC to reset it.
                 MDC.remove("taskId");
               }
             }
 
-            private TaskResult runInternal() {
-              PinotTaskConfig pinotTaskConfig = PinotTaskConfig.fromHelixTaskConfig(_taskConfig);
+            private TaskResult runInternal(PinotTaskConfig pinotTaskConfig) {
               if (StringUtils.isBlank(pinotTaskConfig.getConfigs().get(MinionConstants.AUTH_TOKEN))) {
                 pinotTaskConfig.getConfigs().put(MinionConstants.AUTH_TOKEN,
                     AuthProviderUtils.toStaticToken(MinionContext.getInstance().getTaskAuthProvider()));
               }
 
+              String tableName = pinotTaskConfig.getTableName();
+
               _eventObserver.notifyTaskStart(pinotTaskConfig);
-              _minionMetrics.addMeteredTableValue(taskType, MinionMeter.NUMBER_TASKS_EXECUTED, 1L);
+              _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS_EXECUTED, 1L);
+              if (tableName != null) {
+                _minionMetrics.addMeteredTableValue(tableName, taskType,
+                    MinionMeter.NUMBER_TASKS_EXECUTED, 1L);
+              }
               LOGGER.info("Start running {}: {} with configs: {}", pinotTaskConfig.getTaskType(), _taskConfig.getId(),
                   pinotTaskConfig.getConfigs());
 
               try {
                 Object executionResult = _taskExecutor.executeTask(pinotTaskConfig);
                 _eventObserver.notifyTaskSuccess(pinotTaskConfig, executionResult);
-                _minionMetrics.addMeteredTableValue(taskType, MinionMeter.NUMBER_TASKS_COMPLETED, 1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS_COMPLETED, 1L);
+                if (tableName != null) {
+                  _minionMetrics.addMeteredTableValue(tableName, taskType,
+                      MinionMeter.NUMBER_TASKS_COMPLETED, 1L);
+                }
                 LOGGER.info("Task: {} succeeded", _taskConfig.getId());
                 return new TaskResult(TaskResult.Status.COMPLETED, "Succeeded");
               } catch (TaskCancelledException e) {
                 _eventObserver.notifyTaskCancelled(pinotTaskConfig);
-                _minionMetrics.addMeteredTableValue(taskType, MinionMeter.NUMBER_TASKS_CANCELLED, 1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS_CANCELLED, 1L);
+                if (tableName != null) {
+                  _minionMetrics.addMeteredTableValue(tableName, taskType,
+                      MinionMeter.NUMBER_TASKS_CANCELLED, 1L);
+                }
                 LOGGER.info("Task: {} got cancelled", _taskConfig.getId(), e);
-                return new TaskResult(TaskResult.Status.CANCELED, ExceptionUtils.getStackTrace(e));
+                return new TaskResult(TaskResult.Status.CANCELED, extractAndTrimRootCauseMessage(e));
               } catch (FatalException e) {
                 _eventObserver.notifyTaskError(pinotTaskConfig, e);
-                _minionMetrics.addMeteredTableValue(taskType, MinionMeter.NUMBER_TASKS_FATAL_FAILED, 1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS_FATAL_FAILED, 1L);
+                if (tableName != null) {
+                  _minionMetrics.addMeteredTableValue(tableName, taskType,
+                      MinionMeter.NUMBER_TASKS_FATAL_FAILED, 1L);
+                }
                 LOGGER.error("Caught fatal exception while executing task: {}", _taskConfig.getId(), e);
-                return new TaskResult(TaskResult.Status.FATAL_FAILED, ExceptionUtils.getStackTrace(e));
+                return new TaskResult(TaskResult.Status.FATAL_FAILED, extractAndTrimRootCauseMessage(e));
               } catch (Exception e) {
                 _eventObserver.notifyTaskError(pinotTaskConfig, e);
-                _minionMetrics.addMeteredTableValue(taskType, MinionMeter.NUMBER_TASKS_FAILED, 1L);
+                _minionMetrics.addMeteredValue(taskType, MinionMeter.NUMBER_TASKS_FAILED, 1L);
+                if (tableName != null) {
+                  _minionMetrics.addMeteredTableValue(tableName, taskType,
+                      MinionMeter.NUMBER_TASKS_FAILED, 1L);
+                }
                 LOGGER.error("Caught exception while executing task: {}", _taskConfig.getId(), e);
-                return new TaskResult(TaskResult.Status.FAILED, ExceptionUtils.getStackTrace(e));
+                return new TaskResult(TaskResult.Status.FAILED, extractAndTrimRootCauseMessage(e));
               }
             }
 
@@ -155,6 +207,14 @@ public class TaskFactoryRegistry {
     }
   }
 
+  private static String extractAndTrimRootCauseMessage(Throwable th) {
+    String rootCauseMessage = ExceptionUtils.getStackTrace(th);
+    if (rootCauseMessage != null && rootCauseMessage.length() > MAX_TASK_RESULT_INFO_LEN) {
+      return rootCauseMessage.substring(0, MAX_TASK_RESULT_INFO_LEN);
+    }
+    return rootCauseMessage;
+  }
+
   /**
    * Returns the task factory registry.
    */
