@@ -49,6 +49,7 @@ import org.apache.pinot.core.common.MinionConstants.MergeTask;
 import org.apache.pinot.core.minion.PinotTaskConfig;
 import org.apache.pinot.plugin.minion.tasks.MergeTaskUtils;
 import org.apache.pinot.plugin.minion.tasks.MinionTaskUtils;
+import org.apache.pinot.plugin.minion.tasks.mergerollup.segmentgroupmananger.MergeRollupTaskSegmentGroupManagerProvider;
 import org.apache.pinot.spi.annotations.minion.TaskGenerator;
 import org.apache.pinot.spi.config.table.ColumnPartitionConfig;
 import org.apache.pinot.spi.config.table.SegmentPartitionConfig;
@@ -130,10 +131,14 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
   // delay. Setting the alert on 7 time buckets of delay would be a good starting point.
   private static final String MERGE_ROLLUP_TASK_DELAY_IN_NUM_BUCKETS = "mergeRollupTaskDelayInNumBuckets";
 
+  private static final String MERGE_ROLLUP_TASK_NUM_BUCKETS_TO_PROCESS = "mergeRollupTaskNumBucketsToProcess";
+
   // tableNameWithType -> mergeLevel -> watermarkMs
   private final Map<String, Map<String, Long>> _mergeRollupWatermarks = new HashMap<>();
   // tableNameWithType -> lowestLevelMaxValidBucketEndTime
   private final Map<String, Long> _tableLowestLevelMaxValidBucketEndTimeMs = new HashMap<>();
+  // tableNameWithType -> mergeLevel -> numberBucketsToProcess
+  private final Map<String, Map<String, Long>> _tableNumberBucketsToProcess = new HashMap<>();
 
   @Override
   public String getTaskType() {
@@ -211,6 +216,10 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         }
       }
 
+      // Get scheduling mode which is "processFromWatermark" by default. If "processAll" mode is enabled, there will be
+      // no watermark, and each round we pick the buckets in chronological order which have unmerged segments.
+      boolean processAll = MergeTask.PROCESS_ALL_MODE.equalsIgnoreCase(taskConfigs.get(MergeTask.MODE));
+
       ZNRecord mergeRollupTaskZNRecord = _clusterInfoAccessor
           .getMinionTaskMetadataZNRecord(MinionConstants.MergeRollupTask.TASK_TYPE, tableNameWithType);
       int expectedVersion = mergeRollupTaskZNRecord != null ? mergeRollupTaskZNRecord.getVersion() : -1;
@@ -257,12 +266,18 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
           continue;
         }
 
-        // Get watermark from MergeRollupTaskMetadata ZNode
-        // bucketStartMs = watermarkMs
-        // bucketEndMs = bucketStartMs + bucketMs
-        long watermarkMs =
-            getWatermarkMs(preSelectedSegments.get(0).getStartTimeMs(), bucketMs, mergeLevel, mergeRollupTaskMetadata);
-        long bucketStartMs = watermarkMs;
+        // Get bucket start/end time
+        long preSelectedSegStartTimeMs = preSelectedSegments.get(0).getStartTimeMs();
+        long bucketStartMs = preSelectedSegStartTimeMs / bucketMs * bucketMs;
+        long watermarkMs = 0;
+        if (!processAll) {
+          // Get watermark from MergeRollupTaskMetadata ZNode
+          // bucketStartMs = watermarkMs
+          // bucketEndMs = bucketStartMs + bucketMs
+          watermarkMs = getWatermarkMs(preSelectedSegStartTimeMs, bucketMs, mergeLevel,
+              mergeRollupTaskMetadata);
+          bucketStartMs = watermarkMs;
+        }
         long bucketEndMs = bucketStartMs + bucketMs;
         if (lowerMergeLevel == null) {
           long lowestLevelMaxValidBucketEndTimeMs = Long.MIN_VALUE;
@@ -275,12 +290,21 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
           }
           _tableLowestLevelMaxValidBucketEndTimeMs.put(tableNameWithType, lowestLevelMaxValidBucketEndTimeMs);
         }
-        // Create delay metrics even if there's no task scheduled, this helps the case that the controller is restarted
+        // Create metrics even if there's no task scheduled, this helps the case that the controller is restarted
         // but the metrics are not available until the controller schedules a valid task
-        createOrUpdateDelayMetrics(tableNameWithType, mergeLevel, null, watermarkMs, bufferMs, bucketMs);
-        if (!isValidBucketEndTime(bucketEndMs, bufferMs, lowerMergeLevel, mergeRollupTaskMetadata)) {
-          LOGGER.info("Bucket with start: {} and end: {} (table : {}, mergeLevel : {}) cannot be merged yet",
-              bucketStartMs, bucketEndMs, tableNameWithType, mergeLevel);
+        List<String> sortedMergeLevels =
+            sortedMergeLevelConfigs.stream().map(e -> e.getKey()).collect(Collectors.toList());
+        if (processAll) {
+          createOrUpdateNumBucketsToProcessMetrics(tableNameWithType, mergeLevel, lowerMergeLevel, bufferMs, bucketMs,
+              preSelectedSegments, sortedMergeLevels);
+        } else {
+          createOrUpdateDelayMetrics(tableNameWithType, mergeLevel, null, watermarkMs, bufferMs, bucketMs);
+        }
+
+        if (!isValidBucketEndTime(bucketEndMs, bufferMs, lowerMergeLevel, mergeRollupTaskMetadata, processAll)) {
+          LOGGER.info("Bucket with start: {} and end: {} (table : {}, mergeLevel : {}, mode : {}) cannot be merged yet",
+              bucketStartMs, bucketEndMs, tableNameWithType, mergeLevel, processAll ? MergeTask.PROCESS_ALL_MODE
+                  : MergeTask.PROCESS_FROM_WATERMARK_MODE);
           continue;
         }
 
@@ -289,6 +313,7 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         List<SegmentZKMetadata> selectedSegmentsForBucket = new ArrayList<>();
         boolean hasUnmergedSegments = false;
         boolean hasSpilledOverData = false;
+        boolean areAllSegmentsReadyToMerge = true;
 
         // The for loop terminates in following cases:
         // 1. Found buckets with unmerged segments:
@@ -305,9 +330,12 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
             long endTimeMs = preSelectedSegment.getEndTimeMs();
             if (endTimeMs >= bucketStartMs) {
               // For segments overlapping with current bucket, add to the result list
-              if (!isMergedSegment(preSelectedSegment, mergeLevel)) {
+              if (!isMergedSegment(preSelectedSegment, mergeLevel, sortedMergeLevels)) {
                 hasUnmergedSegments = true;
               }
+              if (!isMergedSegment(preSelectedSegment, lowerMergeLevel, sortedMergeLevels)) {
+                areAllSegmentsReadyToMerge = false;
+              }
               if (hasSpilledOverData(preSelectedSegment, bucketMs)) {
                 hasSpilledOverData = true;
               }
@@ -317,7 +345,7 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
             // Haven't find the first overlapping segment, continue to the next segment
           } else {
             // Has gone through all overlapping segments for current bucket
-            if (hasUnmergedSegments) {
+            if (hasUnmergedSegments && areAllSegmentsReadyToMerge) {
               // Add the bucket if there are unmerged segments
               selectedSegmentsForAllBuckets.add(selectedSegmentsForBucket);
             }
@@ -330,14 +358,18 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
               // TODO: If there are many small merged segments, we should merge them again
               selectedSegmentsForBucket = new ArrayList<>();
               hasUnmergedSegments = false;
+              areAllSegmentsReadyToMerge = true;
               bucketStartMs = (startTimeMs / bucketMs) * bucketMs;
               bucketEndMs = bucketStartMs + bucketMs;
-              if (!isValidBucketEndTime(bucketEndMs, bufferMs, lowerMergeLevel, mergeRollupTaskMetadata)) {
+              if (!isValidBucketEndTime(bucketEndMs, bufferMs, lowerMergeLevel, mergeRollupTaskMetadata, processAll)) {
                 break;
               }
-              if (!isMergedSegment(preSelectedSegment, mergeLevel)) {
+              if (!isMergedSegment(preSelectedSegment, mergeLevel, sortedMergeLevels)) {
                 hasUnmergedSegments = true;
               }
+              if (!isMergedSegment(preSelectedSegment, lowerMergeLevel, sortedMergeLevels)) {
+                areAllSegmentsReadyToMerge = false;
+              }
               if (hasSpilledOverData(preSelectedSegment, bucketMs)) {
                 hasSpilledOverData = true;
               }
@@ -347,7 +379,7 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         }
 
         // Add the last bucket if it contains unmerged segments and is not added before
-        if (hasUnmergedSegments && (selectedSegmentsForAllBuckets.isEmpty() || (
+        if (hasUnmergedSegments && areAllSegmentsReadyToMerge && (selectedSegmentsForAllBuckets.isEmpty() || (
             selectedSegmentsForAllBuckets.get(selectedSegmentsForAllBuckets.size() - 1)
                 != selectedSegmentsForBucket))) {
           selectedSegmentsForAllBuckets.add(selectedSegmentsForBucket);
@@ -365,7 +397,10 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
             watermarkMs, newWatermarkMs);
 
         // Update the delay metrics
-        createOrUpdateDelayMetrics(tableNameWithType, mergeLevel, lowerMergeLevel, newWatermarkMs, bufferMs, bucketMs);
+        if (!processAll) {
+          createOrUpdateDelayMetrics(tableNameWithType, mergeLevel, lowerMergeLevel, newWatermarkMs, bufferMs,
+              bucketMs);
+        }
 
         // Create task configs
         int maxNumRecordsPerTask =
@@ -375,7 +410,7 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         if (segmentPartitionConfig == null) {
           for (List<SegmentZKMetadata> selectedSegmentsPerBucket : selectedSegmentsForAllBuckets) {
             pinotTaskConfigsForTable.addAll(
-                createPinotTaskConfigs(selectedSegmentsPerBucket, tableNameWithType, maxNumRecordsPerTask, mergeLevel,
+                createPinotTaskConfigs(selectedSegmentsPerBucket, tableConfig, maxNumRecordsPerTask, mergeLevel,
                     null, mergeConfigs, taskConfigs));
           }
         } else {
@@ -414,13 +449,13 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
               List<Integer> partition = entry.getKey();
               List<SegmentZKMetadata> partitionedSegments = entry.getValue();
               pinotTaskConfigsForTable.addAll(
-                  createPinotTaskConfigs(partitionedSegments, tableNameWithType, maxNumRecordsPerTask, mergeLevel,
+                  createPinotTaskConfigs(partitionedSegments, tableConfig, maxNumRecordsPerTask, mergeLevel,
                       partition, mergeConfigs, taskConfigs));
             }
 
             if (!outlierSegments.isEmpty()) {
               pinotTaskConfigsForTable.addAll(
-                  createPinotTaskConfigs(outlierSegments, tableNameWithType, maxNumRecordsPerTask, mergeLevel,
+                  createPinotTaskConfigs(outlierSegments, tableConfig, maxNumRecordsPerTask, mergeLevel,
                       null, mergeConfigs, taskConfigs));
             }
           }
@@ -428,14 +463,17 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       }
 
       // Write updated watermark map to zookeeper
-      try {
-        _clusterInfoAccessor
-            .setMinionTaskMetadata(mergeRollupTaskMetadata, MinionConstants.MergeRollupTask.TASK_TYPE, expectedVersion);
-      } catch (ZkException e) {
-        LOGGER.error(
-            "Version changed while updating merge/rollup task metadata for table: {}, skip scheduling. There are "
-                + "multiple task schedulers for the same table, need to investigate!", tableNameWithType);
-        continue;
+      if (!processAll) {
+        try {
+          _clusterInfoAccessor
+              .setMinionTaskMetadata(mergeRollupTaskMetadata, MinionConstants.MergeRollupTask.TASK_TYPE,
+                  expectedVersion);
+        } catch (ZkException e) {
+          LOGGER.error(
+              "Version changed while updating merge/rollup task metadata for table: {}, skip scheduling. There are "
+                  + "multiple task schedulers for the same table, need to investigate!", tableNameWithType);
+          continue;
+        }
       }
       pinotTaskConfigs.addAll(pinotTaskConfigsForTable);
       LOGGER.info("Finished generating task configs for table: {} for task: {}, numTasks: {}", tableNameWithType,
@@ -481,9 +519,9 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       }
       final long finalEarliestStartTimeMsOfInProgressSegments = earliestStartTimeMsOfInProgressSegments;
       return allSegments.stream()
-              .filter(segmentZKMetadata -> segmentZKMetadata.getStatus().isCompleted()
-                  && segmentZKMetadata.getStartTimeMs() < finalEarliestStartTimeMsOfInProgressSegments)
-              .collect(Collectors.toList());
+          .filter(segmentZKMetadata -> segmentZKMetadata.getStatus().isCompleted()
+              && segmentZKMetadata.getStartTimeMs() < finalEarliestStartTimeMsOfInProgressSegments)
+          .collect(Collectors.toList());
     } else {
       return allSegments;
     }
@@ -545,25 +583,46 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
   }
 
   /**
-   * Check if the segment is merged for give merge level
+   * Check if the segment is merged for given and higher merge levels
+   *
+   * @param segmentZKMetadata segment zk metadata
+   * @param baseMergeLevel base merge level
+   * @param sortedMergeLevels sorted merge levels based on buffer time period
+   * @return true if the segment is merged for the base merge level or any level higher than the base merge level
    */
-  private boolean isMergedSegment(SegmentZKMetadata segmentZKMetadata, String mergeLevel) {
+  private boolean isMergedSegment(SegmentZKMetadata segmentZKMetadata, String baseMergeLevel,
+      List<String> sortedMergeLevels) {
     Map<String, String> customMap = segmentZKMetadata.getCustomMap();
-    return customMap != null && mergeLevel.equalsIgnoreCase(
-        customMap.get(MergeRollupTask.SEGMENT_ZK_METADATA_MERGE_LEVEL_KEY));
+    if (baseMergeLevel == null) {
+      return true;
+    }
+    if (customMap == null || customMap.get(MergeRollupTask.SEGMENT_ZK_METADATA_MERGE_LEVEL_KEY) == null) {
+      return false;
+    }
+    String mergeLevel = customMap.get(MergeRollupTask.SEGMENT_ZK_METADATA_MERGE_LEVEL_KEY);
+    boolean isCurLevelLowerThanBase = true;
+    for (String currentMergeLevel : sortedMergeLevels) {
+      if (currentMergeLevel.equalsIgnoreCase(baseMergeLevel)) {
+        isCurLevelLowerThanBase = false;
+      }
+      if (!isCurLevelLowerThanBase && currentMergeLevel.equalsIgnoreCase(mergeLevel)) {
+        return true;
+      }
+    }
+    return false;
   }
 
   /**
    * Check if the bucket end time is valid
    */
   private boolean isValidBucketEndTime(long bucketEndMs, long bufferMs, @Nullable String lowerMergeLevel,
-      MergeRollupTaskMetadata mergeRollupTaskMetadata) {
+      MergeRollupTaskMetadata mergeRollupTaskMetadata, boolean processAll) {
     // Check that bucketEndMs <= now - bufferMs
     if (bucketEndMs > System.currentTimeMillis() - bufferMs) {
       return false;
     }
     // Check that bucketEndMs <= waterMark of the lower mergeLevel
-    if (lowerMergeLevel != null) {
+    if (lowerMergeLevel != null && !processAll) {
       Long lowerMergeLevelWatermarkMs = mergeRollupTaskMetadata.getWatermarkMap().get(lowerMergeLevel);
       return lowerMergeLevelWatermarkMs != null && bucketEndMs <= lowerMergeLevelWatermarkMs;
     }
@@ -593,72 +652,77 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
    * Create pinot task configs with selected segments and configs
    */
   private List<PinotTaskConfig> createPinotTaskConfigs(List<SegmentZKMetadata> selectedSegments,
-      String tableNameWithType, int maxNumRecordsPerTask, String mergeLevel, List<Integer> partition,
+      TableConfig tableConfig, int maxNumRecordsPerTask, String mergeLevel, List<Integer> partition,
       Map<String, String> mergeConfigs, Map<String, String> taskConfigs) {
-    int numRecordsPerTask = 0;
-    List<List<String>> segmentNamesList = new ArrayList<>();
-    List<List<String>> downloadURLsList = new ArrayList<>();
-    List<String> segmentNames = new ArrayList<>();
-    List<String> downloadURLs = new ArrayList<>();
-
-    for (int i = 0; i < selectedSegments.size(); i++) {
-      SegmentZKMetadata targetSegment = selectedSegments.get(i);
-      segmentNames.add(targetSegment.getSegmentName());
-      downloadURLs.add(targetSegment.getDownloadUrl());
-      numRecordsPerTask += targetSegment.getTotalDocs();
-      if (numRecordsPerTask >= maxNumRecordsPerTask || i == selectedSegments.size() - 1) {
-        segmentNamesList.add(segmentNames);
-        downloadURLsList.add(downloadURLs);
-        numRecordsPerTask = 0;
-        segmentNames = new ArrayList<>();
-        downloadURLs = new ArrayList<>();
-      }
-    }
-
+    String tableNameWithType = tableConfig.getTableName();
+    List<List<SegmentZKMetadata>> segmentGroups = MergeRollupTaskSegmentGroupManagerProvider.create(taskConfigs)
+        .getSegmentGroups(tableConfig, _clusterInfoAccessor, selectedSegments);
     List<PinotTaskConfig> pinotTaskConfigs = new ArrayList<>();
 
-    StringBuilder partitionSuffixBuilder = new StringBuilder();
-    if (partition != null && !partition.isEmpty()) {
-      for (int columnPartition : partition) {
-        partitionSuffixBuilder.append(DELIMITER_IN_SEGMENT_NAME).append(columnPartition);
-      }
-    }
-    String partitionSuffix = partitionSuffixBuilder.toString();
+    for (List<SegmentZKMetadata> segments : segmentGroups) {
+      int numRecordsPerTask = 0;
+      List<List<String>> segmentNamesList = new ArrayList<>();
+      List<List<String>> downloadURLsList = new ArrayList<>();
+      List<String> segmentNames = new ArrayList<>();
+      List<String> downloadURLs = new ArrayList<>();
 
-    for (int i = 0; i < segmentNamesList.size(); i++) {
-      String downloadURL = StringUtils.join(downloadURLsList.get(i), MinionConstants.URL_SEPARATOR);
-      Map<String, String> configs = MinionTaskUtils.getPushTaskConfig(tableNameWithType, taskConfigs,
-          _clusterInfoAccessor);
-      configs.put(MinionConstants.TABLE_NAME_KEY, tableNameWithType);
-      configs.put(MinionConstants.SEGMENT_NAME_KEY,
-          StringUtils.join(segmentNamesList.get(i), MinionConstants.SEGMENT_NAME_SEPARATOR));
-      configs.put(MinionConstants.DOWNLOAD_URL_KEY, downloadURL);
-      configs.put(MinionConstants.UPLOAD_URL_KEY, _clusterInfoAccessor.getVipUrl() + "/segments");
-      configs.put(MinionConstants.ENABLE_REPLACE_SEGMENTS_KEY, "true");
-
-      for (Map.Entry<String, String> taskConfig : taskConfigs.entrySet()) {
-        if (taskConfig.getKey().endsWith(MinionConstants.MergeRollupTask.AGGREGATION_TYPE_KEY_SUFFIX)) {
-          configs.put(taskConfig.getKey(), taskConfig.getValue());
+      for (int i = 0; i < segments.size(); i++) {
+        SegmentZKMetadata targetSegment = segments.get(i);
+        segmentNames.add(targetSegment.getSegmentName());
+        downloadURLs.add(targetSegment.getDownloadUrl());
+        numRecordsPerTask += targetSegment.getTotalDocs();
+        if (numRecordsPerTask >= maxNumRecordsPerTask || i == segments.size() - 1) {
+          segmentNamesList.add(segmentNames);
+          downloadURLsList.add(downloadURLs);
+          numRecordsPerTask = 0;
+          segmentNames = new ArrayList<>();
+          downloadURLs = new ArrayList<>();
         }
       }
 
-      configs.put(BatchConfigProperties.OVERWRITE_OUTPUT,
-          taskConfigs.getOrDefault(BatchConfigProperties.OVERWRITE_OUTPUT, "false"));
-      configs.put(MergeRollupTask.MERGE_TYPE_KEY, mergeConfigs.get(MergeTask.MERGE_TYPE_KEY));
-      configs.put(MergeRollupTask.MERGE_LEVEL_KEY, mergeLevel);
-      configs.put(MergeTask.PARTITION_BUCKET_TIME_PERIOD_KEY, mergeConfigs.get(MergeTask.BUCKET_TIME_PERIOD_KEY));
-      configs.put(MergeTask.ROUND_BUCKET_TIME_PERIOD_KEY, mergeConfigs.get(MergeTask.ROUND_BUCKET_TIME_PERIOD_KEY));
-      configs.put(MergeTask.MAX_NUM_RECORDS_PER_SEGMENT_KEY,
-          mergeConfigs.get(MergeTask.MAX_NUM_RECORDS_PER_SEGMENT_KEY));
+      StringBuilder partitionSuffixBuilder = new StringBuilder();
+      if (partition != null && !partition.isEmpty()) {
+        for (int columnPartition : partition) {
+          partitionSuffixBuilder.append(DELIMITER_IN_SEGMENT_NAME).append(columnPartition);
+        }
+      }
+      String partitionSuffix = partitionSuffixBuilder.toString();
 
-      // Segment name conflict happens when the current method "createPinotTaskConfigs" is invoked more than once within
-      // the same epoch millisecond, which may happen when there are multiple partitions.
-      // To prevent such name conflict, we include a partitionSeqSuffix to the segment name.
-      configs.put(MergeRollupTask.SEGMENT_NAME_PREFIX_KEY,
-          MergeRollupTask.MERGED_SEGMENT_NAME_PREFIX + mergeLevel + DELIMITER_IN_SEGMENT_NAME
-              + System.currentTimeMillis() + partitionSuffix + DELIMITER_IN_SEGMENT_NAME + i
-              + DELIMITER_IN_SEGMENT_NAME + TableNameBuilder.extractRawTableName(tableNameWithType));
-      pinotTaskConfigs.add(new PinotTaskConfig(MergeRollupTask.TASK_TYPE, configs));
+      for (int i = 0; i < segmentNamesList.size(); i++) {
+        String downloadURL = StringUtils.join(downloadURLsList.get(i), MinionConstants.URL_SEPARATOR);
+        Map<String, String> configs = MinionTaskUtils.getPushTaskConfig(tableNameWithType, taskConfigs,
+            _clusterInfoAccessor);
+        configs.put(MinionConstants.TABLE_NAME_KEY, tableNameWithType);
+        configs.put(MinionConstants.SEGMENT_NAME_KEY,
+            StringUtils.join(segmentNamesList.get(i), MinionConstants.SEGMENT_NAME_SEPARATOR));
+        configs.put(MinionConstants.DOWNLOAD_URL_KEY, downloadURL);
+        configs.put(MinionConstants.UPLOAD_URL_KEY, _clusterInfoAccessor.getVipUrl() + "/segments");
+        configs.put(MinionConstants.ENABLE_REPLACE_SEGMENTS_KEY, "true");
+
+        for (Map.Entry<String, String> taskConfig : taskConfigs.entrySet()) {
+          if (taskConfig.getKey().endsWith(MinionConstants.MergeRollupTask.AGGREGATION_TYPE_KEY_SUFFIX)) {
+            configs.put(taskConfig.getKey(), taskConfig.getValue());
+          }
+        }
+
+        configs.put(BatchConfigProperties.OVERWRITE_OUTPUT,
+            taskConfigs.getOrDefault(BatchConfigProperties.OVERWRITE_OUTPUT, "false"));
+        configs.put(MergeRollupTask.MERGE_TYPE_KEY, mergeConfigs.get(MergeTask.MERGE_TYPE_KEY));
+        configs.put(MergeRollupTask.MERGE_LEVEL_KEY, mergeLevel);
+        configs.put(MergeTask.PARTITION_BUCKET_TIME_PERIOD_KEY, mergeConfigs.get(MergeTask.BUCKET_TIME_PERIOD_KEY));
+        configs.put(MergeTask.ROUND_BUCKET_TIME_PERIOD_KEY, mergeConfigs.get(MergeTask.ROUND_BUCKET_TIME_PERIOD_KEY));
+        configs.put(MergeTask.MAX_NUM_RECORDS_PER_SEGMENT_KEY,
+            mergeConfigs.get(MergeTask.MAX_NUM_RECORDS_PER_SEGMENT_KEY));
+
+        // Segment name conflict happens when the current method "createPinotTaskConfigs" is invoked more than once
+        // within the same epoch millisecond, which may happen when there are multiple partitions.
+        // To prevent such name conflict, we include a partitionSeqSuffix to the segment name.
+        configs.put(MergeRollupTask.SEGMENT_NAME_PREFIX_KEY,
+            MergeRollupTask.MERGED_SEGMENT_NAME_PREFIX + mergeLevel + DELIMITER_IN_SEGMENT_NAME
+                + System.currentTimeMillis() + partitionSuffix + DELIMITER_IN_SEGMENT_NAME + i
+                + DELIMITER_IN_SEGMENT_NAME + TableNameBuilder.extractRawTableName(tableNameWithType));
+        pinotTaskConfigs.add(new PinotTaskConfig(MergeRollupTask.TASK_TYPE, configs));
+      }
     }
 
     return pinotTaskConfigs;
@@ -712,6 +776,98 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
     });
   }
 
+  /**
+   * Update the number buckets to process for the given table and merge level. We create the new gauge metric if
+   * the metric is not available.
+   * @param tableNameWithType table name with type
+   * @param mergeLevel merge level
+   * @param lowerMergeLevel lower merge level
+   * @param bufferTimeMs buffer time
+   * @param bucketTimeMs bucket time
+   * @param sortedSegments sorted segment list
+   * @param sortedMergeLevels sorted merge level list
+   */
+  private void createOrUpdateNumBucketsToProcessMetrics(String tableNameWithType, String mergeLevel,
+      String lowerMergeLevel, long bufferTimeMs, long bucketTimeMs,
+      List<SegmentZKMetadata> sortedSegments, List<String> sortedMergeLevels) {
+    ControllerMetrics controllerMetrics = _clusterInfoAccessor.getControllerMetrics();
+    if (controllerMetrics == null) {
+      return;
+    }
+
+    // Find all buckets and segments that are ready to merge
+    List<List<SegmentZKMetadata>> selectedSegmentsForAllBuckets = new ArrayList<>();
+    List<SegmentZKMetadata> selectedSegmentsForBucket = new ArrayList<>();
+    long bucketStartMs = sortedSegments.get(0).getStartTimeMs() / bucketTimeMs * bucketTimeMs;
+    long bucketEndMs = bucketStartMs + bucketTimeMs;
+    boolean hasUnmergedSegments = false;
+    boolean isAllSegmentsReadyToMerge = true;
+
+    for (SegmentZKMetadata segment : sortedSegments) {
+      long startTimeMs = segment.getStartTimeMs();
+      if (startTimeMs < bucketEndMs) {
+        long endTimeMs = segment.getEndTimeMs();
+        if (endTimeMs >= bucketStartMs) {
+          // For segments overlapping with current bucket, add to the result list
+          if (!isMergedSegment(segment, mergeLevel, sortedMergeLevels)) {
+            hasUnmergedSegments = true;
+          }
+          if (!isMergedSegment(segment, lowerMergeLevel, sortedMergeLevels)) {
+            isAllSegmentsReadyToMerge = false;
+          }
+          selectedSegmentsForBucket.add(segment);
+        }
+        // endTimeMs < bucketStartMs
+        // Haven't find the first overlapping segment, continue to the next segment
+      } else {
+        // Has gone through all overlapping segments for current bucket
+        if (hasUnmergedSegments && isAllSegmentsReadyToMerge) {
+          // Add the bucket if there are unmerged segments
+          selectedSegmentsForAllBuckets.add(selectedSegmentsForBucket);
+        }
+
+        // Start with a new bucket
+        selectedSegmentsForBucket = new ArrayList<>();
+        hasUnmergedSegments = false;
+        isAllSegmentsReadyToMerge = true;
+        bucketStartMs = (startTimeMs / bucketTimeMs) * bucketTimeMs;
+        bucketEndMs = bucketStartMs + bucketTimeMs;
+        if (bucketEndMs > System.currentTimeMillis() - bufferTimeMs) {
+          break;
+        }
+        if (!isMergedSegment(segment, mergeLevel, sortedMergeLevels)) {
+          hasUnmergedSegments = true;
+        }
+        if (!isMergedSegment(segment, lowerMergeLevel, sortedMergeLevels)) {
+          isAllSegmentsReadyToMerge = false;
+        }
+        selectedSegmentsForBucket.add(segment);
+      }
+    }
+
+    // Add the last bucket if it contains unmerged segments and is not added before
+    if (hasUnmergedSegments && isAllSegmentsReadyToMerge && (selectedSegmentsForAllBuckets.isEmpty() || (
+        selectedSegmentsForAllBuckets.get(selectedSegmentsForAllBuckets.size() - 1)
+            != selectedSegmentsForBucket))) {
+      selectedSegmentsForAllBuckets.add(selectedSegmentsForBucket);
+    }
+
+    Map<String, Long> numBucketsToProcessForTable =
+        _tableNumberBucketsToProcess.computeIfAbsent(tableNameWithType, k -> new ConcurrentHashMap<>());
+    long finalCount = selectedSegmentsForAllBuckets.size();
+    numBucketsToProcessForTable.compute(mergeLevel, (k, v) -> {
+      if (v == null) {
+        LOGGER.info(
+            "Creating the gauge metric for tracking the merge/roll-up number buckets to process for table: {} "
+                + "and mergeLevel: {}.(bufferTimeMs={}, bucketTimeMs={}, numTimeBucketsToProcess={})"
+                + tableNameWithType, mergeLevel, bufferTimeMs, bucketTimeMs, finalCount);
+        controllerMetrics.setOrUpdateGauge(getMetricNameForNumBucketsToProcess(tableNameWithType, mergeLevel),
+            () -> _tableNumberBucketsToProcess.get(tableNameWithType).getOrDefault(mergeLevel, finalCount));
+      }
+      return finalCount;
+    });
+  }
+
   /**
    * Reset the delay metrics for the given table name.
    *
@@ -730,6 +886,13 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         controllerMetrics.removeGauge(getMetricNameForTaskDelay(tableNameWithType, mergeLevel));
       }
     }
+
+    Map<String, Long> numBucketsToProcessForTable = _tableNumberBucketsToProcess.remove(tableNameWithType);
+    if (numBucketsToProcessForTable != null) {
+      for (String mergeLevel : numBucketsToProcessForTable.keySet()) {
+        controllerMetrics.removeGauge(getMetricNameForNumBucketsToProcess(tableNameWithType, mergeLevel));
+      }
+    }
   }
 
   /**
@@ -751,6 +914,13 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
         controllerMetrics.removeGauge(getMetricNameForTaskDelay(tableNameWithType, mergeLevel));
       }
     }
+
+    Map<String, Long> numBucketsToProcessForTable = _tableNumberBucketsToProcess.remove(tableNameWithType);
+    if (numBucketsToProcessForTable != null) {
+      if (numBucketsToProcessForTable.remove(mergeLevel) != null) {
+        controllerMetrics.removeGauge(getMetricNameForNumBucketsToProcess(tableNameWithType, mergeLevel));
+      }
+    }
   }
 
   /**
@@ -774,7 +944,10 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       tableConfigMap.put(tableConfig.getTableName(), tableConfig);
     }
 
-    for (String tableNameWithType : new ArrayList<>(_mergeRollupWatermarks.keySet())) {
+    Set<String> tables = new HashSet<>(_mergeRollupWatermarks.keySet());
+    tables.addAll(_tableNumberBucketsToProcess.keySet());
+
+    for (String tableNameWithType : tables) {
       TableConfig currentTableConfig = tableConfigMap.get(tableNameWithType);
       // Table does not exist in the cluster or merge task config is removed
       if (currentTableConfig == null) {
@@ -792,9 +965,20 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       Map<String, String> taskConfigs = currentTableConfig.getTaskConfig().getConfigsForTaskType(getTaskType());
       Map<String, Map<String, String>> mergeLevelToConfigs = MergeRollupTaskUtils.getLevelToConfigMap(taskConfigs);
       Map<String, Long> tableToWatermark = _mergeRollupWatermarks.get(tableNameWithType);
-      for (String mergeLevel : tableToWatermark.keySet()) {
-        if (!mergeLevelToConfigs.containsKey(mergeLevel)) {
-          resetDelayMetrics(tableNameWithType, mergeLevel);
+      if (tableToWatermark != null) {
+        for (String mergeLevel : tableToWatermark.keySet()) {
+          if (!mergeLevelToConfigs.containsKey(mergeLevel)) {
+            resetDelayMetrics(tableNameWithType, mergeLevel);
+          }
+        }
+      }
+
+      Map<String, Long> tableToNumBucketsToProcess = _tableNumberBucketsToProcess.get(tableNameWithType);
+      if (tableToNumBucketsToProcess != null) {
+        for (String mergeLevel : tableToNumBucketsToProcess.keySet()) {
+          if (!mergeLevelToConfigs.containsKey(mergeLevel)) {
+            resetDelayMetrics(tableNameWithType, mergeLevel);
+          }
         }
       }
     }
@@ -804,4 +988,8 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
     // e.g. mergeRollupTaskDelayInNumBuckets.myTable_OFFLINE.daily
     return MERGE_ROLLUP_TASK_DELAY_IN_NUM_BUCKETS + "." + tableNameWithType + "." + mergeLevel;
   }
+
+  private String getMetricNameForNumBucketsToProcess(String tableNameWithType, String mergeLevel) {
+    return MERGE_ROLLUP_TASK_NUM_BUCKETS_TO_PROCESS + "." + tableNameWithType + "." + mergeLevel;
+  }
 }
