@@ -28,6 +28,7 @@ import org.apache.pinot.query.planner.serde.ProtoProperties;
  * The {@code FieldSelectionKeySelector} simply extract a column value out from a row array {@link Object[]}.
  */
 public class FieldSelectionKeySelector implements KeySelector<Object[], Object[]> {
+  private static final String HASH_ALGORITHM = "absHashCode";
 
   @ProtoProperties
   private List<Integer> _columnIndices;
@@ -84,10 +85,18 @@ public class FieldSelectionKeySelector implements KeySelector<Object[], Object[]
     // TODO: consider better hashing algorithms than hashCode sum, such as XOR'ing
     int hashCode = 0;
     for (int columnIndex : _columnIndices) {
-      hashCode += input[columnIndex].hashCode();
+      Object value = input[columnIndex];
+      if (value != null) {
+        hashCode += value.hashCode();
+      }
     }
 
     // return a positive number because this is used directly to modulo-index
-    return Math.abs(hashCode);
+    return hashCode & Integer.MAX_VALUE;
+  }
+
+  @Override
+  public String hashAlgorithm() {
+    return HASH_ALGORITHM;
   }
 }
