@@ -19,12 +19,9 @@
 package org.apache.pinot.query.mailbox.channel;
 
 public class ChannelUtils {
+  private ChannelUtils() {
+  }
 
   public static final String MAILBOX_METADATA_BUFFER_SIZE_KEY = "buffer.size";
   public static final String MAILBOX_METADATA_BEGIN_OF_STREAM_KEY = "begin.of.stream";
-  public static final String MAILBOX_METADATA_END_OF_STREAM_KEY = "end.of.stream";
-
-  private ChannelUtils() {
-    // do not instantiate.
-  }
 }
