@@ -19,18 +19,18 @@
 package org.apache.pinot.query.runtime.operator;
 
 import com.google.common.base.Preconditions;
+import com.google.common.collect.ImmutableList;
 import java.util.ArrayList;
 import java.util.List;
 import javax.annotation.Nullable;
 import org.apache.pinot.common.datablock.DataBlock;
 import org.apache.pinot.common.utils.DataSchema;
-import org.apache.pinot.core.common.Operator;
-import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.query.planner.logical.RexExpression;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
 import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperand;
-import org.apache.pinot.query.runtime.operator.utils.FunctionInvokeUtils;
+import org.apache.pinot.query.runtime.operator.operands.TransformOperandFactory;
+import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
 
 
 /**
@@ -43,17 +43,19 @@ import org.apache.pinot.query.runtime.operator.utils.FunctionInvokeUtils;
  * Note: Function transform only runs functions from v1 engine scalar function factory, which only does argument count
  * and canonicalized function name matching (lower case).
  */
-public class TransformOperator extends BaseOperator<TransferableBlock> {
+public class TransformOperator extends MultiStageOperator {
   private static final String EXPLAIN_NAME = "TRANSFORM";
-  private final Operator<TransferableBlock> _upstreamOperator;
+
+  private final MultiStageOperator _upstreamOperator;
   private final List<TransformOperand> _transformOperandsList;
   private final int _resultColumnSize;
   // TODO: Check type matching between resultSchema and the actual result.
   private final DataSchema _resultSchema;
   private TransferableBlock _upstreamErrorBlock;
 
-  public TransformOperator(Operator<TransferableBlock> upstreamOperator, DataSchema resultSchema,
-      List<RexExpression> transforms, DataSchema upstreamDataSchema) {
+  public TransformOperator(OpChainExecutionContext context, MultiStageOperator upstreamOperator,
+      DataSchema resultSchema, List<RexExpression> transforms, DataSchema upstreamDataSchema) {
+    super(context);
     Preconditions.checkState(!transforms.isEmpty(), "transform operand should not be empty.");
     Preconditions.checkState(resultSchema.size() == transforms.size(),
         "result schema size:" + resultSchema.size() + " doesn't match transform operand size:" + transforms.size());
@@ -61,15 +63,14 @@ public class TransformOperator extends BaseOperator<TransferableBlock> {
     _resultColumnSize = transforms.size();
     _transformOperandsList = new ArrayList<>(_resultColumnSize);
     for (RexExpression rexExpression : transforms) {
-      _transformOperandsList.add(TransformOperand.toTransformOperand(rexExpression, upstreamDataSchema));
+      _transformOperandsList.add(TransformOperandFactory.getTransformOperand(rexExpression, upstreamDataSchema));
     }
     _resultSchema = resultSchema;
   }
 
   @Override
-  public List<Operator> getChildOperators() {
-    // WorkerExecutor doesn't use getChildOperators, returns null here.
-    return null;
+  public List<MultiStageOperator> getChildOperators() {
+    return ImmutableList.of(_upstreamOperator);
   }
 
   @Nullable
@@ -81,14 +82,16 @@ public class TransformOperator extends BaseOperator<TransferableBlock> {
   @Override
   protected TransferableBlock getNextBlock() {
     try {
-      return transform(_upstreamOperator.nextBlock());
-    } catch (Exception e) {
+      TransferableBlock block = _upstreamOperator.nextBlock();
+      return transform(block);
+    } catch (RuntimeException e) {
       return TransferableBlockUtils.getErrorTransferableBlock(e);
     }
   }
 
-  private TransferableBlock transform(TransferableBlock block)
-      throws Exception {
+  private TransferableBlock transform(TransferableBlock block) {
+    // TODO: Other operators keep the first erroneous block, while this keep the last.
+    //  We should decide what is what we want to do and be consistent with that.
     if (block.isErrorBlock()) {
       _upstreamErrorBlock = block;
     }
@@ -96,17 +99,16 @@ public class TransformOperator extends BaseOperator<TransferableBlock> {
       return _upstreamErrorBlock;
     }
 
-    if (TransferableBlockUtils.isEndOfStream(block) || TransferableBlockUtils.isNoOpBlock(block)) {
+    if (TransferableBlockUtils.isEndOfStream(block)) {
       return block;
     }
 
-    List<Object[]> resultRows = new ArrayList<>();
     List<Object[]> container = block.getContainer();
+    List<Object[]> resultRows = new ArrayList<>(container.size());
     for (Object[] row : container) {
       Object[] resultRow = new Object[_resultColumnSize];
       for (int i = 0; i < _resultColumnSize; i++) {
-        resultRow[i] = FunctionInvokeUtils.convert(_transformOperandsList.get(i).apply(row),
-            _resultSchema.getColumnDataType(i));
+        resultRow[i] = _transformOperandsList.get(i).apply(row);
       }
       resultRows.add(resultRow);
     }
