@@ -27,8 +27,9 @@ import java.nio.file.Files;
 import java.nio.file.attribute.PosixFilePermission;
 import java.util.EnumSet;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
-import java.util.Set;
+import java.util.stream.Collectors;
 import org.apache.commons.configuration.ConfigurationException;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.commons.io.FileUtils;
@@ -36,12 +37,14 @@ import org.apache.pinot.segment.local.segment.index.loader.IndexLoadingConfig;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.converter.SegmentFormatConverter;
 import org.apache.pinot.segment.spi.creator.SegmentVersion;
+import org.apache.pinot.segment.spi.index.IndexService;
+import org.apache.pinot.segment.spi.index.IndexType;
+import org.apache.pinot.segment.spi.index.StandardIndexes;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.segment.spi.index.startree.StarTreeV2Constants;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderContext;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
 import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
-import org.apache.pinot.segment.spi.store.ColumnIndexType;
 import org.apache.pinot.segment.spi.store.SegmentDirectory;
 import org.apache.pinot.segment.spi.store.SegmentDirectoryPaths;
 import org.apache.pinot.spi.env.CommonsConfigurationUtils;
@@ -66,7 +69,7 @@ public class SegmentV1V2ToV3FormatConverter implements SegmentFormatConverter {
     Preconditions.checkNotNull(v2SegmentDirectory, "Segment directory should not be null");
 
     Preconditions.checkState(v2SegmentDirectory.exists() && v2SegmentDirectory.isDirectory(),
-        "Segment directory: " + v2SegmentDirectory.toString() + " must exist and should be a directory");
+        "Segment directory: " + v2SegmentDirectory + " must exist and should be a directory");
 
     LOGGER.info("Converting segment: {} to v3 format", v2SegmentDirectory);
 
@@ -86,7 +89,6 @@ public class SegmentV1V2ToV3FormatConverter implements SegmentFormatConverter {
 
     createMetadataFile(v2SegmentDirectory, v3TempDirectory);
     copyCreationMetadataIfExists(v2SegmentDirectory, v3TempDirectory);
-    copyLuceneTextIndexIfExists(v2SegmentDirectory, v3TempDirectory);
     copyIndexData(v2SegmentDirectory, v2Metadata, v3TempDirectory);
 
     File newLocation = SegmentDirectoryPaths.segmentDirectoryFor(v2SegmentDirectory, SegmentVersion.v3);
@@ -147,36 +149,31 @@ public class SegmentV1V2ToV3FormatConverter implements SegmentFormatConverter {
         SegmentDirectory v3Segment = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
             .load(v3Directory.toURI(), new SegmentDirectoryLoaderContext.Builder().setSegmentName(v2Metadata.getName())
                 .setSegmentDirectoryConfigs(configuration).build())) {
-
-      // for each dictionary and each fwdIndex, copy that to newDirectory buffer
-      Set<String> allColumns = v2Metadata.getAllColumns();
       try (SegmentDirectory.Reader v2DataReader = v2Segment.createReader();
           SegmentDirectory.Writer v3DataWriter = v3Segment.createWriter()) {
-        for (String column : allColumns) {
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.DICTIONARY);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.FORWARD_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.NULLVALUE_VECTOR);
+        for (String column : v2Metadata.getAllColumns()) {
+          for (IndexType<?, ?, ?> indexType : sortedIndexTypes()) {
+            // NOTE: Text index is copied separately
+            if (indexType != StandardIndexes.text()) {
+              copyIndexIfExists(v2DataReader, v3DataWriter, column, indexType);
+            }
+          }
         }
-
-        // Other indexes are intentionally stored at the end of the single file
-        for (String column : allColumns) {
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.INVERTED_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.FST_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.JSON_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.H3_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.RANGE_INDEX);
-          copyIndexIfExists(v2DataReader, v3DataWriter, column, ColumnIndexType.BLOOM_FILTER);
-        }
-
         v3DataWriter.save();
       }
     }
-
+    copyLuceneTextIndexIfExists(v2Directory, v3Directory);
     copyStarTreeV2(v2Directory, v3Directory);
   }
 
+  private List<IndexType<?, ?, ?>> sortedIndexTypes() {
+    return IndexService.getInstance().getAllIndexes().stream()
+        .sorted((i1, i2) -> i1.getId().compareTo(i2.getId()))
+        .collect(Collectors.toList());
+  }
+
   private void copyIndexIfExists(SegmentDirectory.Reader reader, SegmentDirectory.Writer writer, String column,
-      ColumnIndexType indexType)
+      IndexType indexType)
       throws IOException {
     if (reader.hasIndexFor(column, indexType)) {
       readCopyBuffers(reader, writer, column, indexType);
@@ -194,7 +191,7 @@ public class SegmentV1V2ToV3FormatConverter implements SegmentFormatConverter {
   }
 
   private void readCopyBuffers(SegmentDirectory.Reader reader, SegmentDirectory.Writer writer, String column,
-      ColumnIndexType indexType)
+      IndexType indexType)
       throws IOException {
     PinotDataBuffer oldBuffer = reader.getIndexFor(column, indexType);
     long oldBufferSize = oldBuffer.size();
