@@ -22,13 +22,13 @@ import com.google.common.base.Preconditions;
 import it.unimi.dsi.fastutil.ints.IntSet;
 import java.io.IOException;
 import java.math.BigDecimal;
-import java.util.Arrays;
+import java.nio.charset.StandardCharsets;
+import java.util.List;
 import org.apache.pinot.segment.local.io.util.FixedByteValueReaderWriter;
 import org.apache.pinot.segment.local.io.util.ValueReader;
 import org.apache.pinot.segment.local.io.util.VarLengthValueReader;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
 import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
-import org.apache.pinot.spi.utils.ByteArray;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 
@@ -41,21 +41,18 @@ public abstract class BaseImmutableDictionary implements Dictionary {
   private final ValueReader _valueReader;
   private final int _length;
   private final int _numBytesPerValue;
-  private final byte _paddingByte;
 
-  protected BaseImmutableDictionary(PinotDataBuffer dataBuffer, int length, int numBytesPerValue, byte paddingByte) {
+  protected BaseImmutableDictionary(PinotDataBuffer dataBuffer, int length, int numBytesPerValue) {
     if (VarLengthValueReader.isVarLengthValueBuffer(dataBuffer)) {
       VarLengthValueReader valueReader = new VarLengthValueReader(dataBuffer);
       _valueReader = valueReader;
       _length = valueReader.getNumValues();
-      _paddingByte = 0;
     } else {
       Preconditions.checkState(dataBuffer.size() == (long) length * numBytesPerValue,
           "Buffer size mismatch: bufferSize = %s, numValues = %s, numByesPerValue = %s", dataBuffer.size(), length,
           numBytesPerValue);
       _valueReader = new FixedByteValueReaderWriter(dataBuffer);
       _length = length;
-      _paddingByte = paddingByte;
     }
     _numBytesPerValue = numBytesPerValue;
   }
@@ -67,7 +64,6 @@ public abstract class BaseImmutableDictionary implements Dictionary {
     _valueReader = null;
     _length = length;
     _numBytesPerValue = -1;
-    _paddingByte = 0;
   }
 
   @Override
@@ -211,56 +207,13 @@ public abstract class BaseImmutableDictionary implements Dictionary {
     return -(low + 1);
   }
 
-  /**
-   * WARNING: With non-zero padding byte, binary search result might not reflect the real insertion index for the value.
-   * E.g. with padding byte 'b', if unpadded value "aa" is in the dictionary, and stored as "aab", then unpadded value
-   * "a" will be mis-positioned after value "aa"; unpadded value "aab" will return positive value even if value "aab" is
-   * not in the dictionary.
-   * TODO: Clean up the segments with legacy non-zero padding byte, and remove the support for non-zero padding byte
-   */
   protected int binarySearch(String value) {
-    byte[] buffer = getBuffer();
     int low = 0;
     int high = _length - 1;
-    if (_paddingByte == 0) {
-      while (low <= high) {
-        int mid = (low + high) >>> 1;
-        String midValue = _valueReader.getUnpaddedString(mid, _numBytesPerValue, _paddingByte, buffer);
-        int compareResult = midValue.compareTo(value);
-        if (compareResult < 0) {
-          low = mid + 1;
-        } else if (compareResult > 0) {
-          high = mid - 1;
-        } else {
-          return mid;
-        }
-      }
-    } else {
-      String paddedValue = padString(value);
-      while (low <= high) {
-        int mid = (low + high) >>> 1;
-        String midValue = _valueReader.getPaddedString(mid, _numBytesPerValue, buffer);
-        int compareResult = midValue.compareTo(paddedValue);
-        if (compareResult < 0) {
-          low = mid + 1;
-        } else if (compareResult > 0) {
-          high = mid - 1;
-        } else {
-          return mid;
-        }
-      }
-    }
-    return -(low + 1);
-  }
-
-  protected int binarySearch(byte[] value) {
-    int low = 0;
-    int high = _length - 1;
-
+    byte[] utf8 = value.getBytes(UTF_8);
     while (low <= high) {
       int mid = (low + high) >>> 1;
-      byte[] midValue = _valueReader.getBytes(mid, _numBytesPerValue);
-      int compareResult = ByteArray.compare(midValue, value);
+      int compareResult = _valueReader.compareUtf8Bytes(mid, _numBytesPerValue, utf8);
       if (compareResult < 0) {
         low = mid + 1;
       } else if (compareResult > 0) {
@@ -272,19 +225,21 @@ public abstract class BaseImmutableDictionary implements Dictionary {
     return -(low + 1);
   }
 
-  protected String padString(String value) {
-    byte[] valueBytes = value.getBytes(UTF_8);
-    int length = valueBytes.length;
-    String paddedValue;
-    if (length >= _numBytesPerValue) {
-      paddedValue = value;
-    } else {
-      byte[] paddedValueBytes = new byte[_numBytesPerValue];
-      System.arraycopy(valueBytes, 0, paddedValueBytes, 0, length);
-      Arrays.fill(paddedValueBytes, length, _numBytesPerValue, _paddingByte);
-      paddedValue = new String(paddedValueBytes, UTF_8);
+  protected int binarySearch(byte[] value) {
+    int low = 0;
+    int high = _length - 1;
+    while (low <= high) {
+      int mid = (low + high) >>> 1;
+      int compareResult = _valueReader.compareBytes(mid, _numBytesPerValue, value);
+      if (compareResult < 0) {
+        low = mid + 1;
+      } else if (compareResult > 0) {
+        high = mid - 1;
+      } else {
+        return mid;
+      }
     }
-    return paddedValue;
+    return -(low + 1);
   }
 
   protected int getInt(int dictId) {
@@ -308,11 +263,11 @@ public abstract class BaseImmutableDictionary implements Dictionary {
   }
 
   protected byte[] getUnpaddedBytes(int dictId, byte[] buffer) {
-    return _valueReader.getUnpaddedBytes(dictId, _numBytesPerValue, _paddingByte, buffer);
+    return _valueReader.getUnpaddedBytes(dictId, _numBytesPerValue, buffer);
   }
 
   protected String getUnpaddedString(int dictId, byte[] buffer) {
-    return _valueReader.getUnpaddedString(dictId, _numBytesPerValue, _paddingByte, buffer);
+    return _valueReader.getUnpaddedString(dictId, _numBytesPerValue, buffer);
   }
 
   protected String getPaddedString(int dictId, byte[] buffer) {
@@ -326,4 +281,37 @@ public abstract class BaseImmutableDictionary implements Dictionary {
   protected byte[] getBuffer() {
     return new byte[_numBytesPerValue];
   }
+
+  /**
+   * Returns the dictionary id for the given sorted values.
+   * @param sortedValues
+   * @param dictIds
+   */
+  @Override
+  public void getDictIds(List<String> sortedValues, IntSet dictIds) {
+    int valueIdx = 0;
+    int dictIdx = 0;
+    byte[] utf8 = null;
+    boolean needNewUtf8 = true;
+    int sortedValuesSize = sortedValues.size();
+    int dictLength = length();
+    while (valueIdx < sortedValuesSize && dictIdx < dictLength) {
+      if (needNewUtf8) {
+        utf8 = sortedValues.get(valueIdx).getBytes(StandardCharsets.UTF_8);
+      }
+      int comparison = _valueReader.compareUtf8Bytes(dictIdx, _numBytesPerValue, utf8);
+      if (comparison == 0) {
+        dictIds.add(dictIdx);
+        dictIdx++;
+        valueIdx++;
+        needNewUtf8 = true;
+      } else if (comparison > 0) {
+        valueIdx++;
+        needNewUtf8 = true;
+      } else {
+        dictIdx++;
+        needNewUtf8 = false;
+      }
+    }
+  }
 }
