@@ -20,11 +20,16 @@ package org.apache.pinot.segment.local.indexsegment.mutable;
 
 import java.io.File;
 import java.net.URL;
+import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.concurrent.ExecutorService;
+import org.apache.helix.HelixManager;
 import org.apache.pinot.common.metrics.ServerMetrics;
 import org.apache.pinot.segment.local.data.manager.TableDataManager;
 import org.apache.pinot.segment.local.recordtransformer.CompositeTransformer;
 import org.apache.pinot.segment.local.upsert.PartitionUpsertMetadataManager;
+import org.apache.pinot.segment.local.upsert.TableUpsertMetadataManager;
 import org.apache.pinot.segment.local.upsert.TableUpsertMetadataManagerFactory;
 import org.apache.pinot.spi.config.table.HashFunction;
 import org.apache.pinot.spi.config.table.TableConfig;
@@ -52,21 +57,35 @@ public class MutableSegmentImplUpsertTest {
   private MutableSegmentImpl _mutableSegmentImpl;
   private PartitionUpsertMetadataManager _partitionUpsertMetadataManager;
 
-  private void setup(HashFunction hashFunction)
+  private UpsertConfig createPartialUpsertConfig(HashFunction hashFunction) {
+    UpsertConfig upsertConfigWithHash = new UpsertConfig(UpsertConfig.Mode.PARTIAL);
+    upsertConfigWithHash.setPartialUpsertStrategies(new HashMap<>());
+    upsertConfigWithHash.setDefaultPartialUpsertStrategy(UpsertConfig.Strategy.OVERWRITE);
+    upsertConfigWithHash.setComparisonColumns(Arrays.asList("secondsSinceEpoch", "otherComparisonColumn"));
+    upsertConfigWithHash.setHashFunction(hashFunction);
+    return upsertConfigWithHash;
+  }
+
+  private UpsertConfig createFullUpsertConfig(HashFunction hashFunction) {
+    UpsertConfig upsertConfigWithHash = new UpsertConfig(UpsertConfig.Mode.FULL);
+    upsertConfigWithHash.setHashFunction(hashFunction);
+    return upsertConfigWithHash;
+  }
+
+  private void setup(UpsertConfig upsertConfigWithHash)
       throws Exception {
     URL schemaResourceUrl = this.getClass().getClassLoader().getResource(SCHEMA_FILE_PATH);
     URL dataResourceUrl = this.getClass().getClassLoader().getResource(DATA_FILE_PATH);
     _schema = Schema.fromFile(new File(schemaResourceUrl.getFile()));
-    UpsertConfig upsertConfigWithHash = new UpsertConfig(UpsertConfig.Mode.FULL);
-    upsertConfigWithHash.setHashFunction(hashFunction);
     _tableConfig =
         new TableConfigBuilder(TableType.REALTIME).setTableName("testTable").setUpsertConfig(upsertConfigWithHash)
-            .build();
+            .setNullHandlingEnabled(true).build();
     _recordTransformer = CompositeTransformer.getDefaultTransformer(_tableConfig, _schema);
     File jsonFile = new File(dataResourceUrl.getFile());
-    _partitionUpsertMetadataManager =
-        TableUpsertMetadataManagerFactory.create(_tableConfig, _schema, mock(TableDataManager.class),
-            mock(ServerMetrics.class)).getOrCreatePartitionManager(0);
+    TableUpsertMetadataManager tableUpsertMetadataManager = TableUpsertMetadataManagerFactory.create(_tableConfig);
+    tableUpsertMetadataManager.init(_tableConfig, _schema, mock(TableDataManager.class), mock(ServerMetrics.class),
+        mock(HelixManager.class), mock(ExecutorService.class));
+    _partitionUpsertMetadataManager = tableUpsertMetadataManager.getOrCreatePartitionManager(0);
     _mutableSegmentImpl =
         MutableSegmentImplTestUtils.createMutableSegmentImpl(_schema, Collections.emptySet(), Collections.emptySet(),
             Collections.emptySet(), false, true, upsertConfigWithHash, "secondsSinceEpoch",
@@ -87,18 +106,54 @@ public class MutableSegmentImplUpsertTest {
   @Test
   public void testHashFunctions()
       throws Exception {
-    testUpsertIngestion(HashFunction.NONE);
-    testUpsertIngestion(HashFunction.MD5);
-    testUpsertIngestion(HashFunction.MURMUR3);
+    testUpsertIngestion(createFullUpsertConfig(HashFunction.NONE));
+    testUpsertIngestion(createFullUpsertConfig(HashFunction.MD5));
+    testUpsertIngestion(createFullUpsertConfig(HashFunction.MURMUR3));
   }
 
-  private void testUpsertIngestion(HashFunction hashFunction)
+  @Test
+  public void testMultipleComparisonColumns()
       throws Exception {
-    setup(hashFunction);
+    testUpsertIngestion(createPartialUpsertConfig(HashFunction.NONE));
+    testUpsertIngestion(createPartialUpsertConfig(HashFunction.MD5));
+    testUpsertIngestion(createPartialUpsertConfig(HashFunction.MURMUR3));
+  }
+
+  private void testUpsertIngestion(UpsertConfig upsertConfig)
+      throws Exception {
+    setup(upsertConfig);
     ImmutableRoaringBitmap bitmap = _mutableSegmentImpl.getValidDocIds().getMutableRoaringBitmap();
-    Assert.assertFalse(bitmap.contains(0));
-    Assert.assertTrue(bitmap.contains(1));
-    Assert.assertTrue(bitmap.contains(2));
-    Assert.assertFalse(bitmap.contains(3));
+    if (upsertConfig.getComparisonColumns() == null) {
+      // aa
+      Assert.assertFalse(bitmap.contains(0));
+      Assert.assertTrue(bitmap.contains(1));
+      Assert.assertFalse(bitmap.contains(2));
+      Assert.assertFalse(bitmap.contains(3));
+      // bb
+      Assert.assertFalse(bitmap.contains(4));
+      Assert.assertTrue(bitmap.contains(5));
+      Assert.assertFalse(bitmap.contains(6));
+    } else {
+      // aa
+      Assert.assertFalse(bitmap.contains(0));
+      Assert.assertFalse(bitmap.contains(1));
+      Assert.assertTrue(bitmap.contains(2));
+      Assert.assertFalse(bitmap.contains(3));
+      // Confirm that both comparison column values have made it into the persisted upserted doc
+      Assert.assertEquals(1567205397L, _mutableSegmentImpl.getValue(2, "secondsSinceEpoch"));
+      Assert.assertEquals(1567205395L, _mutableSegmentImpl.getValue(2, "otherComparisonColumn"));
+      Assert.assertFalse(_mutableSegmentImpl.getDataSource("secondsSinceEpoch").getNullValueVector().isNull(2));
+      Assert.assertFalse(_mutableSegmentImpl.getDataSource("otherComparisonColumn").getNullValueVector().isNull(2));
+
+      // bb
+      Assert.assertFalse(bitmap.contains(4));
+      Assert.assertTrue(bitmap.contains(5));
+      Assert.assertFalse(bitmap.contains(6));
+      // Confirm that comparison column values have made it into the persisted upserted doc
+      Assert.assertEquals(1567205396L, _mutableSegmentImpl.getValue(5, "secondsSinceEpoch"));
+      Assert.assertEquals(Long.MIN_VALUE, _mutableSegmentImpl.getValue(5, "otherComparisonColumn"));
+      Assert.assertTrue(_mutableSegmentImpl.getDataSource("otherComparisonColumn").getNullValueVector().isNull(5));
+      Assert.assertFalse(_mutableSegmentImpl.getDataSource("secondsSinceEpoch").getNullValueVector().isNull(5));
+    }
   }
 }
