@@ -20,7 +20,6 @@ package org.apache.pinot.segment.local.segment.index.loader;
 
 import java.io.File;
 import java.net.URL;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -28,27 +27,28 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import org.apache.commons.io.FileUtils;
-import org.apache.pinot.common.utils.TarGzCompressionUtils;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentLoader;
 import org.apache.pinot.segment.local.segment.creator.SegmentTestUtils;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentCreationDriverFactory;
 import org.apache.pinot.segment.local.segment.index.converter.SegmentV1V2ToV3FormatConverter;
-import org.apache.pinot.segment.local.segment.index.readers.StringDictionary;
-import org.apache.pinot.segment.spi.ColumnMetadata;
+import org.apache.pinot.segment.local.segment.index.text.TextIndexConfigBuilder;
 import org.apache.pinot.segment.spi.ImmutableSegment;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.segment.spi.creator.SegmentIndexCreationDriver;
 import org.apache.pinot.segment.spi.creator.SegmentVersion;
+import org.apache.pinot.segment.spi.index.ForwardIndexConfig;
+import org.apache.pinot.segment.spi.index.FstIndexConfig;
+import org.apache.pinot.segment.spi.index.StandardIndexes;
+import org.apache.pinot.segment.spi.index.TextIndexConfig;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoader;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderContext;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
-import org.apache.pinot.segment.spi.memory.PinotDataBuffer;
-import org.apache.pinot.segment.spi.store.ColumnIndexType;
 import org.apache.pinot.segment.spi.store.SegmentDirectory;
 import org.apache.pinot.segment.spi.store.SegmentDirectoryPaths;
+import org.apache.pinot.spi.config.table.IndexConfig;
 import org.apache.pinot.spi.data.DimensionFieldSpec;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.MetricFieldSpec;
@@ -57,7 +57,6 @@ import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.BytesUtils;
 import org.apache.pinot.spi.utils.CommonConstants.Segment.BuiltInVirtualColumn;
 import org.apache.pinot.spi.utils.ReadMode;
-import org.apache.pinot.util.TestUtils;
 import org.testng.Assert;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
@@ -65,7 +64,6 @@ import org.testng.annotations.Test;
 import org.testng.collections.Lists;
 
 import static org.apache.pinot.segment.spi.V1Constants.Indexes.FST_INDEX_FILE_EXTENSION;
-import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertTrue;
 
@@ -73,9 +71,6 @@ import static org.testng.Assert.assertTrue;
 public class LoaderTest {
   private static final File INDEX_DIR = new File(LoaderTest.class.getName());
   private static final String AVRO_DATA = "data/test_data-mv.avro";
-  private static final String PADDING_OLD = "data/paddingOld.tar.gz";
-  private static final String PADDING_PERCENT = "data/paddingPercent.tar.gz";
-  private static final String PADDING_NULL = "data/paddingNull.tar.gz";
 
   private static final String TEXT_INDEX_COL_NAME = "column5";
   private static final String FST_INDEX_COL_NAME = "column5";
@@ -222,77 +217,6 @@ public class LoaderTest {
     Assert.assertNotNull(indexSegment.getDataSource(BuiltInVirtualColumn.SEGMENTNAME));
   }
 
-  @Test
-  public void testPadding()
-      throws Exception {
-    // Old Format
-    URL resourceUrl = LoaderTest.class.getClassLoader().getResource(PADDING_OLD);
-    Assert.assertNotNull(resourceUrl);
-    File segmentDirectory =
-        TarGzCompressionUtils.untar(new File(TestUtils.getFileFromResourceUrl(resourceUrl)), INDEX_DIR).get(0);
-    SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(segmentDirectory);
-    ColumnMetadata columnMetadata = segmentMetadata.getColumnMetadataFor("name");
-    Assert.assertEquals(columnMetadata.getPaddingCharacter(), V1Constants.Str.LEGACY_STRING_PAD_CHAR);
-    SegmentDirectory segmentDir = _localSegmentDirectoryLoader.load(segmentDirectory.toURI(),
-        new SegmentDirectoryLoaderContext.Builder().setSegmentName(segmentMetadata.getName())
-            .setSegmentDirectoryConfigs(_pinotConfiguration).build());
-    SegmentDirectory.Reader reader = segmentDir.createReader();
-    PinotDataBuffer dictionaryBuffer = reader.getIndexFor("name", ColumnIndexType.DICTIONARY);
-    StringDictionary dict =
-        new StringDictionary(dictionaryBuffer, columnMetadata.getCardinality(), columnMetadata.getColumnMaxLength(),
-            (byte) columnMetadata.getPaddingCharacter());
-    Assert.assertEquals(dict.getStringValue(0), "lynda 2.0");
-    Assert.assertEquals(dict.getStringValue(1), "lynda");
-    Assert.assertEquals(dict.get(0), "lynda 2.0");
-    Assert.assertEquals(dict.get(1), "lynda");
-    Assert.assertEquals(dict.indexOf("lynda%"), 1);
-    Assert.assertEquals(dict.indexOf("lynda%%"), 1);
-
-    // New Format Padding character %
-    resourceUrl = LoaderTest.class.getClassLoader().getResource(PADDING_PERCENT);
-    Assert.assertNotNull(resourceUrl);
-    segmentDirectory =
-        TarGzCompressionUtils.untar(new File(TestUtils.getFileFromResourceUrl(resourceUrl)), INDEX_DIR).get(0);
-    segmentMetadata = new SegmentMetadataImpl(segmentDirectory);
-    columnMetadata = segmentMetadata.getColumnMetadataFor("name");
-    Assert.assertEquals(columnMetadata.getPaddingCharacter(), V1Constants.Str.LEGACY_STRING_PAD_CHAR);
-    segmentDir = _localSegmentDirectoryLoader.load(segmentDirectory.toURI(),
-        new SegmentDirectoryLoaderContext.Builder().setSegmentName(segmentMetadata.getName())
-            .setSegmentDirectoryConfigs(_pinotConfiguration).build());
-    reader = segmentDir.createReader();
-    dictionaryBuffer = reader.getIndexFor("name", ColumnIndexType.DICTIONARY);
-    dict = new StringDictionary(dictionaryBuffer, columnMetadata.getCardinality(), columnMetadata.getColumnMaxLength(),
-        (byte) columnMetadata.getPaddingCharacter());
-    Assert.assertEquals(dict.getStringValue(0), "lynda 2.0");
-    Assert.assertEquals(dict.getStringValue(1), "lynda");
-    Assert.assertEquals(dict.get(0), "lynda 2.0");
-    Assert.assertEquals(dict.get(1), "lynda");
-    Assert.assertEquals(dict.indexOf("lynda%"), 1);
-    Assert.assertEquals(dict.indexOf("lynda%%"), 1);
-
-    // New Format Padding character Null
-    resourceUrl = LoaderTest.class.getClassLoader().getResource(PADDING_NULL);
-    Assert.assertNotNull(resourceUrl);
-    segmentDirectory =
-        TarGzCompressionUtils.untar(new File(TestUtils.getFileFromResourceUrl(resourceUrl)), INDEX_DIR).get(0);
-    segmentMetadata = new SegmentMetadataImpl(segmentDirectory);
-    columnMetadata = segmentMetadata.getColumnMetadataFor("name");
-    Assert.assertEquals(columnMetadata.getPaddingCharacter(), V1Constants.Str.DEFAULT_STRING_PAD_CHAR);
-    segmentDir = _localSegmentDirectoryLoader.load(segmentDirectory.toURI(),
-        new SegmentDirectoryLoaderContext.Builder().setSegmentName(segmentMetadata.getName())
-            .setSegmentDirectoryConfigs(_pinotConfiguration).build());
-    reader = segmentDir.createReader();
-    dictionaryBuffer = reader.getIndexFor("name", ColumnIndexType.DICTIONARY);
-    dict = new StringDictionary(dictionaryBuffer, columnMetadata.getCardinality(), columnMetadata.getColumnMaxLength(),
-        (byte) columnMetadata.getPaddingCharacter());
-    Assert.assertEquals(dict.getStringValue(0), "lynda");
-    Assert.assertEquals(dict.getStringValue(1), "lynda 2.0");
-    Assert.assertEquals(dict.get(0), "lynda");
-    Assert.assertEquals(dict.get(1), "lynda 2.0");
-    Assert.assertEquals(dict.insertionIndexOf("lynda\0"), -2);
-    Assert.assertEquals(dict.insertionIndexOf("lynda\0\0"), -2);
-  }
-
   /**
    * Tests loading default string column with empty ("") default null value.
    */
@@ -337,7 +261,8 @@ public class LoaderTest {
         SegmentTestUtils.getSegmentGeneratorConfigWithoutTimeColumn(_avroFile, INDEX_DIR, "testTable");
     SegmentIndexCreationDriver driver = SegmentCreationDriverFactory.get(null);
     List<String> fstIndexCreationColumns = Lists.newArrayList(FST_INDEX_COL_NAME);
-    segmentGeneratorConfig.setFSTIndexCreationColumns(fstIndexCreationColumns);
+    FstIndexConfig fstConfig = new FstIndexConfig(segmentGeneratorConfig.getFSTIndexType());
+    segmentGeneratorConfig.setIndexOn(StandardIndexes.fst(), fstConfig, fstIndexCreationColumns);
     segmentGeneratorConfig.setSegmentVersion(segmentVersion);
     driver.init(segmentGeneratorConfig);
     driver.build();
@@ -369,7 +294,7 @@ public class LoaderTest {
         new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_pinotConfiguration).build());
     SegmentDirectory.Reader reader = segmentDir.createReader();
     Assert.assertNotNull(reader);
-    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, ColumnIndexType.FST_INDEX));
+    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, StandardIndexes.fst()));
     indexSegment.destroy();
 
     // CASE 2: set the segment version to load in IndexLoadingConfig as V3
@@ -386,7 +311,7 @@ public class LoaderTest {
         new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_pinotConfiguration).build());
     reader = segmentDir.createReader();
     Assert.assertNotNull(reader);
-    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, ColumnIndexType.FST_INDEX));
+    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, StandardIndexes.fst()));
     indexSegment.destroy();
 
     // Test for scenarios by creating on-disk segment in V1 and then loading
@@ -461,7 +386,7 @@ public class LoaderTest {
         new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_pinotConfiguration).build());
     reader = segmentDir.createReader();
     Assert.assertNotNull(reader);
-    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, ColumnIndexType.FST_INDEX));
+    Assert.assertTrue(reader.hasIndexFor(FST_INDEX_COL_NAME, StandardIndexes.fst()));
     indexSegment.destroy();
   }
 
@@ -471,11 +396,10 @@ public class LoaderTest {
     SegmentGeneratorConfig segmentGeneratorConfig =
         SegmentTestUtils.getSegmentGeneratorConfigWithoutTimeColumn(_avroFile, INDEX_DIR, "testTable");
     SegmentIndexCreationDriver driver = SegmentCreationDriverFactory.get(null);
-    List<String> forwardIndexDisabledColumns = new ArrayList<>();
-    forwardIndexDisabledColumns.add(NO_FORWARD_INDEX_COL_NAME);
-    segmentGeneratorConfig.setForwardIndexDisabledColumns(forwardIndexDisabledColumns);
+    segmentGeneratorConfig.setIndexOn(StandardIndexes.forward(), ForwardIndexConfig.DISABLED,
+        NO_FORWARD_INDEX_COL_NAME);
     if (enableInvertedIndex) {
-      segmentGeneratorConfig.createInvertedIndexForColumn(NO_FORWARD_INDEX_COL_NAME);
+      segmentGeneratorConfig.setIndexOn(StandardIndexes.inverted(), IndexConfig.ENABLED, NO_FORWARD_INDEX_COL_NAME);
     }
     segmentGeneratorConfig.setSegmentVersion(segmentVersion);
     driver.init(segmentGeneratorConfig);
@@ -602,20 +526,18 @@ public class LoaderTest {
     indexSegment.destroy();
 
     // Test scenarios to create a column with no forward index but without enabling inverted index for it
+    // This should still work as the original constraint to enforce that a column has a dictionary + inverted index
+    // has been relaxed.
     try {
       constructSegmentWithForwardIndexDisabled(SegmentVersion.v3, false);
-      Assert.fail("Disabling forward index without enabling inverted index is not allowed!");
     } catch (IllegalStateException e) {
-      assertEquals(String.format("Cannot disable forward index for column %s without inverted index enabled",
-          NO_FORWARD_INDEX_COL_NAME), e.getMessage());
+      Assert.fail("Disabling forward index without enabling inverted index is allowed now");
     }
 
     try {
       constructSegmentWithForwardIndexDisabled(SegmentVersion.v1, false);
-      Assert.fail("Disabling forward index without enabling inverted index is not allowed!");
     } catch (IllegalStateException e) {
-      assertEquals(String.format("Cannot disable forward index for column %s without inverted index enabled",
-          NO_FORWARD_INDEX_COL_NAME), e.getMessage());
+      Assert.fail("Disabling forward index without enabling inverted index is allowed now");
     }
   }
 
@@ -625,9 +547,9 @@ public class LoaderTest {
     SegmentGeneratorConfig segmentGeneratorConfig =
         SegmentTestUtils.getSegmentGeneratorConfigWithoutTimeColumn(_avroFile, INDEX_DIR, "testTable");
     SegmentIndexCreationDriver driver = SegmentCreationDriverFactory.get(null);
-    List<String> textIndexCreationColumns = Lists.newArrayList(TEXT_INDEX_COL_NAME);
     List<String> rawIndexCreationColumns = Lists.newArrayList(TEXT_INDEX_COL_NAME);
-    segmentGeneratorConfig.setTextIndexCreationColumns(textIndexCreationColumns);
+    TextIndexConfig textIndexConfig = new TextIndexConfigBuilder(segmentGeneratorConfig.getFSTIndexType()).build();
+    segmentGeneratorConfig.setIndexOn(StandardIndexes.text(), textIndexConfig, TEXT_INDEX_COL_NAME);
     segmentGeneratorConfig.setRawIndexCreationColumns(rawIndexCreationColumns);
     segmentGeneratorConfig.setSegmentVersion(segmentVersion);
     driver.init(segmentGeneratorConfig);
