@@ -48,6 +48,16 @@ public class PartialUpsertMergerTest {
     assertEquals(3, ignoreMerger.merge(3, null));
   }
 
+  @Test
+  public void testMaxMinMergers() {
+    MaxMerger maxMerger = new MaxMerger();
+    MinMerger minMerger = new MinMerger();
+    assertEquals(1, maxMerger.merge(0, 1));
+    assertEquals(0, minMerger.merge(0, 1));
+    assertEquals(1, maxMerger.merge(1, 0));
+    assertEquals(0, minMerger.merge(1, 0));
+  }
+
   @Test
   public void testOverwriteMergers() {
     OverwriteMerger overwriteMerger = new OverwriteMerger();
