@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.segment.spi.index.metadata;
 
+import com.google.common.base.Preconditions;
 import java.math.BigDecimal;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -34,10 +35,10 @@ import org.apache.pinot.segment.spi.ColumnMetadata;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.V1Constants.MetadataKeys.Column;
 import org.apache.pinot.segment.spi.V1Constants.MetadataKeys.Segment;
+import org.apache.pinot.segment.spi.index.IndexType;
 import org.apache.pinot.segment.spi.partition.PartitionFunction;
 import org.apache.pinot.segment.spi.partition.PartitionFunctionFactory;
 import org.apache.pinot.segment.spi.partition.metadata.ColumnPartitionMetadata;
-import org.apache.pinot.segment.spi.store.ColumnIndexType;
 import org.apache.pinot.spi.data.DateTimeFieldSpec;
 import org.apache.pinot.spi.data.DimensionFieldSpec;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -46,6 +47,7 @@ import org.apache.pinot.spi.data.FieldSpec.FieldType;
 import org.apache.pinot.spi.data.MetricFieldSpec;
 import org.apache.pinot.spi.data.TimeFieldSpec;
 import org.apache.pinot.spi.data.TimeGranularitySpec;
+import org.apache.pinot.spi.env.CommonsConfigurationUtils;
 import org.apache.pinot.spi.utils.BytesUtils;
 
 
@@ -60,20 +62,19 @@ public class ColumnMetadataImpl implements ColumnMetadata {
   private final boolean _minMaxValueInvalid;
   private final boolean _hasDictionary;
   private final int _columnMaxLength;
-  private final char _paddingCharacter;
   private final int _bitsPerElement;
   private final int _maxNumberOfMultiValues;
   private final int _totalNumberOfEntries;
   private final PartitionFunction _partitionFunction;
   private final Set<Integer> _partitions;
-  private final Map<ColumnIndexType, Long> _indexSizeMap;
+  private final Map<IndexType<?, ?, ?>, Long> _indexSizeMap;
   private final boolean _autoGenerated;
 
   private ColumnMetadataImpl(FieldSpec fieldSpec, int totalDocs, int cardinality, boolean sorted,
       Comparable<?> minValue, Comparable<?> maxValue, boolean minMaxValueInvalid, boolean hasDictionary,
-      int columnMaxLength, char paddingCharacter, int bitsPerElement, int maxNumberOfMultiValues,
-      int totalNumberOfEntries, @Nullable PartitionFunction partitionFunction, @Nullable Set<Integer> partitions,
-      Map<ColumnIndexType, Long> indexSizeMap, boolean autoGenerated) {
+      int columnMaxLength, int bitsPerElement, int maxNumberOfMultiValues, int totalNumberOfEntries,
+      @Nullable PartitionFunction partitionFunction, @Nullable Set<Integer> partitions,
+      Map<IndexType<?, ?, ?>, Long> indexSizeMap, boolean autoGenerated) {
     _fieldSpec = fieldSpec;
     _totalDocs = totalDocs;
     _cardinality = cardinality;
@@ -83,7 +84,6 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     _minMaxValueInvalid = minMaxValueInvalid;
     _hasDictionary = hasDictionary;
     _columnMaxLength = columnMaxLength;
-    _paddingCharacter = paddingCharacter;
     _bitsPerElement = bitsPerElement;
     _maxNumberOfMultiValues = maxNumberOfMultiValues;
     _totalNumberOfEntries = totalNumberOfEntries;
@@ -137,11 +137,6 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     return _columnMaxLength;
   }
 
-  @Override
-  public char getPaddingCharacter() {
-    return _paddingCharacter;
-  }
-
   @Override
   public int getBitsPerElement() {
     return _bitsPerElement;
@@ -171,7 +166,7 @@ public class ColumnMetadataImpl implements ColumnMetadata {
 
   @Nullable
   @Override
-  public Map<ColumnIndexType, Long> getIndexSizeMap() {
+  public Map<IndexType<?, ?, ?>, Long> getIndexSizeMap() {
     return _indexSizeMap;
   }
 
@@ -191,30 +186,28 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     ColumnMetadataImpl that = (ColumnMetadataImpl) o;
     return _totalDocs == that._totalDocs && _cardinality == that._cardinality && _sorted == that._sorted
         && _hasDictionary == that._hasDictionary && _columnMaxLength == that._columnMaxLength
-        && _paddingCharacter == that._paddingCharacter && _bitsPerElement == that._bitsPerElement
-        && _maxNumberOfMultiValues == that._maxNumberOfMultiValues
-        && _totalNumberOfEntries == that._totalNumberOfEntries && _autoGenerated == that._autoGenerated && Objects
-        .equals(_fieldSpec, that._fieldSpec) && Objects.equals(_minValue, that._minValue) && Objects
-        .equals(_maxValue, that._maxValue) && Objects.equals(_partitionFunction, that._partitionFunction) && Objects
-        .equals(_partitions, that._partitions);
+        && _bitsPerElement == that._bitsPerElement && _maxNumberOfMultiValues == that._maxNumberOfMultiValues
+        && _totalNumberOfEntries == that._totalNumberOfEntries && _autoGenerated == that._autoGenerated
+        && Objects.equals(_fieldSpec, that._fieldSpec) && Objects.equals(_minValue, that._minValue) && Objects.equals(
+        _maxValue, that._maxValue) && Objects.equals(_partitionFunction, that._partitionFunction) && Objects.equals(
+        _partitions, that._partitions);
   }
 
   @Override
   public int hashCode() {
-    return Objects
-        .hash(_fieldSpec, _totalDocs, _cardinality, _sorted, _minValue, _maxValue, _hasDictionary, _columnMaxLength,
-            _paddingCharacter, _bitsPerElement, _maxNumberOfMultiValues, _totalNumberOfEntries, _partitionFunction,
-            _partitions, _autoGenerated);
+    return Objects.hash(_fieldSpec, _totalDocs, _cardinality, _sorted, _minValue, _maxValue, _hasDictionary,
+        _columnMaxLength, _bitsPerElement, _maxNumberOfMultiValues, _totalNumberOfEntries, _partitionFunction,
+        _partitions, _autoGenerated);
   }
 
   @Override
   public String toString() {
     return "ColumnMetadataImpl{" + "_fieldSpec=" + _fieldSpec + ", _totalDocs=" + _totalDocs + ", _cardinality="
         + _cardinality + ", _sorted=" + _sorted + ", _minValue=" + _minValue + ", _maxValue=" + _maxValue
-        + ", _hasDictionary=" + _hasDictionary + ", _columnMaxLength=" + _columnMaxLength + ", _paddingCharacter="
-        + _paddingCharacter + ", _bitsPerElement=" + _bitsPerElement + ", _maxNumberOfMultiValues="
-        + _maxNumberOfMultiValues + ", _totalNumberOfEntries=" + _totalNumberOfEntries + ", _partitionFunction="
-        + _partitionFunction + ", _partitions=" + _partitions + ", _autoGenerated=" + _autoGenerated + '}';
+        + ", _hasDictionary=" + _hasDictionary + ", _columnMaxLength=" + _columnMaxLength + ", _bitsPerElement="
+        + _bitsPerElement + ", _maxNumberOfMultiValues=" + _maxNumberOfMultiValues + ", _totalNumberOfEntries="
+        + _totalNumberOfEntries + ", _partitionFunction=" + _partitionFunction + ", _partitions=" + _partitions
+        + ", _autoGenerated=" + _autoGenerated + '}';
   }
 
   public static ColumnMetadataImpl fromPropertiesConfiguration(String column, PropertiesConfiguration config) {
@@ -231,7 +224,11 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     FieldType fieldType =
         FieldType.valueOf(config.getString(Column.getKeyFor(column, Column.COLUMN_TYPE)).toUpperCase());
     DataType dataType = DataType.valueOf(config.getString(Column.getKeyFor(column, Column.DATA_TYPE)).toUpperCase());
+    DataType storedType = dataType.getStoredType();
     String defaultNullValueString = config.getString(Column.getKeyFor(column, Column.DEFAULT_NULL_VALUE), null);
+    if (defaultNullValueString != null && storedType == DataType.STRING) {
+      defaultNullValueString = CommonsConfigurationUtils.recoverSpecialCharacterInPropertyValue(defaultNullValueString);
+    }
     FieldSpec fieldSpec;
     switch (fieldType) {
       case DIMENSION:
@@ -262,7 +259,7 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     String minString = (String) config.getProperty(Column.getKeyFor(column, Column.MIN_VALUE));
     String maxString = (String) config.getProperty(Column.getKeyFor(column, Column.MAX_VALUE));
     if (minString != null && maxString != null) {
-      switch (dataType.getStoredType()) {
+      switch (storedType) {
         case INT:
           builder.setMinValue(Integer.valueOf(minString));
           builder.setMaxValue(Integer.valueOf(maxString));
@@ -284,8 +281,8 @@ public class ColumnMetadataImpl implements ColumnMetadata {
           builder.setMaxValue(new BigDecimal(maxString));
           break;
         case STRING:
-          builder.setMinValue(minString);
-          builder.setMaxValue(maxString);
+          builder.setMinValue(CommonsConfigurationUtils.recoverSpecialCharacterInPropertyValue(minString));
+          builder.setMaxValue(CommonsConfigurationUtils.recoverSpecialCharacterInPropertyValue(maxString));
           break;
         case BYTES:
           builder.setMinValue(BytesUtils.toByteArray(minString));
@@ -297,12 +294,10 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     }
     builder.setMinMaxValueInvalid(config.getBoolean(Column.getKeyFor(column, Column.MIN_MAX_VALUE_INVALID), false));
 
-    char paddingCharacter = V1Constants.Str.LEGACY_STRING_PAD_CHAR;
+    // Only support zero padding
     String padding = config.getString(Segment.SEGMENT_PADDING_CHARACTER, null);
-    if (padding != null) {
-      paddingCharacter = StringEscapeUtils.unescapeJava(padding).charAt(0);
-    }
-    builder.setPaddingCharacter(paddingCharacter);
+    Preconditions.checkState(String.valueOf(V1Constants.Str.DEFAULT_STRING_PAD_CHAR)
+        .equals(StringEscapeUtils.unescapeJava(padding)), "Got non-zero string padding: %s", padding);
 
     String partitionFunctionName = config.getString(Column.getKeyFor(column, Column.PARTITION_FUNCTION), null);
     if (partitionFunctionName != null) {
@@ -350,14 +345,13 @@ public class ColumnMetadataImpl implements ColumnMetadata {
     private boolean _minMaxValueInvalid;
     private boolean _hasDictionary;
     private int _columnMaxLength;
-    private char _paddingCharacter;
     private int _bitsPerElement;
     private int _maxNumberOfMultiValues;
     private int _totalNumberOfEntries;
     private PartitionFunction _partitionFunction;
     private Set<Integer> _partitions;
     private boolean _autoGenerated;
-    private Map<ColumnIndexType, Long> _indexSizeMap = new HashMap<>();
+    private Map<IndexType<?, ?, ?>, Long> _indexSizeMap = new HashMap<>();
 
     public Builder setFieldSpec(FieldSpec fieldSpec) {
       _fieldSpec = fieldSpec;
@@ -404,11 +398,6 @@ public class ColumnMetadataImpl implements ColumnMetadata {
       return this;
     }
 
-    public Builder setPaddingCharacter(char paddingCharacter) {
-      _paddingCharacter = paddingCharacter;
-      return this;
-    }
-
     public Builder setBitsPerElement(int bitsPerElement) {
       _bitsPerElement = bitsPerElement;
       return this;
@@ -434,7 +423,7 @@ public class ColumnMetadataImpl implements ColumnMetadata {
       return this;
     }
 
-    public void setIndexSizeMap(Map<ColumnIndexType, Long> indexSizeMap) {
+    public void setIndexSizeMap(Map<IndexType<?, ?, ?>, Long> indexSizeMap) {
       _indexSizeMap = indexSizeMap;
     }
 
@@ -445,9 +434,8 @@ public class ColumnMetadataImpl implements ColumnMetadata {
 
     public ColumnMetadataImpl build() {
       return new ColumnMetadataImpl(_fieldSpec, _totalDocs, _cardinality, _sorted, _minValue, _maxValue,
-          _minMaxValueInvalid, _hasDictionary, _columnMaxLength, _paddingCharacter, _bitsPerElement,
-          _maxNumberOfMultiValues, _totalNumberOfEntries, _partitionFunction, _partitions, _indexSizeMap,
-          _autoGenerated);
+          _minMaxValueInvalid, _hasDictionary, _columnMaxLength, _bitsPerElement, _maxNumberOfMultiValues,
+          _totalNumberOfEntries, _partitionFunction, _partitions, _indexSizeMap, _autoGenerated);
     }
   }
 }
