@@ -39,22 +39,28 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TimeZone;
+import java.util.TreeMap;
 import java.util.concurrent.TimeUnit;
 import javax.annotation.Nullable;
 import org.apache.commons.configuration.Configuration;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.pinot.segment.spi.ColumnMetadata;
 import org.apache.pinot.segment.spi.SegmentMetadata;
+import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.V1Constants.MetadataKeys.Segment;
 import org.apache.pinot.segment.spi.creator.SegmentVersion;
+import org.apache.pinot.segment.spi.index.IndexService;
+import org.apache.pinot.segment.spi.index.IndexType;
 import org.apache.pinot.segment.spi.index.startree.StarTreeV2Constants;
 import org.apache.pinot.segment.spi.index.startree.StarTreeV2Metadata;
+import org.apache.pinot.segment.spi.store.ColumnIndexUtils;
 import org.apache.pinot.segment.spi.store.SegmentDirectoryPaths;
-import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
+import org.apache.pinot.segment.spi.utils.SegmentMetadataUtils;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.env.CommonsConfigurationUtils;
 import org.apache.pinot.spi.utils.JsonUtils;
 import org.apache.pinot.spi.utils.TimeUtils;
+import org.apache.pinot.spi.utils.TimestampIndexUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.joda.time.DateTimeZone;
 import org.joda.time.Duration;
@@ -67,7 +73,7 @@ public class SegmentMetadataImpl implements SegmentMetadata {
   private static final Logger LOGGER = LoggerFactory.getLogger(SegmentMetadataImpl.class);
 
   private final File _indexDir;
-  private final Map<String, ColumnMetadata> _columnMetadataMap;
+  private final TreeMap<String, ColumnMetadata> _columnMetadataMap;
   private String _segmentName;
   private final Schema _schema;
   private long _crc = Long.MIN_VALUE;
@@ -89,11 +95,6 @@ public class SegmentMetadataImpl implements SegmentMetadata {
   private String _startOffset;
   private String _endOffset;
 
-  // TODO: No need to cache this. We cannot modify the metadata if it is from a input stream
-  // Caching properties around can be costly when the number of segments is high according to the
-  // finding in PR #2996. So for now, caching is used only when initializing from input streams.
-  private PropertiesConfiguration _segmentMetadataPropertiesConfiguration = null;
-
   @Deprecated
   private String _rawTableName;
 
@@ -103,16 +104,16 @@ public class SegmentMetadataImpl implements SegmentMetadata {
   public SegmentMetadataImpl(InputStream metadataPropertiesInputStream, InputStream creationMetaInputStream)
       throws IOException {
     _indexDir = null;
-    _columnMetadataMap = new HashMap<>();
+    _columnMetadataMap = new TreeMap<>();
     _schema = new Schema();
 
-    // Caching properties when initializing from input streams.
-    _segmentMetadataPropertiesConfiguration = CommonsConfigurationUtils.fromInputStream(metadataPropertiesInputStream);
-    init(_segmentMetadataPropertiesConfiguration);
-    loadCreationMeta(creationMetaInputStream);
+    PropertiesConfiguration segmentMetadataPropertiesConfiguration =
+        CommonsConfigurationUtils.fromInputStream(metadataPropertiesInputStream);
+    init(segmentMetadataPropertiesConfiguration);
+    setTimeInfo(segmentMetadataPropertiesConfiguration);
+    _totalDocs = segmentMetadataPropertiesConfiguration.getInt(Segment.SEGMENT_TOTAL_DOCS);
 
-    setTimeInfo(_segmentMetadataPropertiesConfiguration);
-    _totalDocs = _segmentMetadataPropertiesConfiguration.getInt(Segment.SEGMENT_TOTAL_DOCS);
+    loadCreationMeta(creationMetaInputStream);
   }
 
   /**
@@ -123,18 +124,19 @@ public class SegmentMetadataImpl implements SegmentMetadata {
   public SegmentMetadataImpl(File indexDir)
       throws IOException {
     _indexDir = indexDir;
-    _columnMetadataMap = new HashMap<>();
-    PropertiesConfiguration segmentMetadataPropertiesConfiguration = getPropertiesConfiguration(indexDir);
+    _columnMetadataMap = new TreeMap<>();
     _schema = new Schema();
 
+    PropertiesConfiguration segmentMetadataPropertiesConfiguration =
+        SegmentMetadataUtils.getPropertiesConfiguration(indexDir);
     init(segmentMetadataPropertiesConfiguration);
+    setTimeInfo(segmentMetadataPropertiesConfiguration);
+    _totalDocs = segmentMetadataPropertiesConfiguration.getInt(Segment.SEGMENT_TOTAL_DOCS);
+
     File creationMetaFile = SegmentDirectoryPaths.findCreationMetaFile(indexDir);
     if (creationMetaFile != null) {
       loadCreationMeta(creationMetaFile);
     }
-
-    setTimeInfo(segmentMetadataPropertiesConfiguration);
-    _totalDocs = segmentMetadataPropertiesConfiguration.getInt(Segment.SEGMENT_TOTAL_DOCS);
   }
 
   /**
@@ -149,18 +151,6 @@ public class SegmentMetadataImpl implements SegmentMetadata {
     _creationTime = creationTime;
   }
 
-  public PropertiesConfiguration getPropertiesConfiguration() {
-    return (_segmentMetadataPropertiesConfiguration != null) ? _segmentMetadataPropertiesConfiguration
-        : SegmentMetadataImpl.getPropertiesConfiguration(_indexDir);
-  }
-
-  public static PropertiesConfiguration getPropertiesConfiguration(File indexDir) {
-    File metadataFile = SegmentDirectoryPaths.findMetadataFile(indexDir);
-    Preconditions.checkNotNull(metadataFile, "Cannot find segment metadata file under directory: %s", indexDir);
-
-    return CommonsConfigurationUtils.fromFile(metadataFile);
-  }
-
   /**
    * Helper method to set time related information:
    * <ul>
@@ -245,6 +235,26 @@ public class SegmentMetadataImpl implements SegmentMetadata {
       _schema.addField(columnMetadata.getFieldSpec());
     }
 
+    // Load index metadata
+    // Support V3 (e.g. SingleFileIndexDirectory only)
+    if (_segmentVersion == SegmentVersion.v3) {
+      File indexMapFile = new File(_indexDir, "v3" + File.separator + V1Constants.INDEX_MAP_FILE_NAME);
+      if (indexMapFile.exists()) {
+        PropertiesConfiguration mapConfig = CommonsConfigurationUtils.fromFile(indexMapFile);
+        for (String key : CommonsConfigurationUtils.getKeys(mapConfig)) {
+          try {
+            String[] parsedKeys = ColumnIndexUtils.parseIndexMapKeys(key, _indexDir.getPath());
+            if (parsedKeys[2].equals(ColumnIndexUtils.MAP_KEY_NAME_SIZE)) {
+              IndexType<?, ?, ?> indexType = IndexService.getInstance().get(parsedKeys[1]);
+              _columnMetadataMap.get(parsedKeys[0]).getIndexSizeMap().put(indexType, mapConfig.getLong(key));
+            }
+          } catch (Exception e) {
+            LOGGER.debug("Unable to load index metadata in {} for {}!", indexMapFile, key, e);
+          }
+        }
+      }
+    }
+
     // Build star-tree v2 metadata
     int starTreeV2Count =
         segmentMetadataPropertiesConfiguration.getInt(StarTreeV2Constants.MetadataKey.STAR_TREE_COUNT, 0);
@@ -281,11 +291,9 @@ public class SegmentMetadataImpl implements SegmentMetadata {
     for (Object o : src) {
       String column = o.toString();
       if (!column.isEmpty() && !dest.contains(column)) {
-        if (column.charAt(0) == '$') {
-          // Skip virtual columns starting with '$', but keep time column with granularity as physical column.
-          if (!TimestampIndexGranularity.isValidTimeColumnWithGranularityName(column)) {
-            continue;
-          }
+        // Skip virtual columns starting with '$', but keep time column with granularity as physical column
+        if (column.charAt(0) == '$' && !TimestampIndexUtils.isValidColumnWithGranularity(column)) {
+          continue;
         }
         dest.add(column);
       }
@@ -399,7 +407,7 @@ public class SegmentMetadataImpl implements SegmentMetadata {
   }
 
   @Override
-  public Map<String, ColumnMetadata> getColumnMetadataMap() {
+  public TreeMap<String, ColumnMetadata> getColumnMetadataMap() {
     return _columnMetadataMap;
   }
 
