@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.spi.config.table;
 
+import com.fasterxml.jackson.databind.JsonNode;
 import java.util.List;
 import java.util.Map;
 import javax.annotation.Nullable;
@@ -26,8 +27,13 @@ import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
 
 
 public class IndexingConfig extends BaseJsonConfig {
+  // Default ratio for overriding dictionary
+  public static final double DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD = 0.85d;
 
-  public static final int DEFAULT_RANGE_INDEX_VERSION = 2;
+  /**
+   * This should be equal to the one specified in RangeIndexType.
+   */
+  private static final int DEFAULT_RANGE_INDEX_VERSION = 2;
 
   private List<String> _invertedIndexColumns;
   private List<String> _rangeIndexColumns;
@@ -68,7 +74,7 @@ public class IndexingConfig extends BaseJsonConfig {
    */
   private boolean _optimizeDictionaryForMetrics;
 
-  private double _noDictionarySizeRatioThreshold;
+  private double _noDictionarySizeRatioThreshold = DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD;
 
   // TODO: Add a new configuration related to the segment generation
   private boolean _autoGeneratedInvertedIndex;
@@ -82,6 +88,8 @@ public class IndexingConfig extends BaseJsonConfig {
    */
   private List<String> _varLengthDictionaryColumns;
 
+  private JsonNode _tierOverwrites;
+
   @Nullable
   public List<String> getInvertedIndexColumns() {
     return _invertedIndexColumns;
@@ -267,6 +275,14 @@ public class IndexingConfig extends BaseJsonConfig {
     _starTreeIndexConfigs = starTreeIndexConfigs;
   }
 
+  public JsonNode getTierOverwrites() {
+    return _tierOverwrites;
+  }
+
+  public void setTierOverwrites(JsonNode tierOverwrites) {
+    _tierOverwrites = tierOverwrites;
+  }
+
   public boolean isEnableDynamicStarTreeCreation() {
     return _enableDynamicStarTreeCreation;
   }
