@@ -29,11 +29,18 @@ import org.apache.pinot.spi.utils.StringUtil;
  * Class to represent granularity from {@link DateTimeFieldSpec}
  */
 public class DateTimeGranularitySpec {
-  // 'size:timeUnit'
-  private static final char SEPARATOR = ':';
-  private static final int SIZE_POSITION = 0;
-  private static final int TIME_UNIT_POSITION = 1;
-  private static final int NUM_TOKENS = 2;
+  // Colon format: 'size:timeUnit'
+  private static final char COLON_SEPARATOR = ':';
+  private static final int COLON_FORMAT_SIZE_POSITION = 0;
+  private static final int COLON_FORMAT_TIME_UNIT_POSITION = 1;
+  private static final int COLON_FORMAT_NUM_TOKENS = 2;
+
+  // Pipe format: 'timeUnit|size'
+  private static final char PIPE_SEPARATOR = '|';
+  private static final int PIPE_FORMAT_TIME_UNIT_POSITION = 0;
+  private static final int PIPE_FORMAT_SIZE_POSITION = 1;
+  private static final int PIPE_FORMAT_MIN_TOKENS = 1;
+  private static final int PIPE_FORMAT_MAX_TOKENS = 2;
 
   private final int _size;
   private final TimeUnit _timeUnit;
@@ -43,23 +50,56 @@ public class DateTimeGranularitySpec {
    */
   public DateTimeGranularitySpec(String granularity) {
     Preconditions.checkArgument(StringUtils.isNotEmpty(granularity), "Must provide granularity");
-    String[] granularityTokens = StringUtil.split(granularity, SEPARATOR, 2);
-    Preconditions.checkArgument(granularityTokens.length >= NUM_TOKENS,
-        "Invalid granularity: %s, must be of format 'size:timeUnit", granularity);
-    try {
-      _size = Integer.parseInt(granularityTokens[SIZE_POSITION]);
-    } catch (Exception e) {
-      throw new IllegalArgumentException(
-          String.format("Invalid size: %s in granularity: %s", granularityTokens[SIZE_POSITION], granularity));
+
+    char separator;
+    int sizePosition;
+    int timeUnitPosition;
+    int minTokens;
+    int maxTokens;
+
+    if (Character.isDigit(granularity.charAt(0))) {
+      // Colon format: 'size:timeUnit'
+      separator = COLON_SEPARATOR;
+      sizePosition = COLON_FORMAT_SIZE_POSITION;
+      timeUnitPosition = COLON_FORMAT_TIME_UNIT_POSITION;
+
+      minTokens = COLON_FORMAT_NUM_TOKENS;
+      maxTokens = COLON_FORMAT_NUM_TOKENS;
+    } else {
+      // Pipe format: 'timeUnit(|size)'
+      separator = PIPE_SEPARATOR;
+      sizePosition = PIPE_FORMAT_SIZE_POSITION;
+      timeUnitPosition = PIPE_FORMAT_TIME_UNIT_POSITION;
+
+      minTokens = PIPE_FORMAT_MIN_TOKENS;
+      maxTokens = PIPE_FORMAT_MAX_TOKENS;
     }
-    Preconditions.checkArgument(_size > 0, "Invalid size: %s in granularity: %s, must be positive", _size, granularity);
+
+    String[] granularityTokens = StringUtil.split(granularity, separator, 2);
+    Preconditions.checkArgument(granularityTokens.length >= minTokens && granularityTokens.length <= maxTokens,
+            "Invalid granularity: %s, must be of format 'timeUnit(|size)' or 'size:timeUnit'", granularity);
+
     try {
-      _timeUnit = TimeUnit.valueOf(granularityTokens[TIME_UNIT_POSITION]);
+      _timeUnit = TimeUnit.valueOf(granularityTokens[timeUnitPosition]);
     } catch (Exception e) {
       throw new IllegalArgumentException(
-          String.format("Invalid time unit: %s in granularity: %s", granularityTokens[TIME_UNIT_POSITION],
+          String.format("Invalid time unit: %s in granularity: %s", granularityTokens[timeUnitPosition],
               granularity));
     }
+
+    // New format without explicitly setting size - use default size = 1
+    if (granularityTokens.length == PIPE_FORMAT_MIN_TOKENS) {
+      _size = 1;
+      return;
+    }
+
+    try {
+      _size = Integer.parseInt(granularityTokens[sizePosition]);
+    } catch (Exception e) {
+      throw new IllegalArgumentException(
+          String.format("Invalid size: %s in granularity: %s", granularityTokens[sizePosition], granularity));
+    }
+    Preconditions.checkArgument(_size > 0, "Invalid size: %s in granularity: %s, must be positive", _size, granularity);
   }
 
   /**
@@ -82,6 +122,15 @@ public class DateTimeGranularitySpec {
 
   /**
    * Converts a granularity to millis.
+   *
+   * Using new format:
+   * <ul>
+   *   <li>1) granularityToMillis(HOURS|1) = 3600000 (60*60*1000)</li>
+   *   <li>2) granularityToMillis(MILLISECONDS|1) = 1</li>
+   *   <li>3) granularityToMillis(MINUTES|15) = 900000 (15*60*1000)</li>
+   * </ul>
+   *
+   * Using old format:
    * <ul>
    *   <li>1) granularityToMillis(1:HOURS) = 3600000 (60*60*1000)</li>
    *   <li>2) granularityToMillis(1:MILLISECONDS) = 1</li>
