@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.spi.stream;
 
+import javax.annotation.Nullable;
 import org.apache.pinot.spi.annotations.InterfaceAudience;
 import org.apache.pinot.spi.annotations.InterfaceStability;
 
@@ -47,8 +48,23 @@ public interface MessageBatch<T> {
    * @param index
    * @return
    */
+  @Deprecated
   T getMessageAtIndex(int index);
 
+  // for backward-compatibility
+  default byte[] getMessageBytesAtIndex(int index) {
+    return (byte[]) getMessageAtIndex(index);
+  }
+
+  default StreamMessage<T> getStreamMessage(int index) {
+    return new LegacyStreamMessage(getMessageBytesAtIndex(index), (StreamMessageMetadata) getMetadataAtIndex(index));
+  }
+
+  class LegacyStreamMessage extends StreamMessage {
+    public LegacyStreamMessage(byte[] value, StreamMessageMetadata metadata) {
+      super(value, value.length, metadata);
+    }
+  }
   /**
    * Returns the offset of the message at a particular index inside a set of messages returned from the stream.
    * @param index
@@ -101,4 +117,17 @@ public interface MessageBatch<T> {
   default boolean isEndOfPartitionGroup() {
     return false;
   }
+
+  /**
+   * This is useful while determining ingestion delay for a message batch. Retaining metadata for last filtered message
+   * in a batch can enable us to estimate the ingestion delay for the batch.
+   * Note that a batch can be fully filtered, and we can still retain the metadata for the last filtered message to
+   * facilitate computing ingestion delay in the face of a fully filtered batch.
+   *
+   * @return null by default.
+   */
+  @Nullable
+  default public StreamMessageMetadata getLastMessageMetadata() {
+    return null;
+  }
 }
