@@ -23,6 +23,7 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.concurrent.TimeUnit;
 import org.apache.pinot.spi.utils.DataSizeUtils;
 import org.apache.pinot.spi.utils.EqualityUtils;
@@ -49,27 +50,32 @@ public class StreamConfig {
   public static final long DEFAULT_FLUSH_THRESHOLD_TIME_MILLIS = TimeUnit.MILLISECONDS.convert(6, TimeUnit.HOURS);
   public static final long DEFAULT_FLUSH_THRESHOLD_SEGMENT_SIZE_BYTES = 200 * 1024 * 1024; // 200M
   public static final int DEFAULT_FLUSH_AUTOTUNE_INITIAL_ROWS = 100_000;
+  public static final String DEFAULT_SERVER_UPLOAD_TO_DEEPSTORE = "false";
 
   public static final String DEFAULT_CONSUMER_FACTORY_CLASS_NAME_STRING =
       "org.apache.pinot.plugin.stream.kafka20.KafkaConsumerFactory";
 
   public static final long DEFAULT_STREAM_CONNECTION_TIMEOUT_MILLIS = 30_000;
   public static final int DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS = 5_000;
+  public static final int DEFAULT_IDLE_TIMEOUT_MILLIS = 3 * 60 * 1000;
 
   private static final String SIMPLE_CONSUMER_TYPE_STRING = "simple";
 
+  private static final double CONSUMPTION_RATE_LIMIT_NOT_SPECIFIED = -1;
+
   private final String _type;
   private final String _topicName;
   private final String _tableNameWithType;
   private final List<ConsumerType> _consumerTypes = new ArrayList<>();
   private final String _consumerFactoryClassName;
-  private final OffsetCriteria _offsetCriteria;
   private final String _decoderClass;
   private final Map<String, String> _decoderProperties = new HashMap<>();
 
   private final long _connectionTimeoutMillis;
   private final int _fetchTimeoutMillis;
 
+  private final long _idleTimeoutMillis;
+
   private final int _flushThresholdRows;
   private final long _flushThresholdTimeMillis;
   private final long _flushThresholdSegmentSizeBytes;
@@ -77,14 +83,24 @@ public class StreamConfig {
 
   private final String _groupId;
 
+  private final double _topicConsumptionRateLimit;
+
   private final Map<String, String> _streamConfigMap = new HashMap<>();
 
+  // Allow overriding it to use different offset criteria
+  private OffsetCriteria _offsetCriteria;
+
+  // Indicates if the segment should be uploaded to the deep store's file system or to the controller during the
+  // segment commit protocol. By default, segment is uploaded to the controller during commit.
+  // If this flag is set to true, the segment is uploaded to deep store.
+  private final boolean _serverUploadToDeepStore;
+
   /**
    * Initializes a StreamConfig using the map of stream configs from the table config
    */
   public StreamConfig(String tableNameWithType, Map<String, String> streamConfigMap) {
     _type = streamConfigMap.get(StreamConfigProperties.STREAM_TYPE);
-    Preconditions.checkNotNull(_type, "Stream type cannot be null");
+    Preconditions.checkNotNull(_type, StreamConfigProperties.STREAM_TYPE + " cannot be null");
 
     String topicNameKey =
         StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_TOPIC_NAME);
@@ -93,19 +109,8 @@ public class StreamConfig {
 
     _tableNameWithType = tableNameWithType;
 
-    String consumerTypesKey =
-        StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_CONSUMER_TYPES);
-    String consumerTypes = streamConfigMap.get(consumerTypesKey);
-    Preconditions.checkNotNull(consumerTypes, "Must specify at least one consumer type " + consumerTypesKey);
-    for (String consumerType : consumerTypes.split(",")) {
-      if (consumerType.equals(
-          SIMPLE_CONSUMER_TYPE_STRING)) { //For backward compatibility of stream configs which referred to lowlevel
-        // as simple
-        _consumerTypes.add(ConsumerType.LOWLEVEL);
-        continue;
-      }
-      _consumerTypes.add(ConsumerType.valueOf(consumerType.toUpperCase()));
-    }
+    validateConsumerType(_type, streamConfigMap);
+    _consumerTypes.add(ConsumerType.LOWLEVEL);
 
     String consumerFactoryClassKey =
         StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_CONSUMER_FACTORY_CLASS);
@@ -164,9 +169,26 @@ public class StreamConfig {
     }
     _fetchTimeoutMillis = fetchTimeoutMillis;
 
+    int idleTimeoutMillis = DEFAULT_IDLE_TIMEOUT_MILLIS;
+    String idleTimeoutMillisKey =
+        StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_IDLE_TIMEOUT_MILLIS);
+    String idleTimeoutMillisValue = streamConfigMap.get(idleTimeoutMillisKey);
+    if (idleTimeoutMillisValue != null) {
+      try {
+        idleTimeoutMillis = Integer.parseInt(idleTimeoutMillisValue);
+      } catch (Exception e) {
+        LOGGER.warn("Invalid config {}: {}, defaulting to: {}", idleTimeoutMillisKey, idleTimeoutMillisValue,
+            DEFAULT_IDLE_TIMEOUT_MILLIS);
+      }
+    }
+    _idleTimeoutMillis = idleTimeoutMillis;
+
     _flushThresholdRows = extractFlushThresholdRows(streamConfigMap);
     _flushThresholdTimeMillis = extractFlushThresholdTimeMillis(streamConfigMap);
     _flushThresholdSegmentSizeBytes = extractFlushThresholdSegmentSize(streamConfigMap);
+    _serverUploadToDeepStore = Boolean.parseBoolean(
+        streamConfigMap.getOrDefault(StreamConfigProperties.SERVER_UPLOAD_TO_DEEPSTORE,
+        DEFAULT_SERVER_UPLOAD_TO_DEEPSTORE));
 
     int autotuneInitialRows = 0;
     String initialRowsValue = streamConfigMap.get(StreamConfigProperties.SEGMENT_FLUSH_AUTOTUNE_INITIAL_ROWS);
@@ -184,9 +206,27 @@ public class StreamConfig {
     String groupIdKey = StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.GROUP_ID);
     _groupId = streamConfigMap.get(groupIdKey);
 
+    String rate = streamConfigMap.get(StreamConfigProperties.TOPIC_CONSUMPTION_RATE_LIMIT);
+    _topicConsumptionRateLimit = rate != null ? Double.parseDouble(rate) : CONSUMPTION_RATE_LIMIT_NOT_SPECIFIED;
+
     _streamConfigMap.putAll(streamConfigMap);
   }
 
+  public static void validateConsumerType(String streamType, Map<String, String> streamConfigMap) {
+    String consumerTypesKey =
+        StreamConfigProperties.constructStreamProperty(streamType, StreamConfigProperties.STREAM_CONSUMER_TYPES);
+    String consumerTypes = streamConfigMap.get(consumerTypesKey);
+    Preconditions.checkNotNull(consumerTypes, consumerTypesKey + " cannot be null");
+    for (String consumerType : consumerTypes.split(",")) {
+      Preconditions.checkState(ConsumerType.LOWLEVEL.name().equalsIgnoreCase(consumerType)
+              || SIMPLE_CONSUMER_TYPE_STRING.equalsIgnoreCase(consumerType),
+          "Realtime tables with HLC consumer (consumer.type=highlevel) is no longer supported in Apache Pinot");
+    }
+  }
+  public boolean isServerUploadToDeepStore() {
+    return _serverUploadToDeepStore;
+  }
+
   private long extractFlushThresholdSegmentSize(Map<String, String> streamConfigMap) {
     long segmentSizeBytes = -1;
     String key = StreamConfigProperties.SEGMENT_FLUSH_THRESHOLD_SEGMENT_SIZE;
@@ -284,6 +324,10 @@ public class StreamConfig {
     return _offsetCriteria;
   }
 
+  public void setOffsetCriteria(OffsetCriteria offsetCriteria) {
+    _offsetCriteria = offsetCriteria;
+  }
+
   public String getDecoderClass() {
     return _decoderClass;
   }
@@ -300,6 +344,10 @@ public class StreamConfig {
     return _fetchTimeoutMillis;
   }
 
+  public long getIdleTimeoutMillis() {
+    return _idleTimeoutMillis;
+  }
+
   public int getFlushThresholdRows() {
     return _flushThresholdRows;
   }
@@ -320,6 +368,11 @@ public class StreamConfig {
     return _groupId;
   }
 
+  public Optional<Double> getTopicConsumptionRateLimit() {
+    return _topicConsumptionRateLimit == CONSUMPTION_RATE_LIMIT_NOT_SPECIFIED ? Optional.empty()
+        : Optional.of(_topicConsumptionRateLimit);
+  }
+
   public String getTableNameWithType() {
     return _tableNameWithType;
   }
@@ -333,11 +386,13 @@ public class StreamConfig {
     return "StreamConfig{" + "_type='" + _type + '\'' + ", _topicName='" + _topicName + '\'' + ", _consumerTypes="
         + _consumerTypes + ", _consumerFactoryClassName='" + _consumerFactoryClassName + '\'' + ", _offsetCriteria='"
         + _offsetCriteria + '\'' + ", _connectionTimeoutMillis=" + _connectionTimeoutMillis + ", _fetchTimeoutMillis="
-        + _fetchTimeoutMillis + ", _flushThresholdRows=" + _flushThresholdRows + ", _flushThresholdTimeMillis="
-        + _flushThresholdTimeMillis + ", _flushSegmentDesiredSizeBytes=" + _flushThresholdSegmentSizeBytes
-        + ", _flushAutotuneInitialRows=" + _flushAutotuneInitialRows + ", _decoderClass='" + _decoderClass + '\''
-        + ", _decoderProperties=" + _decoderProperties + ", _groupId='" + _groupId + ", _tableNameWithType='"
-        + _tableNameWithType + '}';
+        + _fetchTimeoutMillis + ", _idleTimeoutMillis=" + _idleTimeoutMillis + ", _flushThresholdRows="
+        + _flushThresholdRows + ", _flushThresholdTimeMillis=" + _flushThresholdTimeMillis
+        + ", _flushSegmentDesiredSizeBytes=" + _flushThresholdSegmentSizeBytes + ", _flushAutotuneInitialRows="
+        + _flushAutotuneInitialRows + ", _decoderClass='" + _decoderClass + '\'' + ", _decoderProperties="
+        + _decoderProperties + ", _groupId='" + _groupId + "', _topicConsumptionRateLimit=" + _topicConsumptionRateLimit
+        + ", _tableNameWithType='" + _tableNameWithType + ", _serverUploadToDeepStore=" + _serverUploadToDeepStore
+        + "}";
   }
 
   @Override
@@ -352,19 +407,20 @@ public class StreamConfig {
 
     StreamConfig that = (StreamConfig) o;
 
-    return EqualityUtils.isEqual(_connectionTimeoutMillis, that._connectionTimeoutMillis) && EqualityUtils
-        .isEqual(_fetchTimeoutMillis, that._fetchTimeoutMillis) && EqualityUtils
-        .isEqual(_flushThresholdRows, that._flushThresholdRows) && EqualityUtils
-        .isEqual(_flushThresholdTimeMillis, that._flushThresholdTimeMillis) && EqualityUtils
-        .isEqual(_flushThresholdSegmentSizeBytes, that._flushThresholdSegmentSizeBytes) && EqualityUtils
-        .isEqual(_flushAutotuneInitialRows, that._flushAutotuneInitialRows) && EqualityUtils.isEqual(_type, that._type)
-        && EqualityUtils.isEqual(_topicName, that._topicName) && EqualityUtils
-        .isEqual(_consumerTypes, that._consumerTypes) && EqualityUtils
-        .isEqual(_consumerFactoryClassName, that._consumerFactoryClassName) && EqualityUtils
-        .isEqual(_offsetCriteria, that._offsetCriteria) && EqualityUtils.isEqual(_decoderClass, that._decoderClass)
-        && EqualityUtils.isEqual(_decoderProperties, that._decoderProperties) && EqualityUtils
-        .isEqual(_groupId, that._groupId) && EqualityUtils.isEqual(_tableNameWithType, that._tableNameWithType)
-        && EqualityUtils.isEqual(_streamConfigMap, that._streamConfigMap);
+    return EqualityUtils.isEqual(_connectionTimeoutMillis, that._connectionTimeoutMillis) && EqualityUtils.isEqual(
+        _fetchTimeoutMillis, that._fetchTimeoutMillis) && EqualityUtils.isEqual(_idleTimeoutMillis,
+        that._idleTimeoutMillis) && EqualityUtils.isEqual(_flushThresholdRows, that._flushThresholdRows)
+        && EqualityUtils.isEqual(_flushThresholdTimeMillis, that._flushThresholdTimeMillis) && EqualityUtils.isEqual(
+        _flushThresholdSegmentSizeBytes, that._flushThresholdSegmentSizeBytes) && EqualityUtils.isEqual(
+        _flushAutotuneInitialRows, that._flushAutotuneInitialRows) && EqualityUtils.isEqual(_type, that._type)
+        && EqualityUtils.isEqual(_topicName, that._topicName) && EqualityUtils.isEqual(_consumerTypes,
+        that._consumerTypes) && EqualityUtils.isEqual(_consumerFactoryClassName, that._consumerFactoryClassName)
+        && EqualityUtils.isEqual(_offsetCriteria, that._offsetCriteria) && EqualityUtils.isEqual(_decoderClass,
+        that._decoderClass) && EqualityUtils.isEqual(_decoderProperties, that._decoderProperties)
+        && EqualityUtils.isEqual(_groupId, that._groupId) && EqualityUtils.isEqual(_tableNameWithType,
+        that._tableNameWithType) && EqualityUtils.isEqual(_topicConsumptionRateLimit, that._topicConsumptionRateLimit)
+        && EqualityUtils.isEqual(_streamConfigMap, that._streamConfigMap)
+        && _serverUploadToDeepStore == that._serverUploadToDeepStore;
   }
 
   @Override
@@ -376,6 +432,7 @@ public class StreamConfig {
     result = EqualityUtils.hashCodeOf(result, _offsetCriteria);
     result = EqualityUtils.hashCodeOf(result, _connectionTimeoutMillis);
     result = EqualityUtils.hashCodeOf(result, _fetchTimeoutMillis);
+    result = EqualityUtils.hashCodeOf(result, _idleTimeoutMillis);
     result = EqualityUtils.hashCodeOf(result, _flushThresholdRows);
     result = EqualityUtils.hashCodeOf(result, _flushThresholdTimeMillis);
     result = EqualityUtils.hashCodeOf(result, _flushThresholdSegmentSizeBytes);
@@ -383,8 +440,10 @@ public class StreamConfig {
     result = EqualityUtils.hashCodeOf(result, _decoderClass);
     result = EqualityUtils.hashCodeOf(result, _decoderProperties);
     result = EqualityUtils.hashCodeOf(result, _groupId);
+    result = EqualityUtils.hashCodeOf(result, _topicConsumptionRateLimit);
     result = EqualityUtils.hashCodeOf(result, _streamConfigMap);
     result = EqualityUtils.hashCodeOf(result, _tableNameWithType);
+    result = EqualityUtils.hashCodeOf(result, _serverUploadToDeepStore);
     return result;
   }
 }
