@@ -196,6 +196,11 @@ public class JsonUtils {
     return DEFAULT_READER.forType(valueType).readValue(jsonInputStream);
   }
 
+  public static <T> T inputStreamToObject(InputStream jsonInputStream, TypeReference<T> valueTypeRef)
+      throws IOException {
+    return DEFAULT_READER.forType(valueTypeRef).readValue(jsonInputStream);
+  }
+
   public static JsonNode inputStreamToJsonNode(InputStream jsonInputStream)
       throws IOException {
     return DEFAULT_READER.readTree(jsonInputStream);
@@ -351,7 +356,16 @@ public class JsonUtils {
    * </pre>
    */
   public static List<Map<String, String>> flatten(JsonNode node, JsonIndexConfig jsonIndexConfig) {
-    return flatten(node, jsonIndexConfig, 0, "$", false);
+    try {
+      return flatten(node, jsonIndexConfig, 0, "$", false);
+    } catch (OutOfMemoryError oom) {
+      throw new OutOfMemoryError(
+          String.format("Flattening JSON node: %s with config: %s requires too much memory, please adjust the config",
+              node, jsonIndexConfig));
+    } catch (Exception e) {
+      throw new IllegalArgumentException(
+          String.format("Caught exception while flattening JSON node: %s with config: %s", node, jsonIndexConfig), e);
+    }
   }
 
   private static List<Map<String, String>> flatten(JsonNode node, JsonIndexConfig jsonIndexConfig, int level,
