@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.spi.config.table;
 
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 import org.testng.annotations.Test;
@@ -33,7 +34,7 @@ public class UpsertConfigTest {
     assertEquals(upsertConfig1.getMode(), UpsertConfig.Mode.FULL);
 
     upsertConfig1.setComparisonColumn("comparison");
-    assertEquals(upsertConfig1.getComparisonColumn(), "comparison");
+    assertEquals(upsertConfig1.getComparisonColumns(), Collections.singletonList("comparison"));
 
     upsertConfig1.setHashFunction(HashFunction.MURMUR3);
     assertEquals(upsertConfig1.getHashFunction(), HashFunction.MURMUR3);
@@ -45,4 +46,11 @@ public class UpsertConfigTest {
     upsertConfig2.setDefaultPartialUpsertStrategy(UpsertConfig.Strategy.OVERWRITE);
     assertEquals(upsertConfig2.getPartialUpsertStrategies(), partialUpsertStratgies);
   }
+
+  @Test
+  public void testUpsertConfigForDefaults() {
+    UpsertConfig upsertConfig = new UpsertConfig(UpsertConfig.Mode.PARTIAL);
+    assertEquals(upsertConfig.getHashFunction(), HashFunction.NONE);
+    assertEquals(upsertConfig.getDefaultPartialUpsertStrategy(), UpsertConfig.Strategy.OVERWRITE);
+  }
 }
