@@ -74,11 +74,12 @@ public abstract class QuickStartBase {
       "examples/batch/githubEvents",
       "examples/batch/githubComplexTypeEvents",
       "examples/batch/billing",
+      "examples/batch/fineFoodReviews",
   };
 
   protected static final Map<String, String> DEFAULT_STREAM_TABLE_DIRECTORIES = ImmutableMap.<String, String>builder()
       .put("airlineStats", "examples/stream/airlineStats")
-      .put("githubEvents", "examples/minions/stream/githubEvents")
+      .put("githubEvents", "examples/stream/githubEvents")
       .put("meetupRsvp", "examples/stream/meetupRsvp")
       .put("meetupRsvpJson", "examples/stream/meetupRsvpJson")
       .put("meetupRsvpComplexType", "examples/stream/meetupRsvpComplexType")
