@@ -877,6 +877,10 @@ public class QueryRunner extends AbstractBaseCommand implements Command {
       }
     }
 
+    public double getPercentile(double p) {
+      return _statistics.getPercentile(p);
+    }
+
     public void report() {
       synchronized (_statistics) {
         LOGGER.info("--------------------------------------------------------------------------------");
@@ -944,6 +948,16 @@ public class QueryRunner extends AbstractBaseCommand implements Command {
       return _avgClientTime;
     }
 
+    public double getPercentile(double p) {
+      if (_statisticsList == null || _statisticsList.size() == 0) {
+        return 0.0;
+      }
+
+      // the last run's statistics is used;
+      return _statisticsList.get(_statisticsList.size() - 1).getPercentile(p);
+    }
+
+
     public List<Statistics> getStatisticsList() {
       return _statisticsList;
     }
